	class cATTDbg extends cATTParserBase
//	------------------------------------
{
	static	WKC_ESC	= 1;				// Interrupt character

//	Constants for breakpoints

	static	ATTDBG_BREAK_INITIAL		= 1;
	static	ATTDBG_BREAK_ESC		= 2;
	static	ATTDBG_BREAK_STEP		= 3;
	static	ATTDBG_BREAK_AT			= 4;
	static	ATTDBG_BREAK_IMMEDIATE		= 5;
	static	ATTDBG_BREAK_HALT		= 6;

//	Active module types

	static	ATTDBG_MT_PARAM			= 1;	// From command line
	static	ATTDBG_MT_SET			= 2;	// From SET MODULE
	static	ATTDBG_MT_SCOPE			= 3;	// From scope

//	Indexes to c_breaks.addr

	static	IB_MODULE		= 0;
	static	IB_MODIDX		= 1;
	static	IB_ADDR			= 2;		// Line #of func name

//	Indexes to c_watches.addr

	static	IW_MODULE		= 0;
	static	IW_FUNC			= 1;
	static	IW_LINE			= 2;

//	Display flags for Examine and evaluate

	static	ATT_DF_HEX		= 0x01;		// Display in hex
	static	ATT_DF_TILE		= 0x02;		// Disp as tile if int
	static	ATT_DF_SORT		= 0x04;		// Sort table
	static	ATT_DF_FILTER		= 0x08;		// Use c_filter on tables

//	Command execution error codes

	static	ATT_E_SUCCESS	= 1;

	static	ATT_E_BASE		= -100;		// Error code base

	static	ATT_E_FILEOPEN		= -101;
	static	ATT_E_NOTONSTACK	= -102;
	static	ATT_E_UNDEF		= -103;
	static	ATT_E_NOSRCPATH		= -104;
	static	ATT_E_NOMODULE		= -105;
	static	ATT_E_UNKNOWNMODULE	= -106;
	static	ATT_E_NOSOURCEFILE	= -107;
	static	ATT_E_NOSUCHLINE	= -108;
	static	ATT_E_INVLINE		= -109;
	static	ATT_E_IDNOTFOUND	= -110;
	static	ATT_E_EXPRESSION	= -111;
	static	ATT_E_NOFUNCTION	= -112;
	static	ATT_E_CANTCONTINUE	= -113;

	static	s_errstr = ["",
		"Error opening file",				// FILEOPEN
		"Requested scope is not on the stack",		// NOTONSTACK
		"Attempt to use undefined symbol",		// UNDEF
		"No source file path defined",			// NOSRCPATH
		"Module not found",				// NOMODULE
		"Unknown module",				// UNKNOWNMODULE
		"No source file found",				// NOSOURCEFILE
		"Line number does not exist",			// NOSUCHLINE
		"Invalid line number",				// INVLINE
		"Identifier not found",				// IDNOTFOUND
		"Error evaluating expression",			// EXPRESSION
		"Function not found",				// NOFUNCTION
		"You may not continue execution after an exception",//CANTCONTINUE
		];

//	Class variables

	c_init = null;					// Not initialized
	c_running = null;				// Stop for command
	c_halted = null;				// Halted on error
	c_input = null;					// Input line

	c_stepdef = null;				// STEP defaults
	c_breaks = null;				// Breakpoints
	c_watches = null;				// Watch variables
	c_funcs = null;					// Function names
	c_skipfunc = null;				// Do not debug this function

	c_fdlog = null;					// Log file fd
	c_logfile = null;				// Log file name

	c_srcpath = null;				// Source file path
	c_srcfiles = null;				// Source file names
	c_srclines = null;				// Source lines

	c_typedline = null;				// Last line typed
	c_typelines = null;				// Default type lines

	c_tabstop = null;				// Tabstops

	c_filter = null;				// Class/Table list filter
	
	c_module = null;				// SET module
	c_modidx = null;				// SET module index

	c_scopelvl = null;				// Scope level number
	c_scopefile = null;				// Scope source file
	c_scopemodule = null;				// Scope module name
	c_scopefunc = null;				// Scope function name
	c_scopeline = null;				// Line in scope

	c_dhtype = null;				// DebugHook call type
	c_dhfile = null;				// Source filespec
	c_dhline = null;				// Line number
	c_dhfunc = null;				// Function name
	c_dhmodule = null;				// File name only

	c_prompt = null;				// Prompt displayed
	c_autopause = null;				// Pause on breaks
	c_autorun = null;
	c_listfuncs = null;				// Do not list functions

	c_parser = null;				// Parser
	c_exect = null;					// Execution table
	c_cmdlvl = null;				// Command proc level

	constructor() {
	  c_init = false;

	  c_fdlog = cATTIO();				// For log file
	  c_logfile = null;
	}

	function Debug(dhtype,dhfile,dhline,dhfunc);
}
	function cATTDbg::Debug(dhtype,dhfile,dhline,dhfunc)
//	----------------------------------------------------
//	Main entry point for debugger
//	When we return from here, the game gets to run the GUI
//	The game will also run if it is not paused
//	dhtype		'l' | 'c' | 'r' for Line, Call, Return
//	dhfile		Source file name
//	dhline		Line number in file
//	dhfunc		Function name
//	Returns		1 to continue running main script
//			0 to keep calling the DebugHook function
{
	if(c_halted) {		// Main script encountered an execution error
	  if(GetInput()) ParseInput();

	  return 0;		// Do not let main script run any more
	}

	SetDHInfo(dhtype,dhfile,dhline,dhfunc);

	if(c_dhmodule == "cATTDebug") return 1;		// Don't stop in Enable

	if(!c_init) {					// First entry
	  Initialize();
	  return 1;
	}

	local	inpflg = GetInput();			// true if we have input

	if(!inpflg && !c_running) return 0;

	if(c_running) {
	  if(c_watches.len()) TestWatch();

	  if(c_running) TestBreak();			// See if need to break
	}

	if(inpflg) ParseInput();			// User input

	return (c_running) ? 1 : 0;
}
	function cATTDbg::HandleError(errstr)
//	-------------------------------------
//	Called when the user script encoutered an execution error
//	Debugger commands are available, but execution cannot continue
//	The debugger runs instead of the user script, so will not return
{
	OutputE("Execution exception: "+errstr);

	local	level=GetStackLevel();
	local	si = getstackinfos(level);

	SetDHInfo("l",si.src,si.line,si.func);

	Break(ATTDBG_BREAK_HALT);
}
	function cATTDbg::Initialize()
//	------------------------------
//	First call to debugger, initialize instance
{
	c_prompt = "DBG> ";
	c_autopause = true;
	c_autorun = false;
	c_listfuncs = false;

	c_init = true;
	c_running = true;
	c_halted = false;

	c_parser = cATTParser();
	InitParseTable();

	c_cmdlvl = 0;

	c_srcpath = null;				// No path
	c_srcfiles = [];				// Source file names
	c_srclines = [];				// Source lines

	c_typedline = [];				// Last typed line
	c_typelines = 1;				// Type 1 line

	c_tabstop = 8;

	c_watches = [];					// Watches
	c_breaks = [];					// Breakpoints
	c_stepdef = Breakpoint();			// STEP cmd defaults
	c_stepdef.qline = true;				// true for STEP

	FindFunctionNames();				// For breakpoints

	SetScopeZero();
	
	Output("");
	OutputW("ATTDebug version 1.0");
	Output("");

	ExecCmdProc("autoexec");

	SetImmediateBreak();				// Stop on next line

//	Break(ATTDBG_BREAK_INITIAL,true);
}
	function cATTDbg::SetDHInfo(dhtype,dhfile,dhline,dhfunc)
//	--------------------------------------------------------
//	Save info passed to DebugHook by VM
//	Parse and set module name from file name
{
	c_dhtype = dhtype;				// Curr instr info
	c_dhfile = dhfile;
	c_dhline = dhline;
	c_dhfunc = dhfunc;

	c_dhmodule = GetModule(dhfile);			// Find module name
}
	function cATTDbg::Output(str)
//	-----------------------------
//	Output a normal string
{
	AILog.Info("DBG: " + str);
	WriteLog(str);
}
	function cATTDbg::OutputW(str)
//	------------------------------
//	Output a warning string
{
	AILog.Warning("DBG: " + str);
	WriteLog(str);
}
	function cATTDbg::OutputE(str)
//	------------------------------
//	Output a severe error string
{
	AILog.Error("DBG: " + str);
	WriteLog(str);
}
	function cATTDbg::WriteLog(str)
//	-------------------------------
{
	if(c_fdlog.IsOpen()) c_fdlog.Write(str);
}
	function cATTDbg::OutputErrMsg(errcode)
//	---------------------------------------
//	Output debugger or parser error message
{
	OutputW(GetErrorStr(errcode));
}
	function cATTDbg::GetErrorStr(errcode)
//	--------------------------------------
{
	if(errcode <= ATT_E_BASE)
	  return s_errstr[-(errcode - ATT_E_BASE)];

	return c_parser.GetErrorStr(errcode)
}
	function cATTDbg::GetInput()
//	----------------------------
//	Get input line from debug window and echo it
//	Checks for WKC_ESC to see if need to interrupt main script
//	Returns		true if something was entered
{
	c_input = AIController.GetDebugInput();

	if(!c_input.len()) return false;		// No input

	if(c_input[0] == WKC_ESC) {			// Interrupt script
	  local str = c_prompt + "<ESC>";

	  AILog.Warning(str);
	  if(c_fdlog.IsOpen()) c_fdlog.Write(str);
	}
	else {
	  local str = c_prompt + c_input;

	  AILog.Warning(str);
	  if(c_fdlog.IsOpen()) c_fdlog.Write(str);
	}

	return true;
}
	function cATTDbg::PauseGame(override=false)
//	-------------------------------------------
//	Pause game but let AI scripts run
//	override	If true, c_autopause is ignored
{
	if(c_autopause || override) AIController.Pause();	// Pause game
}
	function cATTDbg::UnPauseGame()
//	-------------------------------
//	Let the game run
{
	AIController.UnPause();
}
	function cATTDbg::FindFunctionNames()
//	-------------------------------------
//	Search root table and classes for function names
{
	local	root = getroottable();

	c_funcs = {};

	foreach (name,v in root) {
	  switch (::type(v)) {
	  case "function":
	    c_funcs[name] <- null;
	    break;

	  case "class":
	    foreach (cname,cv in root[name]) {
	      if(::type(cv) == "function" && !(cname in c_funcs))
		c_funcs[cname] <- name;
	    }
	  }
	}
}
