	function cATTDbg::Breakpoint()
//	------------------------------
//	Returns a table with initial values for a breakpoint
{
	local	bp = {};

	bp.addr <- null;			// Module,modidx,line or function
	bp.cond <- null;			// WHEN condition
	bp.docmd <- null;			// DO command

	bp.qafter <- 1;				// Qualifier values
	bp.qmode <- null;
	bp.qline <- false;
	bp.qcall <- false;
	bp.qreturn <- false;
	bp.qinto <- false;			// true means /OVER
	bp.qsilent <- false;
	bp.qsource <- true;
	bp.qtemp <- false;
	bp.calldepth <- 0;

	return bp;
}
	function cATTDbg::TestBreak()
//	-----------------------------
//	If we are running the main script, see if we need to break now
//	Sets c_running to false if break
{
	local	bpfound = false;

	foreach (i,bp in c_breaks) {
	  local bpidx = null,docmd=null;

	  switch (bp.qmode) {
	  case ATTDBG_BREAK_STEP:
	  case ATTDBG_BREAK_AT:
	    if(((c_dhtype == 'l' && bp.qline) ||		// Line break
	        (c_dhtype == 'c' && bp.qcall) ||
	        (c_dhtype == 'r' && bp.qreturn)) &&
	         ((bp.qinto || bp.calldepth <= 0) && !--bp.qafter)) {
	      bp.qafter = 1;				// Check next time too

	      if(bp.cond && !TestCondition(bp.cond)) break;	// WHEN

	      if(!bpfound) Break(bp.qmode,bp.qsource,bp.qsilent);

	      bpidx = i;
	      bpfound = true;

	      if(bp.docmd) docmd = bp.docmd;		// Exec after temp del
	      break;
	    }

	    if(bp.qmode != ATTDBG_BREAK_AT || !bp.addr) break;

//	    Test for line or function break here

	    local addrtype = ::type(bp.addr[IB_ADDR]);

	    if(addrtype == "null") break;		// No addr specified

	    if(addrtype == "integer") {			// Line number
	      if(c_dhtype != 'l') break;		// Not line
	      if(bp.addr[IB_ADDR] != c_dhline) break;
	      if(bp.addr[IB_MODULE].toupper() != c_dhmodule.toupper()) break;
	    }
	    else {
	      if(c_dhtype != 'c') break;		// Not call
	      if(bp.addr[IB_ADDR] != c_dhfunc) break;
	    }

	    if(!--bp.qafter) {
	      bp.qafter = 1;				// Check next time too

	      if(bp.cond && !TestCondition(bp.cond)) break;	// WHEN

	      if(!bpfound) Break(bp.qmode,bp.qsource,bp.qsilent);

	      bpidx = i;
	      bpfound = true;

	      if(bp.docmd) docmd = bp.docmd;		// Exec after temp del
	    }

	    break;

	  case ATTDBG_BREAK_IMMEDIATE:
	    if(c_skipfunc && c_skipfunc == c_dhfunc)
	      break;					// Skip this function

	    if(!bpfound) Break(ATTDBG_BREAK_AT);

	    bpidx = i;
	    bpfound = true;
	    break;
	  }

	  if(bpidx != null && c_breaks[bpidx].qtemp)	// Temp breakpoint
	    c_breaks.remove(bpidx);

	  else if(c_dhtype != 'l') {
	    if(c_dhtype == 'c')				// Function call
	      bp.calldepth++;
	    else 					// Return statement
	      bp.calldepth--;
	  }

	  if(docmd) ParseCommandLine(docmd);		// DO
	}

	if(bpfound) CancelStepBreak();
}
	function cATTDbg::Break(breaktype,source=true,silent=false)
//	-----------------------------------------------------------
//	Break at current instruction
//	breaktype	ATTDBG_BREAK_ value
//	source		Display source line if true
{
	if(!silent) {
	  local	buf = ::format("%s\\%s line %d",c_dhmodule,c_dhfunc,c_dhline);

	  switch(breaktype) {
	  case ATTDBG_BREAK_AT:
	  case ATTDBG_BREAK_INITIAL:
	    Output("break at " + buf);
	    break;

	  case ATTDBG_BREAK_ESC:
	    Output("interrupt at " + buf);
	    break;

	  case ATTDBG_BREAK_STEP:
	    Output("stepped to " + buf);
	    break;

	  case ATTDBG_BREAK_HALT:
	    Output("halted at " + buf);
	    c_halted = true;
	    break;
	  }

	  if(source) {					// Display source line
	    local idx = GetFileIndex(c_dhfile);

	    if(idx >= 0) {
	      if(c_dhline >= c_srclines[idx].len())	// Fix bug in squirrel
		c_dhline = c_srclines[idx].len() - 1;

	      local str = ExpandTabs(c_srclines[idx][c_dhline],c_tabstop);
	      Output(format("%4d: %s",c_dhline,str));
	      c_typedline[idx] = c_dhline;		// Last displayed
	    }
	    else
	      OutputW("Unable to open source file: " + c_dhfile);
	  }
	}

	SetScopeZero();
	PauseGame(breaktype == ATTDBG_BREAK_HALT);

	c_running = false;			// Don't run Main script
}
	function cATTDbg::SetScopeZero()
//	--------------------------------
//	Set scope and module to that of current line
{
	c_scopelvl = 0;
	c_scopefile = c_dhfile;
	c_scopemodule = c_dhmodule;
	c_scopefunc = c_dhfunc;
	c_scopeline = c_dhline;
	c_module = null;			// Use scope
}
	function cATTDbg::SetImmediateBreak()
//	-------------------------------------
//	Set a temp breakpoint to break on next line
{
	SetTempBreak(ATTDBG_BREAK_IMMEDIATE);
}
	function cATTDbg::SetTempBreak(mode)
//	------------------------------------
//	Set a temp breakpoint
//	mode		ATTDBG_BREAK_ code
{
	local	bp = clone c_stepdef;

	bp.qmode = mode;
	bp.qline = true;
	bp.qsilent = false;
	bp.qtemp = true;

	c_breaks.insert(0,bp);
}
	function cATTDbg::CancelStepBreak()
//	-----------------------------------
//	Search c_breaks table and delete temp step breakpoint
//	Called when debugger is disabled
{
	local	bpidx = null;

	foreach (i,bp in c_breaks) {
	  if(bp.qmode == ATTDBG_BREAK_STEP) {
	    bpidx = i;
	    break;
	  }
	}

	if(bpidx != null) c_breaks.remove(bpidx);
}
	function cATTDbg::Watchpoint()
//	------------------------------
//	Returns a table with initial values for a watchpoint
{
	local	wp = {};

	wp.addr <- null;			// Set at scope module,func,line
	wp.expr <- null;			// Expression to watch
	wp.id <- null;				// id part of expression
	wp.loc <- null;				// id location (local or this addr)

	wp.value <- null;			// Previous value

	wp.cond <- null;			// WHEN condition
	wp.docmd <- null;			// DO command

	wp.qafter <- 1;				// Qualifier values
	wp.qinto <- false;			// true means /OVER
	wp.qsilent <- false;
	wp.qsource <- true;
	wp.qtemp <- false;
	wp.calldepth <- 0;

	return wp;
}
	function cATTDbg::TestWatch()
//	-----------------------------
//	Evaluate watch variables
{
	local	level = GetStackLevel();
	local	si = getstackinfos(level + c_scopelvl);
	local	docmd,wparr=[],wpfound = false;

	foreach (i,wp in c_watches) {
	  local ret=null;

	  if(wp.loc == "local") {
	    if(wp.addr[IW_FUNC] == c_dhfunc && wp.id in si.locals)
	      ret = EvaluateExpr(si.locals,wp.expr);
	  }
	  else if("this" in si.locals && wp.loc == si.locals["this"].tostring())
	    ret = EvaluateExpr(si.locals,wp.expr);

	  if(ret && ret.errcode == ATT_E_SUCCESS &&	// Evaluated
	     wp.value != ret.value) {			// Value changed

	    if((wp.qinto || wp.calldepth <= 0) && !--wp.qafter) {
	      wp.qafter = 1;				// Reset

	      if(!wp.cond || TestCondition(wp.cond)) {
	        if(!wp.qsilent) {
	          local	var;

	          if(wp.loc == "local")
	            var = wp.addr[IW_FUNC] + "\\" + wp.expr;
	          else
	            var = wp.expr;

	          Output("watch of " + var);
	          Output("  old value: " + GetValue(wp.value));
	          Output("  new value: " + GetValue(ret.value));
	        }

	        if(!wpfound) {
	          Break(ATTDBG_BREAK_AT,wp.qsource,wp.qsilent);
	          wpfound = true;
	        }

	        if(wp.docmd) docmd = wp.docmd;		// Exec after temp del
	        if(wp.qtemp) wparr.push(i);		// Delete later
	      }
	    }

	    wp.value = ret.value;
	  }
	}

	if(wpfound) {
	  foreach (i,v in wparr)			// Delete temp watches
	    c_watches.remove(v);
	}

	if(docmd) ParseCommandLine(docmd);		// DO
}
	function cATTDbg::TestCondition(cond)
//	-------------------------------------
//	Evaluate condition statement and return true or false
{
	local	level = GetStackLevel();
	local	si = getstackinfos(level);
	local	ret = EvaluateExpr(si.locals,cond);

	return (ret.errcode == ATT_E_SUCCESS && ret.value);
}
