	function cATTDbg::GetActiveModule(module)
//	-----------------------------------------
//	Return the index to the source file for the module (>=0)
//	If the module is null, try the SET module or the scope module
//	Returns		a table with the followign fields
//			errcode		ATT_E_ code
//			module		Module name
//			modidx		Modula source file index
//			modtype		ATTDBG_MT_ code
{
	local	modidx,ret={};

	if(module != null) {
	  ret.modidx <- GetModuleIndex(module);

	  if(ret.modidx < 0) return {errcode = ATT_E_UNKNOWNMODULE};

	  ret.module <- module;
	  ret.modtype <- ATTDBG_MT_PARAM;		// From parameter
	}
	else if(c_module != null) {			// Module SET
	  ret.modidx <- c_modidx;
	  ret.module <- c_module;
	  ret.modtype <- ATTDBG_MT_SET	;		// From SET MODULE
	}
	else {
	  ret.modidx <- GetFileIndex(c_scopefile);

	  if(ret.modidx < 0) return {errcode = ATT_E_NOSOURCEFILE};

	  ret.module <- GetModule(c_scopefile);
	  ret.modtype <- ATTDBG_MT_SCOPE;		// From scope
	}

	ret.errcode <- ATT_E_SUCCESS;
	return ret;
}
	function cATTDbg::HasExecParam(id)
//	----------------------------------
//	Check if c_exect.qualt has the specified id
{
	return ("qualt" in c_exect && id in c_exect.qualt);
}
	function cATTDbg::GetExecParamValueCount(id)
//	--------------------------------------------
//	Return number of values in parameter value array
{
	if(!HasExecParam(id)) return null;

	return c_exect.qualt[id].len();
}
	function cATTDbg::GetExecParamValue(id,idx=0,defval=null)
//	---------------------------------------------------------
//	Return parameter value at idx from execution table
//	id	Parser parameter name
//	idx	Parameter value array index
//	defval	Default value to return if no value exists
{
	if("qualt" in c_exect && id in c_exect.qualt) return c_exect.qualt[id][idx];

	return defval;
}
	function cATTDbg::GetExecParamString(id)
//	----------------------------------------
//	Return parameter string value (not in an array)
{
	if(!HasExecParam(id)) return null;

	return c_exect.qualt[id];
}
	function cATTDbg::Cmd_AT()
//	--------------------------
//	@file
//	Execute command procedure (.adc)
{
	return ExecCmdProc(GetExecParamValue("_P1_"));
}
	function cATTDbg::CmdCancelAll()
//	--------------------------------
//	CANCEL ALL - break and watch
{
	c_breaks = [];
	c_watches = [];

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdCancelBreak()
//	----------------------------------
//	CANCEL BREAK /ALL /CALL /LINE /RETURN [addr[,...]]
{
	if(c_breaks.len() == 0) return ATT_E_SUCCESS;	// Nothing set

	if(HasExecParam("ALL")) {			// /ALL
	  c_breaks = [];
	  return ATT_E_SUCCESS;
	}

	local	qcall = HasExecParam("CALL");
	local	qline = HasExecParam("LINE");
	local	qreturn = HasExecParam("RETURN");

	if(qcall || qline || qreturn) {
	  if(c_breaks[0].addr == null) {
	    local bp = c_breaks[0];

	    if(qcall) bp.qcall = false;
	    if(qline) bp.qline = false;
	    if(qreturn) bp.qreturn = false;

	    if(!(bp.qcall || bp.qline || bp.qreturn)) c_breaks.remove(0);
	  }

	  return ATT_E_SUCCESS;
	}

	local values = GetExecParamValueCount("_P1_");

	if(values == null) return ATT_PE_PARMREQ;	// No address

	for (local idx=0; idx < values; idx+=2) {	// Each breakpoint
	  local module = GetExecParamValue("_P1_",idx);
	  local modt = GetActiveModule(module);
	  local v2 = GetExecParamValue("_P1_",idx+1);
	  local found = false;

	  if(::type(v2) == "string" && modt.modtype == ATTDBG_MT_SCOPE)
	    modt.module = null;				// Any module for func

	  foreach (i,bp in c_breaks) {
	    if(bp.addr && bp.addr[IB_ADDR] == v2) {	// Same value
	      local bpmodule = bp.addr[IB_MODULE];

	      if(bpmodule) bpmodule = bpmodule.toupper();
	      if(modt.module) modt.module = modt.module.toupper();

	      if(bpmodule == modt.module) {
		c_breaks.remove(i);
		found = true;
		break;
	      }
	    }
	  }

	  if(!found) {
	    local str = "Breakpoint not found: ";

	    if(module) str += module + "\\";
	    Output(str + v2);
	  }
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdCancelModule()
//	-----------------------------------
//	CANCEL MODULE
{
	c_module = null;				// Use scope

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdCancelWatch()
//	----------------------------------
//	CANCEL WATCH /ALL [symbol[,...]]
{
	if(c_watches.len() == 0) return ATT_E_SUCCESS;	// Nothing set

	if(HasExecParam("ALL")) {			// /ALL
	  c_watches = [];
	  return ATT_E_SUCCESS;
	}

	local values = GetExecParamValueCount("_P1_");
	local arr=[];

	for (local idx=0; idx < values; idx+=2) {	// Each watchpoint
	  local func = GetExecParamValue("_P1_",idx);
	  local v2 = GetExecParamValue("_P1_",idx+1);
	  local found = -1, dup = false;

	  foreach (j,wp in c_watches) {
	    if(wp.expr == v2) {
	      if(wp.loc == "local") {
		if((func && func == wp.addr[IW_FUNC]) ||
		   (!func && wp.addr[IW_MODULE] == c_scopemodule &&
		    	     wp.addr[IW_FUNC] == c_scopefunc)) {
		  found = j;
		  break;
	        }

		if(!func) {				// See if unique
		  dup = false;

		  for (i=j+1; i < c_watches.len(); i++) {
		    if(c_watches[i].expr == v2) {
		      dup = true;
		      break;
		    }
		  }

		  if(dup)
		    Output("Ambiguous expression: " + v2);
		  else {
		    found = j;
		    break;
	          }
		}
	      }
	      else if(!func) {				// class variable
		found = j;
		break;
	      }
	    }
	  } // foreach

	  if(found >= 0)
	    c_watches.remove(found);
	  else {
	    local str = "Watch not found: ";

	    if(func) str += func + "\\";
	    Output(str + v2);
	  }
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdDefine()
//	-----------------------------
//	DEFINE symbol = parameter
{
	c_parser.DefineSymbol(GetExecParamValue("_P1_"),
			      GetExecParamValue("_P3_"));

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdDelete()
//	-----------------------------
//	DELETE /ALL [symbol_name[,...]]
//	Delete defined symbols.
{
	if(HasExecParam("ALL"))				// /ALL
	  c_parser.DeleteSymbol("$ALL");
	else {
	  local	t = c_parser.GetSymbolTable();
	  local values = GetExecParamValueCount("_P1_");

	  for (local idx=0; idx < values; idx++) {	// Each value
	    local p1 = GetExecParamValue("_P1_",idx);

	    if(!(p1 in t)) return ATT_E_UNDEF;
	  }

	  for (local idx=0; idx < values; idx++) {	// Each value
	    local p1 = GetExecParamValue("_P1_",idx);

	    c_parser.DeleteSymbol(p1);
	  }
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdDeposit()
//	------------------------------
//	DEPOSIT variable = expression
//	Local variables that are passed by value cannot be changed, because
//	si.locals is a copy of the real local symbol table.
//	Variables that are passed by reference can be changed and any variable
//	in the "this" table can be changed as "this" is passed by reference.
{
	local	p1 = GetExecParamValue("_P1_");
	local	p2 = GetExecParamValue("_P2_");
	local	p3 = GetExecParamValue("_P3_");

	local	level=GetStackLevel(),si;
	
	if(si = getstackinfos(level + c_scopelvl)) {
	  local ret = EvaluateExpr(si.locals,p1+p2+p3);

	  if(!ret.errcode) OutputW("Error evaluating expression: " + ret.e);
	}
	else
	  OutputW("Scope is not in stack: " + c_scopelvl);

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdEvaluate()
//	-------------------------------
//	EVALUATE /FILTER /HEX /LIST /SORT /TILE expression
//	Does not list values in containers unless /LIST specified
{
	local	p1 = GetExecParamValue("_P1_");
	local	dspflg = 0,list = false;

	if(HasExecParam("FILTER")) dspflg = dspflg | ATT_DF_FILTER;
	if(HasExecParam("HEX")) dspflg = dspflg | ATT_DF_HEX;
	if(HasExecParam("LIST")) list = true;
	if(HasExecParam("SORT")) {
	  list = true;
	  dspflg = dspflg | ATT_DF_SORT;
	}
	if(HasExecParam("TILE")) dspflg = dspflg | ATT_DF_TILE;

	Evaluate(p1,list,dspflg);

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdExamine()
//	------------------------------
//	EXAMINE /FILTER /HEX /LOCALS /SORT /TILE [var[,var]]
//	Evaluate expression, list values in containers
{
	local	values = GetExecParamValueCount("_P1_");
	local	locals = HasExecParam("LOCALS");
	local	dspflg = 0;

	if(HasExecParam("FILTER")) dspflg = dspflg | ATT_DF_FILTER;
	if(HasExecParam("HEX")) dspflg = dspflg | ATT_DF_HEX;
	if(HasExecParam("SORT")) dspflg = dspflg | ATT_DF_SORT;
	if(HasExecParam("TILE")) dspflg = dspflg | ATT_DF_TILE;

	if(locals) {
	  if(values) return ATT_PE_TOOMANY;

	  local	level=GetStackLevel(),si;
	
	  if(si = getstackinfos(level + c_scopelvl))
	    Examine(si.func+"\\locals",si.locals,true,dspflg);
	  else
	    OutputW("Scope is not in stack: " + c_scopelvl);
	}
	else {
	  if(!values) return ATT_PE_INVPARM;

	  for (local idx=0; idx < values; idx++) {	// Each value
	    local p1 = GetExecParamValue("_P1_",idx);

	    Evaluate(p1,true,dspflg);
	  }
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::Evaluate(expr,list,dspflg=0)
//	----------------------------------------------
//	Evaluate an exression and print the result
//	expr		Expression string
//	list		true to list values in containers
//	dspflg		ATT_DF_ display flag bits
{
	local	level=GetStackLevel(),si;
	
	if(si = getstackinfos(level + c_scopelvl)) {
	  local ret = EvaluateExpr(si.locals,expr);

	  if(ret.errcode == ATT_E_SUCCESS)
	    Examine(si.func+"\\"+expr,ret.value,list,dspflg);
	  else
	    OutputW("Error evaluating expression: " + ret.e);
	}
	else
	  OutputW("Scope is not in stack: " + c_scopelvl);
}
	function cATTDbg::EvaluateExpr(locals,expr)
//	-------------------------------------------
//	Evaluate an expression
//	locals		Local variables to use
//	expr		Expression string
//	Returns		A table with the following fields
//			errcode		0 for failure
//			e		Caught error message
//			value		Evaluated expression value
{
	local	buf = "return function (";
	local	args = [], count = 0,ret={};

	args.push(locals["this"]);

	foreach (name,v in locals) {
	  if(name != "this" && name[0] != '@') {
	    if(count) buf += ",";

	    count++;
	    args.push(v);
	    buf += name;
	  }
	}

	buf += ") {return (" + expr + ");}";

	notifyallexceptions(false);

	try {
	  local	func = ::compilestring(buf,"Evaluate_buf");

	  ret.value <- func().pacall(args);
	  ret.errcode <- ATT_E_SUCCESS;
	}
	catch(e) {
	  ret.errcode <- 0;
	  ret.e <- e;
	}

	notifyallexceptions(true);

	return ret;
}
	function cATTDbg::Examine(objname,obj,list=true,dspflg=0)
//	---------------------------------------------------------
//	Examine a variable
//	objname	Object name as a string
//	obj	Object
//	list	true to list values in containers
//	dspflg	ATT_DF_ display flag bits
{
	local	otype = ::type(obj),v;

	if(otype == "weakref") {
	  Examine("*"+objname,obj.ref(),dspflg);
	  return;
	}

	switch(otype) {
	case "table":
	case "array":
	  if(!obj.len()) {
	    Output(otype + " " + objname + " is empty");
	    return;
	  }

	  Output(otype + " " + objname);

	  if(otype == "array") {
	    foreach (i,val in obj)			// List array
	      Output(" [" + i + "]: " + GetValue(val,dspflg));

	    return;
	  }

	  break;

	case "class":
	case "instance":
	  Output(otype + " " + objname);
	  break;

	default:
	  Output(objname + ": " + GetValue(obj,dspflg));
	  return;
	}

	if(list) pListContainer(obj,dspflg);
}
	function cATTDbg::CmdGo()
//	-------------------------
//	GO /PAUSED /RUN
{
	if(c_halted) return ATT_E_CANTCONTINUE;

	local	run = c_autorun;

	if(HasExecParam("PAUSED")) run = false;
	if(HasExecParam("RUN")) run = true;

	if(run) UnPauseGame();
	c_running = true;

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetAutopause()
//	-----------------------------------
//	SET AUTOPAUSE
{
	c_autopause = true;
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetBreak()
//	-------------------------------
//	SET BREAK /AFTER:n /CALL /INTO /LINE /OVER /RETURN /[NO]SILENT /[NO]SOURCE
//		  /TEMPORARY [addr[,...]] [WHEN (condition)] [DO (command;...)]
{
	local	n,s,bparr=[],bp = Breakpoint();

	if(HasExecParam("RETURN")) bp.qreturn = true;
	if(HasExecParam("CALL"))   bp.qcall = true;
	if(HasExecParam("LINE"))   bp.qline = true;

	if(HasExecParam("INTO")) {
	  if(!(bp.qcall || bp.qline || bp.qreturn)) return ATT_PE_INVUSE;
	  bp.qinto = true;
	}

	if(HasExecParam("OVER")) {
	  if(!(bp.qcall || bp.qline || bp.qreturn)) return ATT_PE_INVUSE;
	  bp.qinto = false;
	}

	if(HasExecParam("SILENT"))   bp.qsilent = true;
	if(HasExecParam("NOSILENT")) bp.qsilent = false;

	if(HasExecParam("SOURCE"))   bp.qsource = true;
	if(HasExecParam("NOSOURCE")) bp.qsource = false;

	if(HasExecParam("TEMPORARY")) bp.qtemp = true;

	if((n = GetExecParamValue("AFTER"))) bp.qafter = n;

	if((s = GetExecParamString("_WHEN_"))) bp.cond = s;
	if((s = GetExecParamString("_DO_"))) bp.docmd = s;

	bp.qmode = ATTDBG_BREAK_AT;

	local values = GetExecParamValueCount("_P1_");

	if(values == null) {				// No address
	  if(!bp.qcall && !bp.qreturn && !bp.qline) return ATT_PE_PARMREQ;

	  if(c_breaks.len() > 0 &&
	     (c_breaks[0].qline || c_breaks[0].qcall || c_breaks[0].qreturn))
	    c_breaks[0] = bp;				// Replace breakpoint
	  else
	    c_breaks.insert(0,bp);			// Insert breakpoint
	}
	else {						// Verify values
	  for (local idx=0; idx < values; idx+=2) {	// Each breakpoint
	    local module = GetExecParamValue("_P1_",idx);
	    local modt = GetActiveModule(module);

	    if(modt.errcode != ATT_E_SUCCESS) return modt.errcode;

	    local v2 = GetExecParamValue("_P1_",idx+1);

	    if(::type(v2) == "integer") {		// Line number
	      local	maxline = c_srclines[modt.modidx].len() - 1;

	      if(v2 > maxline) return ATT_E_NOSUCHLINE;
	    }
	    else {					// Function name
	      if(!(v2 in c_funcs)) return ATT_E_NOFUNCTION;

	      if(modt.modtype == ATTDBG_MT_SCOPE)
	        modt.module = null;			// Any module for func
	    }

	    local bp1 = clone bp;

	    bp1.addr = [];
	    bp1.addr.push(modt.module);			// Module name
	    bp1.addr.push(modt.modidx);			// Module index
	    bp1.addr.push(v2);				// Line or function

	    bparr.push(bp1);
	  }

	  foreach (i,val in bparr) {
	    local replaced = false;

	    foreach (j,bp in c_breaks) {		// Check to replace
	      if(bp.addr && bp.addr[IB_ADDR] == val.addr[IB_ADDR]) {
		local bpmodule = bp.addr[IB_MODULE];
		local valmodule = val.addr[IB_MODULE];

		if(bpmodule) bpmodule = bpmodule.toupper();
		if(valmodule) valmodule = valmodule.toupper();

		if(bpmodule == valmodule) {
	          c_breaks[j] = val;
		  replaced = true;
		  break;
		}
	      }
	    }

	    if(!replaced) c_breaks.push(val);		// Add to breakpoints
	  }
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetFilter()
//	--------------------------------
//	SET FILTER symbol_name[,...]
{
	local	values = GetExecParamValueCount("_P1_");

	c_filter = [];

	for (local idx=0; idx < values; idx++) 		// Each value
	  c_filter.push(GetExecParamValue("_P1_",idx));

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetFunctions()
//	-----------------------------------
//	SET FUNCTIONS
{
	c_listfuncs = true;
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetLines()
//	-------------------------------
//	SET LINES n
//	Sets the number of lines to type if no range is given
{
	c_typelines = GetExecParamValue("_P1_");

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetLog()
//	-----------------------------
//	SET LOG [filename]
//	Uses current date if filename is omitted
{
	if(c_fdlog.IsOpen()) {
	  c_fdlog.Close();
	  c_logfile = null;
	}

	local	p1 = GetExecParamValue("_P1_");

	if(!p1) {
	  local	t = date();

	  p1 = format("ATTDebug %d-%02d-%02d %02d.%02d.%02d.log",
		      t.year,t.month,t.day,t.hour,t.min,t.sec);
	}

	if(!c_fdlog.Open(p1,"w")) return ATT_E_FILEOPEN;

	c_logfile = p1;
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetModule()
//	--------------------------------
//	SET MODULE module_name
{
	local	p1 = GetExecParamValue("_P1_");
	local	idx;

	if((idx = GetModuleIndex(p1)) >= 0) {		// File known
	  c_module = p1;
	  c_modidx = idx;
	}
	else {						// See if on stack
	  local	match=false,scope=0,si,level = GetStackLevel();
	  local	modarr = ::split(p1,"\\");

	  modarr.reverse();

	  while (si = getstackinfos(level + scope)) {
	    if(MatchModule(si.src,modarr)) {
	      c_module = p1;
	      c_modidx = GetFileIndex(si.src);
	      match = true;
	      break;
	    }

	    scope++;
	  }

	  if(!match) {					// Try using path
	    if(c_srcpath == null) return ATT_E_NOSRCPATH;

	    local filename = c_srcpath + "\\" + p1 + ".nut";
	    local fd = cATTIO();

	    if(!fd.Open(filename)) return ATT_E_NOMODULE;

	    c_modidx = GetFileIndex(filename);
	    c_module = p1;
	  }
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetNoautopause()
//	-----------------------------------
//	SET NOAUTOPAUSE
{
	c_autopause = false;
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetNofunctions()
//	-------------------------------------
//	SET NOFUNCTIONS
//	Do not display function names from tables
{
	c_listfuncs = false;
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetNolog()
//	-------------------------------
//	SET NOLOG
{
	if(c_fdlog.IsOpen()) {
	  c_fdlog.Close();
	  c_logfile = null;
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetScope()
//	-------------------------------
//	SET SCOPE {n | function_name}
{
	local	scope,si,level = GetStackLevel();
	local	p1 = GetExecParamValue("_P1_");

	if(p1[0] <= '9') {				// Number
	  scope = p1.tointeger();

	  if(!(si = getstackinfos(level + scope))) return ATT_E_NOTONSTACK;
	}
	else {						// Function name
	  scope = -1;

	  do {
	    scope++;

	    if(!(si = getstackinfos(level + scope))) return ATT_E_NOTONSTACK;

	  } while (si.func != p1);
	}

	c_scopelvl = scope;
	c_scopefile = si.src;
	c_scopemodule = GetModule(si.src);
	c_scopefunc = si.func;
	c_scopeline = si.line;
	c_module = null;				// Use scope

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetSource()
//	--------------------------------
//	SET SOURCE path
//	Sets path to use to find modules not yet seen and not on stack
{
	c_srcpath = GetExecParamValue("_P1_");

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetStep()
//	------------------------------
//	SET STEP /CALL /INTO /LINE /OVER /RETURN /[NO]SILENT /[NO]SOURCE
{
	if(HasExecParam("RETURN")) c_stepdef.qreturn = true;
	if(HasExecParam("CALL"))   c_stepdef.qcall = true;
	if(HasExecParam("LINE"))   c_stepdef.qline = true;

	if(HasExecParam("INTO"))   c_stepdef.qinto = true;
	if(HasExecParam("OVER"))   c_stepdef.qinto = false;

	if(HasExecParam("SILENT"))   c_stepdef.qsilent = true;
	if(HasExecParam("NOSILENT")) c_stepdef.qsilent = false;

	if(HasExecParam("SOURCE"))   c_stepdef.qsource = true;
	if(HasExecParam("NOSOURCE")) c_stepdef.qsource = false;

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetTabstop()
//	---------------------------------
//	SET TABSTOP n
{
	c_tabstop = GetExecParamValue("_P1_");

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdSetWatch()
//	-------------------------------
//	SET WATCH /AFTER:n /INTO /OVER /[NO]SILENT /[NO]SOURCE
//		  [watch_expr[,...]] [WHEN (condition)] [DO (command;...)]
{
	local	n,s,wparr=[],wp = Watchpoint();
	local	hasinto=false,hasover=false;

	if(HasExecParam("INTO")) {wp.qinto = true;  hasinto = true;}
	if(HasExecParam("OVER")) {wp.qinto = false; hasover = true;}

	if(HasExecParam("SILENT"))   wp.qsilent = true;
	if(HasExecParam("NOSILENT")) wp.qsilent = false;

	if(HasExecParam("SOURCE"))   wp.qsource = true;
	if(HasExecParam("NOSOURCE")) wp.qsource = false;

	if(HasExecParam("TEMPORARY")) wp.qtemp = true;

	if((n = GetExecParamValue("AFTER"))) wp.qafter = n;

	if((s = GetExecParamString("_WHEN_"))) wp.cond = s;
	if((s = GetExecParamString("_DO_"))) wp.docmd = s;

	local values = GetExecParamValueCount("_P1_");

	for (local idx=0; idx < values; idx+=2) {	// Each watchpoint
	  local func = GetExecParamValue("_P1_",idx);
	  local v2 = GetExecParamValue("_P1_",idx+1);
	  local idt = GetIdentifier(v2);

	  if(!idt.loc) return ATT_E_IDNOTFOUND;
	  if(!("value" in idt)) return ATT_E_EXPRESSION;

	  local wp1 = clone wp;

	  wp1.addr = [c_scopemodule,c_scopefunc,c_scopeline];
	  wp1.expr = v2;
	  wp1.id = idt.id;
	  wp1.loc = idt.loc;
	  wp1.value = idt.value;

	  if(!hasinto && !hasover)			// No qualifiers
	    wp1.qinto = (idt.loc == "local") ? false : true;

	  wparr.push(wp1);
	}

	foreach (i,val in wparr) {
	  local replaced = false;

	  foreach (j,wp in c_watches) {			// Check to replace
	    if(wp.expr == val.expr && wp.loc == val.loc &&
	       (wp.loc != "local" || wp.addr[IW_FUNC] == val.addr[IW_FUNC])) {
	      c_watches[j] = val;
	      replaced = true;
	      break;
	    }
	  }

	  if(!replaced) c_watches.push(val);		// Add to watches
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowAutopause()
//	------------------------------------
//	SHOW AUTOPAUSE
{
	Output("autopause: " + (c_autopause ? "true" : "false"));
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowBreak()
//	--------------------------------
//	SHOW BREAK
{
	if(!c_breaks.len()) {
	  Output("No breakpoints set");
	  return ATT_E_SUCCESS;
	}

	foreach (i,bp in c_breaks) {
	  if(bp.addr != null) {
	    if(bp.addr[IB_MODULE])
	      Output(::format("breakpoint at %s\\%s",
			bp.addr[IB_MODULE],bp.addr[IB_ADDR].tostring()));
	    else
	      Output("breakpoint at " + bp.addr[IB_ADDR]);
	  }
	  else
	    Output("breakpoint on");

	  local	str="";

	  if(bp.qafter > 1) str += "  after: " + bp.qafter;

	  if(bp.addr == null) {
	    if(bp.qcall) str += "  call";
	    if(bp.qline) str += "  line";
	    if(bp.qreturn) str += "  return";
	    str += (bp.qinto) ? "  into" : "  over";
	  }

	  if(bp.qsilent) str += "  silent";
	  if(!bp.qsource) str += "  nosource";
	  if(bp.qtemp) str+= "  temporary";

	  if(str != "") Output(str);

	  if(bp.cond) Output("  when (" + bp.cond + ")");
	  if(bp.docmd) Output("  do (" + bp.docmd + ")");
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowCalls()
//	--------------------------------
//	SHOW CALLS [n]
{
	local	level=GetStackLevel(),i=0,n;
	local	si,module,str,func;

	n = GetExecParamValue("_P1_",0,0);

	Output("scope module\\function");

	while (si = getstackinfos(level)) {
	  func = si.func;

	  if(si.src == "NATIVE") {
	    str = ::format("%2d  NATIVE\\%s",i,func);
	  }
	  else {
	    module = GetModule(si.src);
	    str = ::format("%4d  %s line %d",i,module+"\\"+func,si.line);
	  }

	  Output(str);

	  if(n && !--n) break;				// List limit reached

	  i++;						// Stack index
	  level++;
	}

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowFilter()
//	---------------------------------
//	SHOW FILTER
{
	local	str = "";

	if(c_filter) {
	  foreach (i,val in c_filter) {
	    if(i) str += ",";

	    str += val;
	  }
	}

	Output("filter: " + ((str != "") ? str : "not set"));
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowFunctions()
//	------------------------------------
//	SHOW FUNCTIONS
{
	Output("functions: " + (c_listfuncs ? "true" : "false"));
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowLines()
//	--------------------------------
//	SHOW LINES
{
	Output("lines: " + c_typelines);
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowLog()
//	------------------------------
//	SHOW LOG
{
	Output("log: " + (c_logfile ? c_logfile : "not set"));
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowModule()
//	---------------------------------
//	SHOW MODULE
{
	Output("module: " + (c_module ? c_module : "not set"));
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowScope()
//	--------------------------------
//	SHOW SCOPE
{
	Output(format("scope %d: %s\\%s",c_scopelvl,c_scopemodule,c_scopefunc));
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowSource()
//	---------------------------------
//	SHOW SOURCE
{
	Output("source: " + (c_srcpath ? c_srcpath : "not set"));
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowStep()
//	-------------------------------
//	SHOW STEP
//	Show defaults for STEP command
{
	local	brk = "";

	if(c_stepdef.qcall) brk += "call, ";
	if(c_stepdef.qline) brk += "line, ";
	if(c_stepdef.qreturn) brk += "return, ";

	Output("step: " + brk +
		((c_stepdef.qinto) ? "into, " : "over, ") +
		((c_stepdef.qsilent) ? "silent, " : "nosilent, ") +
		((c_stepdef.qsource) ? "source" : "nosource"));

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowSymbol()
//	---------------------------------
//	SHOW SYMBOL [symbol_name[,...]]
//	symbol_name may have * and % wildcard characters
{
	local	item=[],t=c_parser.GetSymbolTable();

	foreach (i,val in t)
	  item.push(i);

	item.sort();

	local	match=0,values = GetExecParamValueCount("_P1_");

	if(!values) {					// List all
	  foreach (i,val in item) {			// List sorted table
	    Output(val + " = " + t[val]);
	    match++;
	  }
	}
	else {
	  for (local idx=0; idx < values; idx++) {	// Each value
	    local p1 = GetExecParamValue("_P1_",idx);
	    local ex = ConvertWildcards(p1);

	    foreach (i,val in item) {			// List sorted table
	      if(ex.match(val)) {
//	      if(MatchPattern(val,p1)) {
	        Output(val + " = " + t[val]);
	        match++;
	      }
	    }
	  }
	}

	if(!match) Output("No symbols matched");

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowTabstop()
//	----------------------------------
//	SHOW TABSTOP
{
	Output("tabstop: " + c_tabstop);
	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdShowWatch()
//	--------------------------------
//	SHOW WATCH
{
	if(!c_watches.len()) {
	  Output("No watchpoints set");
	  return ATT_E_SUCCESS;
	}

	foreach (i,wp in c_watches) {
	  local	var;

	  if(wp.loc == "local")
	    var = wp.addr[IW_FUNC] + "\\" + wp.expr;
	  else
	    var = wp.expr;

	  Output(format("%s watch set at %s\\%s\\%d for %s",
		(wp.loc == "local") ? "local" : "class",
		wp.addr[IW_MODULE],wp.addr[IW_FUNC],wp.addr[IW_LINE],var));

	  local	str="";

	  if(wp.qafter > 1) str += "  after: " + wp.qafter;

	  if((wp.loc == "local" && wp.qinto) || (wp.loc != "local" && !wp.qinto))
	    str += (wp.qinto) ? "  into" : "  over";

	  if(wp.qsilent) str += "  silent";
	  if(!wp.qsource) str += "  nosource";
	  if(wp.qtemp) str+= "  temporary";

	  if(str != "") Output(str);

	  if(wp.cond) Output("  when (" + wp.cond + ")");
	  if(wp.docmd) Output("  do (" + wp.docmd + ")");
	}

	return ATT_E_SUCCESS;
	  
}
	function cATTDbg::CmdStep()
//	---------------------------
//	STEP /CALL /INTO /LINE /OVER /RETURN /[NO]SILENT /[NO]SOURCE [n]
{
	if(c_halted) return ATT_E_CANTCONTINUE;

	local	n;
	local	bp = clone c_stepdef;		// Default STEP parameters

	if(HasExecParam("RETURN")) {bp.qreturn = true; bp.qline = false;}
	if(HasExecParam("CALL"))   {bp.qcall = true;   bp.qline = false;}
	if(HasExecParam("LINE"))   bp.qline = true;

	if(HasExecParam("INTO")) bp.qinto = true;
	if(HasExecParam("OVER")) bp.qinto = false;

	if(HasExecParam("SILENT"))   bp.qsilent = true;
	if(HasExecParam("NOSILENT")) bp.qsilent = false;

	if(HasExecParam("SOURCE"))   bp.qsource = true;
	if(HasExecParam("NOSOURCE")) bp.qsource = false;

	if((n = GetExecParamValue("_P1_"))) bp.qafter = n;

	bp.qmode = ATTDBG_BREAK_STEP;		// Step break
	bp.qtemp = true;			// Temporary breakpoint

	c_breaks.insert(0,bp);			// Add as first entry

	c_running = true;

	return ATT_E_SUCCESS;
}
	function cATTDbg::CmdType()
//	---------------------------
//	TYPE [module\]line[:line]
{
	local	module = GetExecParamValue("_P1_",0);
	local	modt = GetActiveModule(module);

	if(modt.errcode != ATT_E_SUCCESS) return modt.errcode;

	local	line1  = GetExecParamValue("_P1_",1);
	local	line2  = GetExecParamValue("_P1_",2);
	local	lines;

	if(HasExecParam("LINES")) {			// Check LINES
	  if(line2 != null) return ATT_PE_TOOMANY;

	  lines = GetExecParamValue("LINES");
	}
	else
	  lines = c_typelines;

	local	maxline = c_srclines[modt.modidx].len() - 1;

	if(line1 == null) line1 = c_typedline[modt.modidx] + 1;
	if(line1 > maxline) return ATT_E_NOSUCHLINE;

	if(line2 == null) line2 = line1 + lines - 1;
	if(line2 < line1) return ATT_E_INVLINE;

	if(line2 > maxline) line2 = maxline;

	Output("module: " + modt.module);

	for (local i=line1; i <= line2; i++) {
	  local str = ExpandTabs(c_srclines[modt.modidx][i],c_tabstop);
	  Output(format("%4d: %s",i,str));
	}

	c_typedline[modt.modidx] = line2;		// Last line displayed

	return ATT_E_SUCCESS;
}


//	Command function table

	local	s_cmdfunc = {
		_AT	= cATTDbg.Cmd_AT,
		CANALL	= cATTDbg.CmdCancelAll,
		CANBRK	= cATTDbg.CmdCancelBreak,
		CANMOD	= cATTDbg.CmdCancelModule,
		CANWAT	= cATTDbg.CmdCancelWatch,
		DEFINE	= cATTDbg.CmdDefine,
		DELETE	= cATTDbg.CmdDelete,
		DEPOS	= cATTDbg.CmdDeposit,
		EVAL	= cATTDbg.CmdEvaluate,
		EXAMIN	= cATTDbg.CmdExamine,
		GO	= cATTDbg.CmdGo,
		SETAUP	= cATTDbg.CmdSetAutopause,
		SETBRK	= cATTDbg.CmdSetBreak,
		SETFIL	= cATTDbg.CmdSetFilter,
		SETFUN	= cATTDbg.CmdSetFunctions,
		SETLIN	= cATTDbg.CmdSetLines,
		SETLOG	= cATTDbg.CmdSetLog,
		SETMOD	= cATTDbg.CmdSetModule,
		SETNAP	= cATTDbg.CmdSetNoautopause,
		SETNFN	= cATTDbg.CmdSetNofunctions,
		SETNLG	= cATTDbg.CmdSetNolog,
		SETSCP	= cATTDbg.CmdSetScope,
		SETSRC	= cATTDbg.CmdSetSource,
		SETSTP	= cATTDbg.CmdSetStep,
		SETTAB	= cATTDbg.CmdSetTabstop,
		SETWAT	= cATTDbg.CmdSetWatch,
		SHOAUP	= cATTDbg.CmdShowAutopause,
		SHOBRK	= cATTDbg.CmdShowBreak,
		SHOCAL	= cATTDbg.CmdShowCalls,
		SHOFIL	= cATTDbg.CmdShowFilter,
		SHOFUN	= cATTDbg.CmdShowFunctions,
		SHOLIN	= cATTDbg.CmdShowLines,
		SHOLOG	= cATTDbg.CmdShowLog,
		SHOMOD	= cATTDbg.CmdShowModule,
		SHOSCP	= cATTDbg.CmdShowScope,
		SHOSRC	= cATTDbg.CmdShowSource,
		SHOSTP	= cATTDbg.CmdShowStep,
		SHOSYM	= cATTDbg.CmdShowSymbol,
		SHOTAB	= cATTDbg.CmdShowTabstop,
		SHOWAT	= cATTDbg.CmdShowWatch,
		STEP	= cATTDbg.CmdStep,
		TYPE	= cATTDbg.CmdType,
		};

	function cATTDbg::ExecuteCmd():(s_cmdfunc)
//	------------------------------------------
//	Execute command
{
	if(c_exect.funcid in s_cmdfunc) {
	  local errcode = s_cmdfunc[c_exect.funcid].call(this);

	  if(errcode == null) {
	    OutputE("Command did not return a value: " + c_exect.funcid);
	    return ATT_PE_PARSER;
	  }

	  if(errcode != ATT_E_SUCCESS) {
	    if(c_cmdlvl) OutputW(c_exect.cmd);		// Echo command

	    OutputErrMsg(errcode);
	  }

	  return errcode;
	}

	OutputE("No function in s_cmdfunc: " + c_exect.funcid);
	return ATT_PE_PARSER;
}
	function cATTDbg::GetIdentifier(var)
//	------------------------------------
//	Scan var for identifier part of variable ie: t.a and t[1] return t
//	Checks if id exist in local or this table
//	Returns		table with the following fields:
//			id	the identifier
//			loc	location (null if not found, "local" or
//				this.tostring() for class instance id
//			value	Current value or not in table if can't evaluate
{
	local	id = null;

	for (local i=0; i < var.len(); i++) {
	  if(var[i] == '.' || var[i] == '[') {
	    id = var.slice(0,i);
	    break;
	  }
	}

	if(!id) id = var;

//	Find id location

	local	loc = null;
	local	level = GetStackLevel();
	local	si = getstackinfos(level + c_scopelvl);
	
	if(id in si.locals)
	  loc = "local";

	else if("this" in si.locals && id in si.locals["this"])
	  loc = si.locals["this"].tostring();

	local	ret={};

	if(loc) {
	  local expret = EvaluateExpr(si.locals,var);

	  if(expret.errcode == ATT_E_SUCCESS) ret.value <- expret.value;
	}

	ret.id <- id;
	ret.loc <- loc;

	return ret;
}
