	function cATTDbg::ReadFile(filename)
//	------------------------------------
//	Read a source file into an array and return it or null if failed
{
	local	fd = cATTIO();

	if(!fd.Open(filename)) return null;

	local	arr = [];

	arr.push("");				// Index lines starting 1

	while (!fd.EOF())
	  arr.push(fd.Read());

	return arr;
}
	function cATTDbg::GetFileIndex(filename)
//	----------------------------------------
//	Search c_srcfiles for the filename. If not found, load the file
//	and add its name to c_srcfiles and the lines to c_srclines
//	If the file was not loaded, add its name to c_srcfiles with a null
//	array added to c_srclines
//	Return index into these two tables or -1 if could not find/load file
{
	foreach (i,str in c_srcfiles)
	  if(str == filename) return (c_srclines[i] != null) ? i : -1;

	local lines = ReadFile(filename);

	c_srcfiles.push(filename);
	c_typedline.push(0);				// No typed line

	if(lines) {
	  c_srclines.push(lines);
	  return c_srcfiles.len() - 1;
	}

	c_srclines.push(null);				// No source lines
	return -1;					// Not found/loaded
}
	function cATTDbg::GetModuleIndex(module)
//	----------------------------------------
//	Find filename using module specification
//	module		ex: dir\file or just file
//	Returns		Index into source tables or -1 if not found
{
	local	modarr = ::split(module,"\\");

	modarr.reverse();

	foreach (i,filename in c_srcfiles) {
	  if(MatchModule(filename,modarr)) return i;
	}

	return -1;
}
	function cATTDbg::MatchModule(filename,modarr)
//	----------------------------------------------
//	Match a filename to a module specification
//	filename	Full file specification, including ".nut"
//	modarr		Reversed module name array
//	Returns		true if matched
{
	local	filarr = ::split(filename,"\\.");

	if(filarr.len() - 1 < modarr.len()) return false;

	filarr.reverse();
	filarr.remove(0);				// "nut"

	foreach (j,str in modarr) {
	  if(str.toupper() != filarr[j].toupper()) return false;
	}

	return true;
}
	function cATTDbg::GetModule(filespec)
//	-------------------------------------
//	Return the module name from the file specification
{
	local	arr = ::split(filespec,"\\.");

	return arr[arr.len() - 2];
}
	function cATTDbg::ExpandTabs(inp,tabstop)
//	-----------------------------------------
//	Replace tabs with spaces, return new string
//	inp		String to process
//	tabstop		Tab stop value
{
	local	nt = tabstop;			// Next tab stop
	local	j = 0;				// Output pointer
	local	i = 0, ilen = inp.len();
	local	out = "";

	while (ilen--) {
	  if(inp[i] != '\t') {
	    out += inp[i++].tochar();
	    j++;
	  }
	  else {				// TAB
	    while (j >= nt)			// Calc next stop
	      nt += tabstop;

	    while (j < nt) {
	      out += " ";
	      j++;
	    }

	    i++;
	  }
	}

	return out;
}
