require("cATTIO.nut");
require("cATTParser_base.nut");
require("cATTParser.nut");
require("cATTDbg.nut");
require("cATTDbg_src.nut");
require("cATTDbg_break.nut");
require("cATTDbg_cmd.nut");
require("cATTDbg_parse.nut");

	class cATTDebug
//	---------------
{
	c_classname = null;

	c_dbg = null;
	c_enabled = null;

	constructor() {
	  c_classname = "!cATTDebug";		// Do not save with game

	  c_enabled = false;			// Not enabled yet
	  c_dbg = cATTDbg();			// The actual debugger

	  ::getroottable().ATTDebug <- this;	// Set debug object to this
	}

	function Enable();			// Enable debugger
	function Disable();			// Disable debugger
	function WriteLog(text);		// Write to log file
	function SkipFunction(funcname);	// Function no to be debugged
}
	function cATTDebug::Enable(silent=false)
//	----------------------------------------
//	Set debug hook to enable debugging a section
//	silent		Unless it is the 1st call, does not cause break if true
{
	if(c_enabled) Disable();

	if(!c_dbg.c_init)			// Not initialized
	  seterrorhandler(DebugEH);
	else if(!silent)
	  c_dbg.SetImmediateBreak();		// Stop on next line

	AIController.SetCommandDelay(0);	// No AI suspend on Do commands
	::setdebughook(DebugHook);

	c_enabled = true;
}
	function cATTDebug::Disable()
//	-----------------------------
//	Disable debugging until Enable is called
//	Returns		true if was enabled
{
	local	enabled = c_enabled;

	if(c_enabled) {
	  ::setdebughook(null);

//	  c_dbg.CancelStepBreak();
	  AIController.SetCommandDelay(1);	// Suspend AI on Do commands

	  c_enabled = false;
	}

	return enabled;
}
	function cATTDebug::SkipFunction(funcname)
//	------------------------------------------
//	Set a function name in which the debugger will not stop
//	Used to allow return from a function that enables the debugger
{
	c_dbg.c_skipfunc = funcname;
}
	function cATTDebug::WriteLog(str)
//	---------------------------------
//	Write a message to the debugger output log
//	Can be called by the main app to log its output to a file
{
	c_dbg.WriteLog(str);
}
	function DebugHook(type,file,line,func)
//	---------------------------------------
//	Called by VM each time a VM line is executed
//	To be called, sq_enabledebuginfo(this->vm,1) must be added to
//	  trunk\src\script\squirrel.cpp at end of Squirrel::Squirrel()
//	Returns		0 to keep calling the debugger on same instruction
//			1 to allow main script to continue
{
	return ::getroottable().ATTDebug.c_dbg.Debug(type,file,line,func);
}
	function DebugEH(errstr)
//	------------------------
//	Set as the error handler when the debugger is initialized
//	Called by VM when an execution error is detected
{
	::getroottable().ATTDebug.c_dbg.HandleError(errstr);
}
