require("..\\ATTDebug\\cATTDebug.nut");

	class cATT extends cATTBase
//	---------------------------
{
	c_classname = null;

	c_roadmgr = null;			// Road manager
	c_airmgr = null;			// Aircraft manager

	c_loaded = null;			// Game loaded from save

	c_var = null;

	c_gFindService = null;			// FindService generator
	c_compete = null;			// Compete for cargo
	c_srvcfnd = null;			// Service found
	c_inConsiderRoute = null;		// Considering a route
	c_plan = null;				// Planned route table
	c_nofunds = null;			// No funds for planned route
	c_dtdate = null;			// Daily task date

	constructor() {
	  s_g.debug <- null;			// ATT debugger
	  s_g.dbgenabled <- false;

	  s_g.rtelimchk <- true;		// Check route limits (not testing)

	  s_g.avgrtecosts <- null;		// Average route costs
	  s_g.methods <- YATT_CMB_ALL;		// Use all methods

	  s_g.tasks <- cATTTaskList();		// Scheduled tasks
	  s_g.save <- [];			// Save money for task

	  s_g.routes <- {};			// All routes: code = route
	  s_g.rtesrc <- {};			// Route sources: code = route
	  s_g.rtestp <- {};			// Have stop here code = arr[routes..]
	  s_g.airports <- {};			// Airports: code = arr[cattairport...]

	  s_g.rtebad <- {};			// Unbuildable routes
	  s_g.indcls <- [];			// Industry closing in 30 days

	  s_g.depord <- {};			// Orders for vehicle in depot
	  s_g.depots <- {};			// Tile : Depot xref

	  s_g.restab <- [];			// Funds reserved for routes
	  s_g.ressum <- 0;			// Total reserved funds

	  s_g.abort <- false;			// Abort route planning

	  s_g.ais <- 0;				// Competitors of note

	  s_g.att <- this;			// Used by pathfinders

	  c_classname = "cATT";

	  c_roadmgr = cATTRoadMgr();
	  c_airmgr = cATTAirMgr();

	  c_loaded = false;			// Assume new game

	  c_gFindService = null;		// FindService generator
	  c_compete = false;			// Do not compete yet
	  c_inConsiderRoute = false;
	  c_plan = null;

	  c_dtdate = AIDate.GetCurrentDate()-1;	// Yesterday
	}
}
	function cATT::Main()
//	---------------------
{
	s_g.debug = cATTDebug();

//	s_cfg.road = 0;
//	s_cfg.air = 0;
	s_cfg.ship = 0;
	s_cfg.subsidies = 0;

	AIRoad.SetCurrentRoadType(AIRoad.ROADTYPE_ROAD);

	if(!c_loaded) {				// New game
	  Info("----- YATT started -----");

//	  ShowIDs();				// Town and industry IDs

//	  RepayLoan();

	  ListSettings();
	  ScanAIs();
	  FindInitialVehicles();

	  DebugEnable();

//	local route = BuildRoute(0,31,14);		// Passengers
//	local route = BuildRoute(0,1,2);		// Passengers
//	local route = BuildRoute(1,45,20,false,false);	// Coal
//	local route = BuildRoute(8,46,23,false,false);	// Iron
//	local route = BuildRoute(4,48,42,false,false);	// Livestock
//	local route = BuildRoute(6,25,42,false,false);	// Grain
//	local route1 = BuildRoute(5,5,29,false);
//	local route = BuildRoute(7,12,46,false,false);	// Wood

	  if(s_cfg.air && c_airmgr.EstimateAvgAirRouteCost() < GetAvailableFunds())
	    s_g.methods = YATT_CMB_AIR;			// Air only for now

	  AddTask(YATT_TSK_FINDSERVICE,0);	// Find a service immediately
	  AddTask(YATT_TSK_BUILDSTATUES,90);

	  AddYearEndTask();

//	  local	yatt = YATT.instance();
//	  yatt.c_att = this;

//	  local	t = yatt.Save();

//	  yatt.Load(1,t);
	}
	else {
	  Info("----- YATT continuing from saved game  -----");

	  DebugEnable();

	  if(c_plan) {
	    ConsiderRoute(c_plan);

	    AddTask(YATT_TSK_FINDSERVICE,0);	// Find a service immediately
	  }
	}

	while (true) {
	  CheckGameTasks();
	}
}
	function cATT::CheckGameTasks()
//	-------------------------------
//	This is the main AI routine
{
	CheckDailyTasks();

//	Handle FindService generator
//	If a route was just built and we have enough funds to build the most
//	expensive route type plus funds for half the max vehicles for the
//	route just built, continue with generator, otherwise reset it
//	and schedule another search in 90 days

	local	t;

	if(c_gFindService && !c_inConsiderRoute) {
	  if((t = resume c_gFindService) != null) {	// Service found
	    local route = ConsiderRoute(t);

	    if(typeof route == "integer") {
	      switch (route) {
	      case YATT_E_NOFUNDS:
		c_nofunds = true;			// Do not incr distance
		break;
	      }
	    }
	    else {					// Built and started
	      c_srvcfnd = true;

	      if(GetAvailableFunds() < GetMaxArrValue(s_g.avgrtecosts) + s_g.ressum) {
	        c_gFindService = null;			// Reset generator
		AddTask(YATT_TSK_FINDSERVICE,30);	// Reschedule
	      }
	    }
	  }
	  else {
	    c_gFindService = null;

	    if(!c_srvcfnd && !c_nofunds) {		// No buildable service found
	      if(s_g.methods != YATT_CMB_ALL) {		// Not all methods used
		c_airmgr.SetSpread(false);
	        AddTask(YATT_TSK_FINDSERVICE,90);	// Reschedule
	      }
	      else if(!c_compete) {			// Not competing
		Warning("Switching to competing routes");
	        c_compete = true;			// Start competing
	        AddTask(YATT_TSK_FINDSERVICE,0);	// Try again now
	      }
	      else {					// Increase distances
		local newdist = false;

		for (local i=YATT_CMI_RAIL; i <= YATT_CMI_HELI; i++) {
		  if(s_cdist.max[i] < s_cdist.abs[i] &&
		     GetAllowedVehCount(i) >= s_minveh[i]) {
		    s_cdist.max[i] += 20;
		    newdist = true;			// Something increased
		  }
		}

		if(newdist) {
	          Warning("Switching to longer routes");
		  c_compete = false;

	          AddTask(YATT_TSK_FINDSERVICE,0);	// Try again now
		}
		else {
		  Info("All methods at absolute distance or max vehicles");
	          AddTask(YATT_TSK_FINDSERVICE,90);	// Reschedule
		}
	      }
	    }
	    else
	      AddTask(YATT_TSK_FINDSERVICE,(c_nofunds) ? 30 : 0);// Reschedule
	  }
	}
}
	function cATT::CheckDailyTasks()
//	--------------------------------
{
	local	tsk,today = AIDate.GetCurrentDate();

	if(c_dtdate == today) return;		// Already ran today

	c_dtdate = today;

	CheckEvents();

	while (s_g.save.len()) {		// Saving for something
	  tsk = s_g.save[0];

	  if(!GetFunds(tsk.funds)) break;

	  s_g.save.remove(0);
	  tsk.obj[tsk.func](tsk);
	}

//	Handle scheduled tasks

	while (tsk = s_g.tasks.GetNextTask())
	  tsk.obj[tsk.func](tsk);

	RepayLoan();
}
	function cATT::BuildRoute(cargo,srcid,dstid,srctown=true,dsttown=true)
//	----------------------------------------------------------------------
//	Builds a route. Bypasses route limit checks
//	Returns		route or null
{
	local	t = SetRouteInfo(srcid,srctown,dstid,dsttown,cargo);

	s_g.rtelimchk = false;
	local	ret = BuildBestConnection(t);
	s_g.rtelimchk = true;

	return ret;
}
	function cATT::ConsiderRoute(t)
//	-------------------------------
//	Returns		route or error code
{
	c_inConsiderRoute = true;			// Protect this code
	s_g.abort = false;

	local	ret = pConsiderRoute(t);

	c_plan = null;					// Planned route
	c_inConsiderRoute = false;

	if(typeof ret == "integer") {
	  Info("Failed to build: " + GetErrorStr(ret));
	  return ret;
	}

	return ret;
}
	function cATT::pConsiderRoute(t)
//	--------------------------------
//	Returns		null if route cannot be built at this time
//			YATT_E_NOFUNDS if not enough money
//			route if route built and service started
{
	local	rtecode = ((t.srctown) ? "T" : "I") + t.srcid + "_" +
			  ((t.dsttown) ? "T" : "I") + t.dstid + "_" + t.cargo;

	local	rtename = AICargo.GetCargoLabel(t.cargo) + " " +
			  t.srcname + " to " + t.dstname;

	local	rtechar = ["T","R","A","S","H"];
	local	rtedescr = rtename + " by "+s_rtemethod[t.method];
	local	info = true;

	if(info) Info("Considering " + rtedescr);

	if(rtecode+"_"+rtechar[t.method] in s_g.rtebad) return YATT_E_MARKEDASBAD;

	if(GetAllowedVehCount(t.method) < s_minveh[t.method]) return YATT_E_VEHLIMIT;

	if(s_g.indcls.len() && !t.srctown) {	// Check closing industries
	  foreach (i,iid in s_g.indcls) {
	    if(t.srcid == iid || (t.dstid == iid && !t.dsttown))
	      return YATT_E_INDCLOSING;
	  }
	}

	local	planrte;

	c_plan = t;				// Planned route table

	switch (t.method) {
	case YATT_CMI_ROAD:
	  planrte = c_roadmgr.EvaluateRoadRoute(t);
	  break;

	case YATT_CMI_AIR:
	  planrte = c_airmgr.EvaluateAirRoute(t);
	  break;

	default:
	  Error("Unsupported transportation type: "+t.method);
	  c_inConsiderRoute = false;
	  return YATT_E_INTERNAL;
	}

	if(typeof planrte == "integer")	{			// Error code
	  SetBadRoute(rtecode,rtechar[t.method],planrte);
	  return planrte;
	}
	else {
	  if(!GetNewRouteFunds(planrte.cost)) return YATT_E_NOFUNDS;

	  if(GetAllowedVehCount(t.method) < s_minveh[t.method]) return YATT_E_VEHLIMIT;

	  DisplayRouteEstimateData(t,planrte,rtedescr);

	  switch (t.method) {
	  case YATT_CMI_ROAD:
	    if(!c_roadmgr.StartRoadService(planrte.route)) return YATT_E_FAILEDTOSTART;
	    break;

	  case YATT_CMI_AIR:
	    if(!c_airmgr.StartAirService(planrte.route)) return YATT_E_FAILEDTOSTART;
	    break;
	  }
	}

	return planrte.route;
}
	function cATT::SetBadRoute(rtecode,rtetype,errcode)
//	---------------------------------------------------
//	Add an entry to s_g.rtebad for route type, based on error code
{
	switch (errcode) {
	case YATT_E_RTETOOSHORT:
	case YATT_E_NOPATH:
	case YATT_E_NODEPOT:
	case YATT_E_INEFFICIENT:
	  s_g.rtebad[rtecode+"_"+rtetype] <- errcode;
	  break;
	}
}
	function cATT::SetStartupReserve(route,vehicles)
//	------------------------------------------------
//	Add reserve table for startup number of vehicles (2/3) for route
//	route		Route to reserve for
//	vehicles	Estimated number of vehicles needed
{
	if(vehicles > 0) {
	  local	t = {route=route, vehicles=vehicles};

	  t.funds <- route.c_engprice * t.vehicles;

	  AddReserve(t);

	  AddTask(YATT_TSK_ENDRESERVE,365,route);	// End reserve in 1 year
	}
}
	function cATT::DisplayRouteEstimateData(t,planrte,rtedescr)
//	-----------------------------------------------------------
{
	Info("Plan done: "+rtedescr);
	Info("Length: "+planrte.route.c_length+" Cost: "+planrte.cost);
	Info("Days: "+t.vt.days);
	Info("Vehicles: "+t.vehicles+" Max: "+planrte.route.c_maxveh);
	Info("Profit: "+t.vehicles * t.vt.net+" Per vehicle: "+t.vt.net);
}
	function cATT::AddTask(task,days,p1=null,p2=null)
//	-------------------------------------------------
{
	if(task == YATT_TSK_FINDSERVICE) Warning("AddTask: "+days);

	return cATTBase.AddTask(this,"DoTask",task,days,p1,p2);
}
	function cATT::DoTask(t)
//	------------------------
{
	this = t.obj;

	switch (t.task) {
	case YATT_TSK_FINDSERVICE:
	  if(IsSaving()) {				// Saving for something
	    Warning("DoTask: Saving");
	    AddTask(YATT_TSK_FINDSERVICE,10);		// Reschedule
	    break;
	  }

	  local	availfunds = GetAvailableFunds() - s_g.ressum;
	  s_g.avgrtecosts = GetAvgRouteCosts();

	  Warning("DoTask: "+availfunds+" "+s_g.avgrtecosts[2]);

	  if(availfunds < GetMinArrValue(s_g.avgrtecosts)) {
	    Warning("DoTask: Nofunds");
	    AddTask(YATT_TSK_FINDSERVICE,30);		// Reschedule
	  }
	  else {
	    c_srvcfnd = false;
	    c_nofunds = false;				// We think
	    c_gFindService = FindService(c_compete,availfunds,s_g.avgrtecosts,
					 s_cfg.pax,s_cfg.freight,
					 s_g.methods);
	  }

	  break;

	case YATT_TSK_YEAREND:
	  DoYearEndTask();
	  break;

	case YATT_TSK_ENDRESERVE:	// Do not reserve funds for route anymore
	  RemoveReserve(t.p1);		// p1 = route
	  break;

	case YATT_TSK_INDCLOSED:			// Industry closed
	  DoIndustryClosedTask(t.p1);			// p1 = industry id
	  break;

	case YATT_TSK_BUILDSTATUES:
	  AddTask(YATT_TSK_BUILDSTATUES,90);
	  BuildStatues();
	  break;
	}
}
	function cATT::DoYearEndTask()
//	------------------------------
{
	AddYearEndTask();				// Schedule next year

	ReportYearEnd();
	c_roadmgr.DoYearEnd();
//	c_airmgr.DoYearEnd();

	for (local m=YATT_CMI_RAIL; m <= YATT_CMI_HELI; m++)
	  UpdateBestVehicle(m);				// Check reliability
}
	function cATT::ReportYearEnd()
//	------------------------------
{
	local	year = AIDate.GetYear(AIDate.GetCurrentDate()) - 1;

	Info("");
	Info("Year End Report for "+year);
	Info("");

	Info("Road routes");
	c_roadmgr.ReportYearEnd();

	Info("Air routes");
	c_airmgr.ReportYearEnd();

	Info("");
}
	function cATT::AddYearEndTask()
//	-------------------------------
//	Schedule year-end report for 1st of Jan next year
{
	local	today = AIDate.GetCurrentDate();
	local	year = AIDate.GetYear(today);
	local	days = AIDate.GetDate(year+1,1,1) - today;

	AddTask(YATT_TSK_YEAREND,days);
}
	function cATT::DoIndustryClosedTask(iid)
//	----------------------------------------
//	Industry closed, cleanup all references to it as iid will be reused
{
	foreach (i,val in s_g.indcls) {		// Close no longer pending
	  if(val == iid) {
	    s_g.indcls.remove(i);
	    break;
	  }
	}

//	Remove bad routes with industry id in them

	local	arr = [];
	local	str = "I"+iid+"_";

	foreach (rtecode,errcode in s_g.rtebad) {
	  if(rtecode.find(str)) arr.push(rtecode);
	}

	foreach (i,rtecode in arr)
	  delete s_g.rtebad[rtecode];
}
	function cATT::ScanAIs()
//	------------------------
{
	local	name;

	for (local cid=0; cid <= 15; cid++) {
	  if((name = AICompany.GetName(cid)) && !AICompany.IsMine(cid))
	    FindAI(name);
	}
}
	function cATT::FindAI(name)
//	---------------------------
{
	Info("Company found: "+name);

	if(name.find("YATT") != null)
	  s_g.ais = s_g.ais | AI_YATT;

	else if(name.find("Rondje") != null)
	  s_g.ais = s_g.ais | AI_RONDJE;
}
	function cATT::BuildStatues()
//	-----------------------------
{
	local	funds = GetAvailableFunds();
	local	needed = GetMaxArrValue(s_g.avgrtecosts)*2 + s_g.ressum + 500000;

	if(funds < needed) return;

	local	t = {};
	local	slist = AIStationList(AIStation.STATION_ANY);

	slist.Valuate(AIStation.GetNearestTown);

	foreach (sid,tid in slist) {		// Count station in towns
	  if(tid in t)
	    t[tid]++;
	  else
	    t[tid] <- 1;
	}

	while (1) {
	  if(!GetFunds(needed)) return;

	  local	max=0,townid;

	  foreach (tid,count in t) {
	    if(count > max) {
	      max = count;
	      townid = tid;
	    }
	  }

	  if(AITown.IsActionAvailable(townid,AITown.TOWN_ACTION_BUILD_STATUE) &&
	     AITown.PerformTownAction(townid,AITown.TOWN_ACTION_BUILD_STATUE))
	    Info("Built statue in "+AITown.GetName(townid));

	  delete t[townid];

	  if(!t.len()) return;
	}
}
