	function cATT::CheckEvents()
//	----------------------------
{
	while (AIEventController.IsEventWaiting()) {
	  local	event = AIEventController.GetNextEvent();
	  local	evtype = event.GetEventType();

	  switch (evtype) {
	  case AIEvent.AI_ET_SUBSIDY_OFFER:
	    if(s_cfg.subsidies) {
	      event = AIEventSubsidyOffer.Convert(event);
	      OnSubsidyOffer(event.GetSubsidyID());
	    }
	    break;

	  case AIEvent.AI_ET_COMPANY_IN_TROUBLE:
	    event = AIEventCompanyInTrouble.Convert(event);

	    if(AICompany.IsMine(event.GetCompanyID())) GetMaxLoan();
	    break;

	  case AIEvent.AI_ET_ENGINE_AVAILABLE:
	    event = AIEventEngineAvailable.Convert(event);

	    OnEngineAvailable(event.GetEngineID());
	    break;

	  case AIEvent.AI_ET_VEHICLE_WAITING_IN_DEPOT:
	    event = AIEventVehicleWaitingInDepot.Convert(event);

	    OnVehicleWaitingInDepot(event.GetVehicleID());
	    break;

	  case AIEvent.AI_ET_INDUSTRY_CLOSE:
	    event = AIEventIndustryClose.Convert(event);

	    OnIndustryClose(event.GetIndustryID());
	    break;

	  case AIEvent.AI_ET_COMPANY_NEW:
	    event = AIEventCompanyNew.Convert(event);

	    FindAI(AICompany.GetName(event.GetCompanyID()));
	    break;

	  case AIEvent.AI_ET_VEHICLE_CRASHED:
	    event = AIEventVehicleCrashed.Convert(event);

	    OnVehicleCrashed(event.GetVehicleID(),event.GetCrashSite(),
			     event.GetCrashReason());
	    break;
	  }
	}
}
	function cATT::OnSubsidyOffer(sid)
//	----------------------------------
//	sid		Subsidy id
{
	if(!c_inConsiderRoute) {
	  local	t = GetSubsidyInfo(sid);

	  if(!IsSelfCompetingRoute(t)) BuildBestConnection(t);
	}
}
	function cATT::OnEngineAvailable(engid)
//	---------------------------------------
{
	local	method = null;

	switch (AIEngine.GetVehicleType(engid)) {
	case AIVehicle.VT_ROAD:
	  method = YATT_CMI_ROAD;
	  break;

	case AIVehicle.VT_AIR:
	  break;
	}

	if(method != null) {
	  Info("New engine: "+AIEngine.GetName(engid));

	  UpdateBestVehicle(method);
	}
}
	function cATT::OnVehicleWaitingInDepot(vehid)
//	---------------------------------------------
{
	if(vehid in s_g.depord) {
	  local tsk = s_g.depord[vehid];
	  tsk.obj[tsk.func](tsk);
	}
	else {						// Must be old
	  switch (AIVehicle.GetVehicleType(vehid)) {
	  case AIVehicle.VT_ROAD:
	    c_roadmgr.VehicleWaitingInDepot(vehid);
	    break;

	  case AIVehicle.VT_AIR:
	    c_airmgr.VehicleWaitingInDepot(vehid);
	    break;
	  }
	}
}
	function cATT::OnIndustryClose(iid)
//	-----------------------------------
{
	Info("Industry closing: "+AIIndustry.GetName(iid));

	s_g.indcls.push(iid);		// Do not connect to this ind any more

	AddTask(YATT_TSK_INDCLOSED,35,iid);	// Process closed industry task

	local	idcode = "I"+iid;

	if(idcode in s_g.rtestp) {
	  Info("  Routes affected:");

	  foreach (i,route in s_g.rtestp[idcode]) {
	    Info("    "+route.GetRouteName());

	    StopService(route,(iid == route.c_srcid));
	  }
	}

	if(c_inConsiderRoute &&
	   ((!c_plan.srctown && c_plan.srcid == iid) ||
	    (!c_plan.dsttown && c_plan.dstid == iid))) s_g.abort = true;
}
	function cATT::OnVehicleCrashed(vehid,tile,reason)
//	--------------------------------------------------
{
	Warning("Vehicle crashed: "+vehid);

	switch (reason) {
	case AIEventVehicleCrashed.CRASH_RV_LEVEL_CROSSING:
	case AIEventVehicleCrashed.CRASH_RV_UFO:
	case AIEventVehicleCrashed.CRASH_FLOODED:
	  c_roadmgr.VehicleCrashed(vehid,tile,reason);
	  break;

	case AIEventVehicleCrashed.CRASH_PLANE_LANDING:
	case AIEventVehicleCrashed.CRASH_AIRCRAFT_NO_AIRPORT:
	  c_airmgr.VehicleCrashed(vehid,tile,reason);
	  break;
	}
}
	function cATT::GetSubsidyInfo(subid)
//	------------------------------------
//	Get subsidy information
//	subid		Subsidy id
//	Returns		a table with the following fields:
//			srcid		Source town/industry id
//			dstid		Destination town/industry id
//			srctown		Source id is town id
//			dsttown		Dest id is town id
//			srctile		Source tile id
//			dsttile		Destination tile id
//			dist		Distance between src and dst
{
	return SetRouteInfo(AISubsidy.GetSourceIndex(subid),
			    AISubsidy.GetSourceType(subid) == AISubsidy.SPT_TOWN,
			    AISubsidy.GetDestinationIndex(subid),
			    AISubsidy.GetDestinationType(subid) == AISubsidy.SPT_TOWN,
			    AISubsidy.GetCargoType(subid));
}
	function cATT::StopService(route,unload)
//	----------------------------------------
//	Sell all vehicles and stop service on route
{
	local	obj = [null,c_roadmgr,null,null,null];

	obj[route.c_type].StopService(route,unload);
}
