	function cATT::FindService(compete,availfunds,avgrtecosts,
				   pax,freight,methods)
//	----------------------------------------------------------
//	Generator to find a service
//	compete		true if we should compete for cargo
//	availfunds	Funds avaliable
//	avgrtecosts	Estimated cost to build avr rte of each rte type
//	pax		True to allow pax routes
//	freight		True to allow freight routes
//	methods		YATT_CMB_ flag bits to allow connection methods
{
	local	scores = AIList(), routes = [], idx=0;
	local	cargolist = AICargoList();

	Warning("FindService started");

	foreach (cargo,clv in cargolist) {		// Cargo
	  if(cargo == s_crgpassengers) {
	    if(!pax) continue;				// pax not allowed
	  }
	  else if(!freight)
	    continue;					// freight not allowed

//	  Find best vehicle to carry cargo for each route type
//	  If we don't have enough funds to build route type, push null

	  local veharr = [];	// Best vehicle for route type for cargo

	  for (local b=1,i=YATT_CMI_RAIL; i <= YATT_CMI_HELI; i++, b = b<<1) {
	    if(!avgrtecosts[i] || avgrtecosts[i] > availfunds || !(methods & b))
	      veharr.push(null);
	    else
	      veharr.push(GetBestVehicle(i,cargo));
	  }

	  if(!veharr.len()) continue;			// No vehicles

//	  Find producers and consumers

	  local src = GetProducerList(cargo,compete);
	  if(src.list.IsEmpty()) continue;

	  local dst = GetConsumerList(cargo);
	  if(dst.list.IsEmpty()) continue;

//	  Find sources

	  foreach (srcid,srcprod in src.list) {			// Source
	    local srctile,srcname;

	    if(src.town) {
	      srctile = AITown.GetLocation(srcid);
	      srcname = AITown.GetName(srcid);
	    }
	    else {
	      srctile = AIIndustry.GetLocation(srcid);
	      srcname = AIIndustry.GetName(srcid);
	    }

	    if(src.town && dst.town) dst.list.RemoveItem(srcid);

//	    Check on daily tasks

	    CheckDailyTasks();

//	    Find destinations

	    foreach (dstid,dstprod in dst.list) {		// Destination
	      local dsttile,dstname;

	      if(dst.town) {
	        dsttile = AITown.GetLocation(dstid);
	        dstname = AITown.GetName(dstid);
	      }
	      else {
	        dsttile = AIIndustry.GetLocation(dstid);
	        dstname = AIIndustry.GetName(dstid);
	      }

	      local distm = AITile.GetDistanceManhattanToTile(srctile,dsttile);
	      local dists = (sqrt(AIMap.DistanceSquare(srctile,dsttile))+0.5).tointeger();
	      local conn = GetPossibleConnections({distm=distm,dists=dists});

	      if(!conn) continue;

//	      Save route info and route score

	      local t =  {srcid=srcid, srctown=src.town, srctile=srctile,
			  dstid=dstid, dsttown=dst.town, dsttile=dsttile,
			  srcname=srcname, dstname=dstname,
			  distm=distm, dists=dists, cargo=cargo};

	      local i,b=1,rt,score;

	      for (i=YATT_CMI_RAIL; i <= YATT_CMI_HELI; i++, b = b<<1) {
		if(!(conn & b) || !veharr[i]) continue;

	        score = GetRouteScore(t,veharr[i]);

	        if(score) {
		  rt = clone t;
		  rt.method <- i;			// Transport method
		  rt.vt <- clone veharr[i];

	          scores.AddItem(idx,score);
	          routes.push(rt);
	          idx++;
		}
	      }
	    }
	  }
	}

	foreach (i,score in scores) {
	  if(!IsSelfCompetingRoute(routes[i]) &&
	     GetAllowedVehCount(routes[i].method) >= s_minveh[routes[i].method])
	    yield routes[i];
	}

	Warning("FindService terminated");

	return null;
}
	function cATT::GetProducerList(cargo,compete)
//	---------------------------------------------
//	compete		false if we don't want to compete taking cargo
//	Returns		Table with the following fields:
//			list		Town or Industry list
//			town		true if town list
{
	local	list;

	if(cargo == s_crgpassengers || cargo == s_crgmail) {
	  list = AITownList();

	  if(!compete) {
	    list.Valuate(AITown.GetLastMonthTransported,cargo);
	    list.KeepValue(0);
	  }

	  list.Valuate(AITown.GetLastMonthProduction,cargo);
	  list.KeepAboveValue((cargo == s_crgpassengers) ? 89 : 120);

	  return {list = list, town = true};
	}

	list = AIIndustryList_CargoProducing(cargo);

	if(!compete) {
	  list.Valuate(AIIndustry.GetLastMonthTransported,cargo);
	  list.KeepValue(0);
	}

	list.Valuate(AIIndustry.GetLastMonthProduction,cargo);
	list.KeepAboveValue(
		(AICargo.HasCargoClass(cargo,AICargo.CC_ARMOURED)) ? 29 : 74);

	return {list = list, town = false};
}
	function cATT::GetConsumerList(cargo)
//	------------------------------------
{
	local	list,effect;

	switch (effect = AICargo.GetTownEffect(cargo)) {
	case AICargo.TE_NONE:
	case AICargo.TE_WATER:
	  list = AIIndustryList_CargoAccepting(cargo);

	  return {list = list, town = false};
	}

	list = AIList();
	list.AddList(AITownList());

	switch (effect) {
	case AICargo.TE_PASSENGERS:
	case AICargo.TE_MAIL:
	  list.Valuate(AITown.GetLastMonthProduction,cargo);
	  list.KeepAboveValue((cargo == s_crgpassengers) ? 89 : 120);
	  break;

	case AICargo.TE_GOODS:
	  list.Valuate(AITown.GetPopulation);
	  list.KeepAboveValue(1500);

	case AICargo.TE_FOOD:
	  list.Valuate(AITown.GetPopulation);
	  list.KeepAboveValue(500);
	}

	return {list = list, town = true};
}
