	function cATT::SetRouteInfo(srcid,srctown,dstid,dsttown,cargo)
//	--------------------------------------------------------------
//	Return table with route info fields
{
	local	t = {srcid=srcid,srctown=srctown,dstid=dstid,dsttown=dsttown,
		     cargo=cargo};

	if(t.srctown) {
	  t.srctile <- AITown.GetLocation(t.srcid);
	  t.srcname <- AITown.GetName(t.srcid);
	}
	else {
	  t.srctile <- AIIndustry.GetLocation(t.srcid);
	  t.srcname <- AIIndustry.GetName(t.srcid);
	}

	if(t.dsttown) {
	  t.dsttile <- AITown.GetLocation(t.dstid);
	  t.dstname <- AITown.GetName(t.dstid);
	}
	else {
	  t.dsttile <- AIIndustry.GetLocation(t.dstid);
	  t.dstname <- AIIndustry.GetName(t.dstid);
	}

	t.distm <- AIMap.DistanceManhattan(t.srctile,t.dsttile);
	t.dists <- (sqrt(AIMap.DistanceSquare(t.srctile,t.dsttile))+0.5).tointeger();

	return t;
}
	function cATT::BuildBestConnection(t)
//	-------------------------------------
//	t	Table with basic route paramters as filled by SetRouteInfo
//	Returns		route or YATT_E_ code
{
	local	conn = GetPossibleConnections(t);

	if(!conn) return;					// No way

	local	availfunds = GetAvailableFunds() - s_g.ressum;
	local	avgrtecosts = GetAvgRouteCosts();

	if(availfunds < GetMinArrValue(avgrtecosts)) return;	// No funds

	local	i,vt=array(5),scores=[],b=1;

	for (i=YATT_CMI_RAIL; i <= YATT_CMI_HELI; i++, b = b<<1) {
	  local score = 0;

	  if((conn & b) &&
	     avgrtecosts[i] && avgrtecosts[i] <= availfunds &&
	     (vt[i] = GetBestVehicle(i,t.cargo))) score = GetRouteScore(t,vt[i]);

	  scores.push(score);
	}

	local	route = null;

	while (true) {				// Find method with best score
	  local max=0,method=-1;

	  foreach (i,score in scores) {
	    if(score > max) {
	      max = score;
	      method = i;
	    }
	  }

	  if(method < 0) break;

	  t.method <- method;				// Connection method
	  t.vt <- vt[method];				// Best vehicle

	  if(typeof (route = ConsiderRoute(t)) != "integer") return route;

	  scores[method] = 0;
	}

	return YATT_E_NOROUTE;
}
	function cATT::GetPossibleConnections(t)
//	----------------------------------------
//	Find connection methods that can be used based on distance
//	t	Route description table
{
	if(!s_g.rtelimchk) return 0x1F;		// Allow all connections

	local	i,b,f = 0, dist = t.distm;

	for (b=YATT_CMB_RAIL,i=YATT_CMI_RAIL; i <= YATT_CMI_HELI; i++,b=b<<1) {
	  if(i == YATT_CMI_AIR) dist = t.dists;

	  if(dist >= s_cdist.min[i] && dist <= s_cdist.max[i]) f = f | b;
	}

	return f;
}
	function cATT::GetAvgRouteCosts()
//	---------------------------------
//	Return avg cost of building each route type
{
	return	[null,						// Rail
		 cATTRoadMgr.EstimateAvgRoadRouteCost(),	// Road
	         c_airmgr.EstimateAvgAirRouteCost(),		// Air
	         null,						// Ship
	         null];						// Heli
}
	function cATT::IsSelfCompetingRoute(t)
//	--------------------------------------
//	Check if we'd be competing with ourselves
//	Passengers can be taken from same town to different towns
{
	if(t.cargo == s_crgpassengers) {
	  local rtecode = ((t.srctown) ? "T" : "I") + t.srcid + "_" +
			  ((t.dsttown) ? "T" : "I") + t.dstid + "_" + t.cargo;

	  return (rtecode in s_g.routes);
	}

	local src = ((t.srctown) ? "T" : "I") + t.srcid + "_" + t.cargo;

	if(src in s_g.rtesrc) return true;	// Already taking cargo

	if(t.dsttown) {
	  local dst = "T" + t.dstid + "_" + t.cargo;

	  if(dst in s_g.rtesrc) return true;	// Already taking cargo
	}

	return false;
}
	function cATT::GetRouteScore(t,vt)
//	----------------------------------
//	Gets an estimate of profits for the route
//	Adds vehicles field to table t
//	t		Table describing the route
//	vt		Element of vehicle array with best vehicle
//	Returns		net_profit
{
	vt.net <- EstimateProfit(vt,t.distm);		// One vehicle

	if(vt.net == 0) return 0;			// Too long

	local	prod = GetProduction(t);

//	if(t.srctown) prod *= 2;			// Both ways

	t.vehicles <- (prod*12 / (vt.capacity * vt.trips)).tointeger();

	local	score = vt.net * t.vehicles;		// Profit for 1 way

	return score;					// Route score
//	return vt.net;					// Profit per vehicle
}
	function cATT::GetProduction(t)
//	-------------------------------
//	Estimate source cargo production per month, at 75% 
//	For towns, use lower production end
//	t	Route description table
{
	local	sprod,dprod;

	if(t.srctown) {
	  sprod = AITown.GetLastMonthProduction(t.srcid,t.cargo) -
		  AITown.GetLastMonthTransported(t.srcid,t.cargo);

	  dprod = AITown.GetLastMonthProduction(t.dstid,t.cargo) -
		  AITown.GetLastMonthTransported(t.dstid,t.cargo);

	  if(dprod < sprod) sprod = dprod;		// Use lower value
	}
	else if(s_g.rtelimchk)
	  sprod = AIIndustry.GetLastMonthProduction(t.srcid,t.cargo) -
		  AIIndustry.GetLastMonthTransported(t.srcid,t.cargo);
	else
	  sprod = 80;					// Assume for testing

	return sprod * 3 / 4;
}
