	class cATTAirMgr extends cATTManager
//	------------------------------------
{
	c_classname = null;
	c_spread = null;

	constructor()
//	-------------
	{
	  ::cATTManager.constructor(YATT_CMI_AIR);

	  c_classname = "cATTAirMgr";
	  c_spread = true;
	}
}
	function cATTAirMgr::EstimateAvgAirRouteCost()
//	----------------------------------------------
//	Estimate minimum cost for building an air route
//	Uses passenger service for calculation
{
	local	engid = cATTAirRoute.SelectAircraft(s_crgpassengers);

	if(engid == null) return null;

	local	actype = AIEngine.GetPlaneType(engid);
	local	aptype = (actype == AIAirport.PT_BIG_PLANE) ?
			  AIAirport.AT_LARGE : AIAirport.AT_SMALL;

	local	cost = cATTAirport.GetBuildCost(aptype) * 2;

	if(c_spread) {
	  if(c_routes.len() == 0) cost += AIEngine.GetPrice(engid);
	}
	else
	  cost += AIEngine.GetPrice(engid);

	return cost;
}
	function cATTAirMgr::EvaluateAirRoute(t)
//	----------------------------------------
//	t		Table with route info, prepared by GetSubsidyInfo
//			or FindService
//	Returns		YATT_E_ code or table with the following fields:
//			route		Surveyed route
//			cost		Total cost to build
{
	local	route,cost;

	route = cATTAirRoute();
	route.InitFromTable(t);

	if((cost = route.EstimateAirRouteCost()) < 0) return cost;	// Error

	if(!c_spread || c_routes.len() == 0)
	  cost += route.GetNewPlanePrice();

	return {route = route, cost = cost};
}
	function cATTAirMgr::StartAirService(route)
//	-------------------------------------------
//	Returns		true if service started ok
{
	local	errcode,success = route.BuildAirRoute();

	if(success) {
	  Warning(route.GetRouteName() + ": Built air route");

//	  if(!c_spread || c_routes.len() == 0) {
	    switch (errcode = BuyAircraft(route)) {
	    case YATT_E_SUCCESS:
	      BuyAircraft(route);
	      break;

	    case YATT_E_NOFUNDS:
	      Info("No funds to buy first plane");
	      errcode = YATT_E_SUCCESS;
	      break;

	    default:
	      Error("Unable to start air service: "+GetErrorStr(errcode));
	    }
//	  }
//	  else {
//	    Info("Not buying first plane");
//	    errcode = YATT_E_SUCCESS;
//	  }

	  if(errcode == YATT_E_SUCCESS) {
	    AddRoute(route);
	    route.c_stops[0].AddRoute(route);
	    route.c_stops[1].AddRoute(route);

	    AddTask(YATT_TSK_CHECKROUTE,30,route);

	    route.SetServiceState(cATTRoute.YATT_SST_BUYING);

//	    if(c_spread) {
//	      if(c_routes.len() == 1)
//		s_g.att.SetStartupReserve(route,2-route.GetVehicleCount());

//	      local limitnoise = AIGameSettings.GetValue("station_noise_level");

//	      if(c_routes.len() == ((limitnoise) ? 6 : 3)) SetSpread(false);
//	      if(c_routes.len() == 2) SetSpread(false);

//	      s_g.avgrtecosts[YATT_CMI_AIR] = EstimateAvgAirRouteCost();
//	    }
//	    else
	      s_g.att.SetStartupReserve(route,2-route.GetVehicleCount());
	  }
	  else
	    success = false;
	}
	else
	  DebugEnable();

	return success;
}
	function cATTAirMgr::AddTask(task,days,p1=null,p2=null)
//	-------------------------------------------------------
{
	return ::cATTBase.AddTask(this,"DoTask",task,days,p1,p2);
}
	function cATTAirMgr::DoTask(t)
//	------------------------------
{
	this = t.obj;

	switch (t.task) {
	case YATT_TSK_CHECKROUTE:
	  if(t.p1.c_srvstate != cATTRoute.YATT_SST_STOPPING) CheckAirRoute(t.p1);
	  break;

	default:
	  Error("Task not handled: "+t.task);
	}
}
	function cATTAirMgr::DoDepotOrder(t)
//	------------------------------------
{
	this = t.obj;

	switch (t.order) {
	case YATT_ORD_SELL:
	case YATT_ORD_UPGRADE:
	  delete c_vehs[t.vehid];			// Remove from xref
	  t.route.SellVehicle(t.vehid);

	  if(t.route.c_srvstate == cATTRoute.YATT_SST_STOPPING) {
	    if(!t.route.GetVehicleCount()) OnLastVehicleSold(t.route);
	  }
	  else if(t.order == YATT_ORD_UPGRADE)
	    BuyAircraft(t.route,true);

	  break;
	}

	delete s_g.depord[t.vehid];
}
	function cATTAirMgr::DoSavedTask(t)
//	-----------------------------------
//	Money for a task has been saved, execute task
{
	this = t.obj;

	switch (t.task) {
	case YATT_TSK_BUYVEHICLE:	// p1=route p2=renew
	  t.p1.c_saving = false;

	  if(t.p1.c_srvstate != cATTRoute.YATT_SST_STOPPING)
	    BuyAircraft(t.p1,t.p2,true);

	  break;
	}
}
	function cATTAirMgr::CheckAirRoute(route)
//	-----------------------------------------
{
	if(!c_spread ||
//	   (route == c_routes[0] && route.GetVehicleCount() < 2)) {
	   (route.GetVehicleCount() < 2)) {

	  switch (route.CheckAirRouteProduction()) {
	  case YATT_MGR_BUYVEHICLE:
	    BuyAircraft(route);
	    break;

	  case YATT_MGR_SELLVEHICLE:
	    SendAircraftToDepot(route,route.c_vehids[0],YATT_ORD_SELL);
	    break;

	  case YATT_MGR_UPGRADEVEHICLE:
	    SendAircraftToDepot(route,route.c_vehids[0],YATT_ORD_UPGRADE);
	    break;

	  case YATT_MGR_UPGRADEFLEET:
	    foreach (i,vehid in route.c_vehids) {
	      if(AIVehicle.GetAge(vehid) > 365 * 3)
	        SendAircraftToDepot(route,vehid,YATT_ORD_UPGRADE);
	    }
	    break;
	  }
	}

	local	days = (route.c_srvstate == cATTRoute.YATT_SST_STEADY) ? 45 : 15;

	AddTask(YATT_TSK_CHECKROUTE,days,route);
}
	function cATTAirMgr::BuyAircraft(route,renew=false,saved=false)
//	---------------------------------------------------------------
//	Try and buy vehicle now. If not enough funds, set saving mode
//	and reschedule buying vehicle.
//	route		Route to buy vehicle for
//	renew		True if a vehicle is being renewed, save for it if needed
//	saved		Using saved funds, don't test global save
//	Returns		YATT_E_ code
{
	local	errcode;

	if(saved || !IsSaving()) {			// Global saving
	  switch (errcode = route.BuildAircraft(renew)) {
	  case YATT_E_SUCCESS:
	    AddVehicle(route);				// Last added
	    return YATT_E_SUCCESS;

	  case YATT_E_OTTD:
	    switch (AIError.GetLastError()) {
	    case AIVehicle.ERR_VEHICLE_TOO_MANY:
	      return YATT_E_VEHLIMIT;

	    case AIError.ERR_NOT_ENOUGH_CASH:
	      break;

	    default:
	      Error("Could not buy aircraft: "+AIError.GetLastErrorString());
	      return YATT_E_OTTD;
	    }
//	    Fall thru

	  case YATT_E_NOFUNDS:
	    SaveFunds(this,"DoSavedTask",YATT_TSK_BUYVEHICLE,route.c_engprice,
		      saved,route,renew);
	    route.c_saving = true;			// Route saving
	    return YATT_E_NOFUNDS;

	  default:
	    Error("Could not buy aircraft: "+errcode);
	    return errcode;
	  }
	}
	else if(renew) {		// Replacing a vehicle, save for it
	  SaveFunds(this,"DoSavedTask",YATT_TSK_BUYVEHICLE,route.c_engprice,
		    true,route,renew);
	  return YATT_E_NOFUNDS;
	}
	else if(!route.c_saving) {
	  SaveFunds(this,"DoSavedTask",YATT_TSK_BUYVEHICLE,route.c_engprice,
		    false,route,renew);
	  route.c_saving = true;
	  return YATT_E_NOFUNDS;
	}

	return YATT_E_SAVING;
}
	function cATTAirMgr::SendAircraftToDepot(route,vehid,order,unload=true)
//	-----------------------------------------------------------------------
//	Send oldest vehicle to depot to be sold
{
	local	t = {obj=this, func = "DoDepotOrder",
		     route=route, vehid=vehid, order=order};

	s_g.depord[vehid] <- t;

	route.SendVehicleToDepot(vehid,unload);
}
	function cATTAirMgr::VehicleWaitingInDepot(vehid)
//	-------------------------------------------------
//	Vehicle in depot, must be old if it had no task
{
	local	route = c_vehs[vehid];

	delete c_vehs[vehid];				// Remove from xref
	route.SellVehicle(vehid,true);			// Sell for renew

	BuyAircraft(route,true);
}
	function cATTAirMgr::VehicleCrashed(vehid,tile,reason)
//	------------------------------------------------------
{
	if(vehid in c_vehs) {
	  local route = c_vehs[vehid];

	  delete c_vehs[vehid];

	  if(vehid in s_g.depord) delete s_g.depord[vehid];

	  route.RemoveVehicle(vehid);

	  if(route.GetVehicleCount() < 2) BuyAircraft(route,true);
	}
}
	function cATTAirMgr::SetSpread(mode)
//	------------------------------------
{
	Warning("Air spread: "+mode);

	if(!(c_spread = mode)) {
	  s_g.avgrtecosts[YATT_CMI_AIR] = EstimateAvgAirRouteCost();
	  s_g.methods = YATT_CMB_ALL;
	}
}
