	class cATTAirport extends cATTStation
//	-------------------------------------
{
//	small, large, metropolitan, international, commuter, intercontinental
//	heliport, helidepot, helistation

	static	s_parking = [2, 3, 3, 6, 3, 8, 0,0,0];
	static	s_maxac   = [2, 8,11,14, 6,24, 0,0,0];
	static	s_helipad = [0,0,0,2,2,2, 1,1,3];

	c_classname = null;

	c_tile = null;				// Airport North tile
	c_aptype = null;			// Airport type
	c_hangar = null;			// Hangar tile
	c_idcode = null;			// Location id code

	constructor(tile,aptype)
//	------------------------
//	tile		Airport North tile
//	aptype		AIAirport type
	{
	  ::cATTStation.constructor(YATT_STN_AIR);

	  c_classname = "cATTAirport";
	  c_tile = tile;
	  c_aptype = aptype;
	  c_hangar = null;
	}
}
	function cATTAirport::BuildAirport(idcode,level=false,id=AIStation.STATION_NEW)
//	-------------------------------------------------------------------------------
//	Builds airport and adds this to s_g.airports
//	idcode		Location id, format {T|I}id
//	level		Leveling needed if true
//	id		Attach to station id
//	Returns		true if airport built ok
{
	if(c_built) return true;		// Already built

	local	stat = true;

	if(level) stat = LevelArea(c_tile,AIAirport.GetAirportWidth(c_aptype),
					  AIAirport.GetAirportHeight(c_aptype));

	if(stat) {
	  while (!(stat = AIAirport.BuildAirport(c_tile,c_aptype,id))) {
	    switch (AIError.GetLastError()) {
	    case AIError.ERR_NOT_ENOUGH_CASH:
	      WaitForFunds();
	      break;

	    case AIError.ERR_LOCAL_AUTHORITY_REFUSES:
	      local exclude = cATTTileList();

	      exclude.AddRectangleWH(c_tile,AIAirport.GetAirportWidth(c_aptype),
					    AIAirport.GetAirportHeight(c_aptype));

	      foreach (tile,v in exclude)		// Airport area
		if(!CanBuildOnTile(tile,true,exclude)) return false;

	      break;

	    default:
	      Error("Could not build airport: "+AIError.GetLastErrorString());
	      return false;
	    }
	  }
	}

	if(stat) {
	  c_hangar = AIAirport.GetHangarOfAirport(c_tile);

	  SetStationInfo(c_tile);
	  c_built = true;
	  c_refcnt = 0;

	  AddKeyValue(s_g.airports,idcode,this);	// Location xref
	  c_idcode = idcode;				// Location
	}

	return stat;
}
	function cATTAirport::GetParkingCount(aptype)
//	---------------------------------------------
{
	return cATTAirport.s_parking[aptype];
}
	function cATTAirport::DemolishAirport()
//	---------------------------------------
{
	if(!c_built) return true;			// Never built

	if(c_refcnt > 0) return true;			// Being used

	while (!AIAirport.RemoveAirport(c_tile)) {
	  switch (AIError.GetLastError()) {
	  case AIError.ERR_NOT_ENOUGH_CASH:
	    WaitForFunds();
	    break;

	  default:
	    Error("Error removing airport: "+AIError.GetLastErrorString());
	    return false;
	  }
	}

	c_id = null;
	c_built = false;

	RemoveKeyValue(s_g.airports,c_idcode,this);

	return true;
}
	function cATTAirport::GetBuildCost(aptype=null,level=true)
//	----------------------------------------------------------
//	aptype		AIAirport.AT_ code or null to use c_aptype
//	level		Add estimated cost for leveling area
//	Return cost of building an airport, plus cost of clearing if needed
{
	if(aptype == null) aptype = c_aptype;

	local	area = AIAirport.GetAirportWidth(aptype) *
		       AIAirport.GetAirportHeight(aptype);
	local	cost = AIAirport.GetPrice(aptype);

	cost += AITile.GetBuildCost(AITile.BT_CLEAR_ROUGH) * area;

	if(level) cost += AITile.GetBuildCost(AITile.BT_TERRAFORM) * area / 2;

	return cost;
}
	function cATTAirport::GetAircraftCount()
//	----------------------------------------
//	Return the number of aircraft within 20 tiles of the airport
{
	local	list = AIVehicleList_Station(c_id);

	list.Valuate(AIVehicle.GetVehicleType);
	list.KeepValue(AIVehicle.VT_AIR);

	local	hlist = AIList();

	hlist.AddList(list);
	hlist.Valuate(AIVehicle.GetEngineType);
	hlist.Valuate(AIEngine.GetPlaneType);
	hlist.KeepValue(AIAirport.PT_HELICOPTER);

	list.RemoveList(hlist);

	list.Valuate(AIVehicle.GetLocation);

	local	count = 0;

	foreach (vehid,tile in list) {
	  if(AIStation.GetDistanceSquareToTile(c_id,tile) < 400) count++;
	}

	return count;
}
	function cATTAirport::HasLandingSlot()
//	--------------------------------------
//	Returns true if airport can accept more aircraft
{
	return (GetAircraftCount() < s_maxac[c_aptype]);
}
	function cATTAirport::AddRoute(route)
//	-------------------------------------
{
	c_routes.push(route);
	c_refcnt++;
}
	function cATTAirport::RemoveRoute(route)
//	----------------------------------------
{
	foreach (i,rte in c_routes) {
	  if(rte == route) {
	    c_routes.remove(i);
	    c_refcnt--;
	    break;
	  }
	}
}
