	class cATTAirRoute extends cATTRoute
//	------------------------------------
{
	c_classname = null;
	c_sstop = null;					// Source stop info
	c_dstop = null;					// Dest stop info

	constructor()
//	-------------
	{
	  cATTRoute.constructor(YATT_CMI_AIR);

	  c_classname = "cATTAirRoute";
	}
}
	function cATTAirRoute::SelectAircraft(cargo)
//	--------------------------------------------
//	Select a plane to carry cargo
//	Returns		Engine id selected or null if no plane found
{
	local	t = GetBestVehicle(YATT_CMI_AIR,cargo);

	return (t) ? t.engid : null;
}
	function cATTAirRoute::SelectAirport()
//	--------------------------------------
//	Select largest airport we can build (Generator)
{
	local airports = [AIAirport.AT_INTERCON,
			  AIAirport.AT_INTERNATIONAL,
			  AIAirport.AT_METROPOLITAN,
			  AIAirport.AT_LARGE,
			  AIAirport.AT_COMMUTER,
			  AIAirport.AT_SMALL
			 ];

	foreach (airport in airports)
	  if(AIAirport.IsValidAirportType(airport)) yield airport;

	return null;
}
	function cATTAirRoute::EstimateAirRouteCost()
//	---------------------------------------------
//	Evaluate route and return implementation cost
//	Returns		total cost to implement or YATT_E_ code
{
	if(s_g.rtelimchk) {
	  if(c_dist < s_cdist.min[YATT_CMI_AIR]) return YATT_E_RTETOOSHORT;
	  if(c_dist > s_cdist.max[YATT_CMI_AIR]) return YATT_E_RTETOOLONG;

	  if(c_srctown) {
	    if(AITown.GetPopulation(c_srcid) < 500) return YATT_E_LOWPOPULATION;
	    if(AITown.GetPopulation(c_dstid) < 500) return YATT_E_LOWPOPULATION;
	  }
	  else if(AIIndustry.GetLastMonthProduction(c_srcid,c_cargo) < 80)
	    return YATT_E_LOWPRODUCTION;
	}

	if(!(c_engsel = SelectAircraft(c_cargo))) return YATT_E_NOVEHICLE;

	local stat = SurveyAirRoute();
	if(stat < 0) return stat;

//	return c_cost + AIEngine.GetPrice(c_engsel);
	return c_cost;
}
	function cATTAirRoute::SurveyAirRoute()
//	---------------------------------------
//	Survey an air route
//	Enters route code into s_g.rtebad if route cannot ever be built
//	Calculates c_cost = cost of airports and the estimated
//			    cost of terraforming
//	Returns		YATT_E_ code
{
	c_cost = YATT_E_NOSTOP;				// Temp

	if(c_srctown)
	  c_cost = SurveyTownRoute();
//	else if(c_dsttown)
//	  c_cost = SurveySupplyRoute();
//	else
//	  c_cost = SurveyIndustryRoute();

	if(c_cost < 0) return c_cost;			// Error returned

	if("airport" in c_sstop)			// Source exists
	  c_stops.push(c_sstop.airport);
	else
	  c_stops.push(cATTAirport(c_sstop.tile,c_sstop.aptype));

	if("airport" in c_dstop)			// Destination exists
	  c_stops.push(c_dstop.airport);
	else
	  c_stops.push(cATTAirport(c_dstop.tile,c_dstop.aptype));

	c_length = c_dist;
	c_maxveh = 0;

	return YATT_E_SUCCESS;
}
	function cATTAirRoute::SurveyTownRoute()
//	----------------------------------------
//	Returns		Cost to build both airports or
//			YATT_E_ code
{
	if(IsTownRatingLow(c_srcid) || IsTownRatingLow(c_dstid))
	  return YATT_E_TOWNRATING;

	if(!(c_sstop = SurveyTownAirport(c_srcid))) return YATT_E_NOSTOP;

	local	smallonly = IsSmallAirportType(c_sstop.aptype);

	if(!(c_dstop = SurveyTownAirport(c_dstid,smallonly))) return YATT_E_NOSTOP;

	if(!smallonly && IsSmallAirportType(c_dstop.aptype) &&
	   !(c_sstop = SurveyTownAirport(c_srcid,true))) return YATT_E_NOSTOP;

	if(AITile.GetDistanceManhattanToTile(c_sstop.tile,c_dstop.tile) <
					     s_cdist.min[YATT_CMI_AIR])
	  return YATT_E_RTETOOSHORT;

	return c_sstop.cost + c_dstop.cost;
}
	function cATTAirRoute::SurveyTownAirport(townid,smallonly=false)
//	----------------------------------------------------------------
//	smallonly	Surveys only small airport
//	Returns		null or a table with the following fields:
//			tile	Airport tile
//			aptype	Airport type
//			cost	Build cost, including leveling
//			level	Leveling needed (new airport only)
//			airport	cATTAirport object if airport already exists
{
	local	t = SurveyExistingAirport(true,townid,smallonly);

	if(t) return t;

	local	noise = AITown.GetAllowedNoise(townid);

//	If no noise limit set, only two airports are allowed per town
//	In this case GetAllowedNoise returns the number of airports that
//	can still be built at the town, not the actual noise level

	if(!noise) return null;

	local	tlist = GetTownAuthorityTiles(townid);
	local	limitnoise = AIGameSettings.GetValue("station_noise_level");

	tlist.Valuate(AITile.IsBuildable);
	tlist.KeepValue(1);

	tlist.Valuate(AITile.IsCoastTile);
	tlist.KeepValue(0);

	local	gSelectAirport = SelectAirport();

	local	aptype;
	local	test = AITestMode();
	local	cost,acc = AIAccounting();

	while ((aptype = resume gSelectAirport) != null) {
	  if(smallonly && !IsSmallAirportType(aptype)) continue;

	  local	list = AITileList();
	  list.AddList(tlist);

	  if(limitnoise) {
	    list.Valuate(AIAirport.GetNoiseLevelIncrease,aptype);
	    list.KeepBelowValue(noise + 1);

	    if(!list.Count()) continue;
	  }

	  local	wid = AIAirport.GetAirportWidth(aptype);
	  local	hig = AIAirport.GetAirportHeight(aptype);
	  local	rad = AIAirport.GetAirportCoverageRadius(aptype);

	  list.Valuate(AITile.GetCargoAcceptance,c_cargo,wid,hig,rad);
	  list.KeepAboveValue((c_cargo == s_crgpassengers) ? 74 : 100);

	  list.Sort(AIList.SORT_BY_VALUE,false);

	  foreach (tile,v in list) {
	    if(AITile.IsBuildableRectangle(tile,wid,hig)) {

//	      if(limitnoise &&
//		 AIAirport.GetNoiseLevelIncrease(tile,aptype) >
//		 AITown.GetAllowedNoise(AIAirpor.GetNearestTown(tile,aptype)))
//	        continue;

	      local	level = false;			// No need to level

	      acc.ResetCosts();

	      if(AIAirport.BuildAirport(tile,aptype,AIStation.STATION_NEW))
		cost = acc.GetCosts();
	      else {
		local err = AIError.GetLastError();

		if(err != AIError.ERR_FLAT_LAND_REQUIRED) continue;

		local levelcost = LevelArea(tile,wid,hig);

		if(levelcost == -1) continue;

		cost = levelcost + cATTAirport.GetBuildCost(aptype,false);
		level = true;				// Need leveling
	      }

	      if(WillOverlapAirport(tile,wid,hig,rad)) continue;

	      return {tile=tile, aptype=aptype, cost=cost, level=level};
	    }
	  }
	}

	return null;
}
	function cATTAirRoute::SurveyExistingAirport(town,id,smallonly)
//	---------------------------------------------------------------
//	Check if an orphan airport already exists at town
//	town		True if id is townid, false if industry id
//	id		Town or industry id
//	smallonly	Find small airport only
//	Returns		null or a table with the following fields:
//			airport		cATTAirport object
//			aptype		AIAirport.AT_ code
//			tile		Airport tile
//			cost		Set to 0
{
	local	idcode = ((town) ? "T" : "I" ) + id;

	if(idcode in s_g.airports) {
	  local	airport = s_g.airports[idcode][0];

	  if(!airport.c_routes.len()) {			// Orphan
	    DebugEnable();

	    if(!smallonly || (smallonly && IsSmallAirportType(airport.c_aptype)))
	      return {airport=airport, aptype=airport.c_aptype,
		      tile=airport.c_tile, cost=0};
	  }
	}

	return null;
}
	function cATTAirRoute::BuildAirRoute()
//	--------------------------------------
{
	local	success;

	success = c_stops[0].BuildAirport(GetSrcIDCode(),c_sstop.level);

	if(success)
	  success = c_stops[1].BuildAirport(GetDstIDCode(),c_dstop.level);

	return success;
}
	function cATTAirRoute::BuildAircraft(renew=false)
//	-------------------------------------------------
//	Build selected aircraft and put it into service
//	Returns		YATT_E_ code
{
	c_engsel = SelectAircraft(c_cargo);

	if(!CanUseSelectedAircraft()) return YATT_E_NOVEHICLE;
	  
	c_engprice = AIEngine.GetPrice(c_engsel);

	if(!GetFunds(c_engprice)) return YATT_E_NOFUNDS;

	local	startidx = 0;				// Start at this end

	if(c_srctown && GetVehicleCount() > 0 &&
	   GetCargoWaitingAtStop(1) < GetCargoWaitingAtStop(0)) startidx = 1;

	local	vehid = AIVehicle.BuildVehicle(c_stops[startidx].c_hangar,c_engsel);

	if(!AIVehicle.IsValidVehicle(vehid)) return YATT_E_OTTD;

	if(AIEngine.GetCargoType(c_engsel) != c_cargo)		// Refit needed
	  AIVehicle.RefitVehicle(vehid,c_cargo);

	c_vehids.push(vehid);

	c_totcap += AIVehicle.GetCapacity(vehid,c_cargo);	// Total capacity

//	Add orders to handle old age
//	1:Goto stop 0
//	2:Goto depot 0
//	3:Goto stop 1
//	4:Jump to order +3 (7) when age >= max_age - 1
//	5:Goto depot 1
//	6:Jump to order 1
//	7:Jump to order +2 (9) when load == 0
//	8:Go to stop 0 (Unload)
//	9:Go to nearest depot

	local	breakdowns = AIGameSettings.GetValue("vehicle_breakdowns");
	local	srcorder = 0, dstorder = 0;

	if(!c_srctown)
	  srcorder = srcorder | AIOrder.AIOF_FULL_LOAD_ANY;
//	else if(GetVehicleCount() == 1) {			// First aircraft
	else {
	  srcorder = srcorder | AIOrder.AIOF_FULL_LOAD_ANY;
	  dstorder = dstorder | AIOrder.AIOF_FULL_LOAD_ANY;
	}

	AIOrder.AppendOrder(vehid,c_stops[0].c_tile,srcorder);	// Stop 0

	if(breakdowns)						// Depot 0
	  AIOrder.AppendOrder(vehid,c_stops[0].c_hangar,AIOrder.AIOF_NONE);

	AIOrder.AppendOrder(vehid,c_stops[1].c_tile,dstorder);	// Stop 1
	local	dstordidx = AIOrder.GetOrderCount(vehid) - 1;

	local ageidx = AppendCondOrder(vehid,0,AIOrder.OC_AGE,AIOrder.CF_MORE_EQUALS,
					       AIVehicle.GetMaxAge(vehid)/365-1);

	if(breakdowns)						// Depot 1
	  AIOrder.AppendOrder(vehid,c_stops[1].c_hangar,AIOrder.AIOF_NONE);

	AppendCondOrder(vehid,0);				// Jump to order 1

	local loadidx = AppendCondOrder(vehid,0,AIOrder.OC_LOAD_PERCENTAGE,
						AIOrder.CF_EQUALS,0);

	AIOrder.AppendOrder(vehid,c_stops[0].c_tile,
			    AIOrder.AIOF_UNLOAD | AIOrder.AIOF_NO_LOAD);

	AIOrder.AppendOrder(vehid,0,
			    AIOrder.AIOF_GOTO_NEAREST_DEPOT |
			    AIOrder.AIOF_STOP_IN_DEPOT);

	AIOrder.SetOrderJumpTo(vehid,ageidx,loadidx);		// Adjust jumpto
	AIOrder.SetOrderJumpTo(vehid,loadidx,loadidx+2);	// Adjust jumpto

	if(startidx) AIOrder.SkipToOrder(vehid,dstordidx);	// Other end

//	Create a vehicle group for the route, if needed

	if(c_grpid == null) CreateGroup(AIVehicle.VT_AIR);

	AIGroup.MoveVehicle(c_grpid,vehid);
	AIVehicle.StartStopVehicle(vehid);

	if(!renew) c_datebought = AIDate.GetCurrentDate();

	Info(GetRouteName() + ": Bought " + AIVehicle.GetName(vehid));

	return YATT_E_SUCCESS;
}
	function cATTAirRoute::CheckAirRouteProduction()
//	------------------------------------------------
//	Check production on a route, look for improvements
//	Returns		YATT_MGR code, tells mgr what to do
{
	local	surplus = GetCargoSurplus();

	switch (NeedMoreAircraft(surplus)) {
	case YATT_NMV_YES:
	  if(!GetAllowedVehCount(YATT_CMI_AIR)) break;	// Limit reached

	  c_engprice = AIEngine.GetPrice(c_engsel);
	  c_srvstate = YATT_SST_BUYING;

	  return YATT_MGR_BUYVEHICLE;

	case YATT_NMV_UPGRADE:
	  c_engprice = AIEngine.GetPrice(c_engsel);
	  return YATT_MGR_UPGRADEVEHICLE;

	case YATT_NMV_UPGRADEFLEET:
	  c_engprice = AIEngine.GetPrice(c_engsel);
	  return YATT_MGR_UPGRADEFLEET;
	}

	if(NeedLessAircraft(surplus)) {
	  c_srvstate = YATT_SST_SELLING;

	  return YATT_MGR_SELLVEHICLE;
	}

	if(c_srvstate != YATT_SST_STEADY) {
	  local	date = AIDate.GetCurrentDate();

	  if(date - c_datebought > 180 && date - c_datesold > 180)
	    c_srvstate = YATT_SST_STEADY;
	}

	return YATT_MGR_NONE;
}
	function cATTAirRoute::NeedMoreAircraft(surplus)
//	------------------------------------------------
//	Check if we need to buy a new aircraft
//	If surplus < c_psurplus, surplus going down, don't buy
//	Returns		YATT_NMV_ code
{
	c_engsel = SelectAircraft(c_cargo);

	if(!CanUseSelectedAircraft()) return YATT_NMV_NO;
	if(GetVehicleCount() < 2) return YATT_NMV_YES;	// No vehicles

	local	vehid = c_vehids[0];
	local	vehcap = AIEngine.GetCapacity(c_engsel);

	if(c_waiting >= vehcap &&
	   AIVehicle.GetEngineType(vehid) != c_engsel &&
	   AIVehicle.GetAge(vehid) > 365 * 3 &&
	   AIDate.GetCurrentDate() - c_datebought >= 365) return YATT_NMV_UPGRADE;

	if(c_stops[c_waitidx].GetCargoRating(c_cargo) > 50) {
	  if(surplus < c_psurplus) {
	    c_psurplus -= (c_psurplus - surplus) / 6;
	    return YATT_NMV_NO;
	  }

	  c_psurplus = surplus;

	  if(surplus < vehcap) return YATT_NMV_NO;
	}

	if(c_waiting < vehcap / 2) return YATT_NMV_NO;

	return (c_stops[0].HasLandingSlot() && c_stops[1].HasLandingSlot()) ? 
		YATT_NMV_YES : YATT_NMV_NO;
}
	function cATTAirRoute::NeedLessAircraft(surplus)
//	------------------------------------------------
//	surplus		-ve number indicating overcapacity
//	Returns		true if we need to sell a vehicle
{
	if(!GetVehicleCount()) return false;		// No vehicles

	if(c_waiting > 0 || c_stops[c_waitidx].GetCargoRating(c_cargo) < 67) 
	  return false;

	local	vehid = c_vehids[0];
	local	vehcap = AIVehicle.GetCapacity(vehid,c_cargo);
	local	days = AIDate.GetCurrentDate() - c_datesold;

//	Overcapacity > 6x and more than 30 days since we sold a vehicle, sell
	if(-surplus > vehcap * 6 && days > 30) return true;

//	Less than 90 days sice vehicle sold a vehicle or overcapacity < 3x, keep
	if(days < 90 || -surplus < vehcap * 3) return false;

	local age = AIVehicle.GetAge(vehid);		// Oldest vehicle age (days)

//	Older than 3 years and overcapacity > 5x, sell
	if(age > 3 * 365 && -surplus > vehcap * 5) return true;

	return (age > AIVehicle.GetMaxAge(vehid) / 2);
}
	function cATTAirRoute::WillOverlapAirport(tile,wid,hig,covrad)
//	--------------------------------------------------------------
//	Check if an airport built on tile would overlap catchment of
//	one of our airports already built. Larger covrad ignores smaller ones
//	tile		Proposed airport tile
//	wid,hig		Airport dimension
//	covrad		Airport catchment radius
//	Returns		True if overlap
{
	local	tile1 = GetRectTileIndex(tile,wid,hig);
	local	list = cATTTileList();

	list.AddArea(tile,tile1,20);

	local	elist = ExcludeStationTiles(list,true,covrad);

	if(elist == null) return false;

	return (elist.HasItem(tile) || elist.HasItem(tile1) ||
	        elist.HasItem(tile+s_offset[DIR_I_SW]*(wid-1)) ||
	        elist.HasItem(tile+s_offset[DIR_I_SE]*(hig-1)));
}
	function cATTAirRoute::ExcludeStationTiles(list,intown,covrad)
//	--------------------------------------------------------------
//	list		List of tiles to scan for stations
//	intown		Tiles are in town, try not to overlap our airports
//	covrad		Coverage radius of proposed airport
//			Larger covrad ignores smaller ones
//	Returns		The list is modified by removing
//			tiles that are stations, plus the list of removed tiles
//			is returned as well
{
	local	slist = AITileList();

	slist.AddList(list);

	slist.Valuate(AITile.IsStationTile);
	slist.KeepValue(1);
	if(slist.IsEmpty()) return null;

	local	elist = cATTTileList();

	foreach (tile,v in slist) {
	  if(intown && AICompany.IsMine(AITile.GetOwner(tile)))	{// Our station
	    local id = AIStation.GetStationID(tile);

	    if(AIStation.HasStationType(id,AIStation.STATION_AIRPORT)) {
	      local aptype = AIAirport.GetAirportType(tile);
	      local apcov = AIAirport.GetAirportCoverageRadius(aptype);

	      if(covrad <= apcov)
		elist.AddSquare(tile,apcov+covrad-2);
	      else
		elist.AddSquare(tile,(apcov+covrad)/2);
	    }
	  }
	}

//	Remove tiles that can be overlapped by airports

	elist.Valuate(AITile.IsWaterTile);	// Can overlap water
	elist.KeepValue(0);

	list.RemoveList(elist);

	return elist;
}
	function cATTAirRoute::CanUseSelectedAircraft()
//	-----------------------------------------------
//	Make sure the current best plane can service the route airports
{
	if((IsSmallAirportType(c_stops[0].c_aptype) ||
	    IsSmallAirportType(c_stops[1].c_aptype)) &&
	   (!c_engsel ||
	    AIEngine.GetPlaneType(c_engsel) != AIAirport.PT_SMALL_PLANE)) {

	  local	t = FindBestVehicle(YATT_CMI_AIR,c_cargo,true);

	  if(t == null) return false;
	  
	  c_engsel = t.engid;
	}

	return true;
}
	function cATTAirRoute::GetNewPlanePrice()
//	-----------------------------------------
{
	c_engsel = SelectAircraft(c_cargo);

	if(!CanUseSelectedAircraft()) return null;

	return AIEngine.GetPrice(c_engsel);
}
	function cATTAirRoute::IsSmallAirportType(aptype)
//	-------------------------------------------------
{
	return (aptype == AIAirport.AT_SMALL || aptype == AIAirport.AT_COMMUTER);
}
