	class cATTAStar extends cATTBase
//	--------------------------------
{
//	cQueue = import("queue.binary_heap","",1);
//	cQueue = import("queue.fibonacci_heap","",2);

	c_classname = null;

	c_cbthis = null;			// "this" for callbacks

	c_cbCost = null;			// Cost callback
	c_cbEstimate = null;			// Estimate callback
	c_cbNeighbors = null;			// Neighbors callback
	c_cbDirection = null;			// Direction callback

	c_open = null;
	c_closed = null;
	c_goals = null;

	constructor(cbthis,cbCost,cbEstimate,cbNeighbors,cbDirection)
//	-------------------------------------------------------------
	{
	  c_classname = "cATTAStar";

	  c_cbthis = cbthis;

	  c_cbCost = cbCost;
	  c_cbEstimate = cbEstimate;
	  c_cbNeighbors = cbNeighbors;
	  c_cbDirection = cbDirection;
	}
}
	function cATTAStar::Initialize(start,goals,ignore=null)
//	-------------------------------------------------------
//	start		An array of [tile,dir] pairs
//			tile	Possible start tile
//			dir	DIR_M_ code, restricts direction from
//				start tile. If DIR_M_UNKNOWN, no restriction.
//	goals		An array of tiles or [tile,prev_tile] pairs
//			tile		Possible goal tile
//			prev_tile	Restricts path thru this tile to tile
//	ignore		An array of tiles to ignore
{
//	c_open = cQueue();
	c_open = cATTPQueue();
	c_closed = cATTList();

	foreach (node in start) {
	  if(typeof(node) == "array") {
	    local path = Path(null,node[0],node[1]);

	    c_open.Insert(path,
		          path.cost + c_cbEstimate(c_cbthis,node[0],node[1],goals));
	  }
	  else
	    c_open.Insert(node,node.cost);
	}

	c_goals = goals;

	if(ignore) {
	  foreach (tile in ignore)
	    c_closed.AddItem(tile, ~0);
	}
}
	function cATTAStar::Path(ppath,tile,dir)
//	----------------------------------------
//	ppath		Previous path table
//	tile		Tile id
//	dir		Tile direction (DIR_M_ index)
//	Returns path table with the following fields:
//			prev		Previous path table
//			tile		Tile
//			dir		Direction (DIR_M_ index)
//			dirb		Direction (DIR_B_ code)
//			cost		Cost up to this point
//			length		Number of tiles up to this point
{
	local	path = {prev = ppath, tile = tile, dir = dir};

	path.dirb <- s_mapdirb[dir];
	path.cost <- c_cbCost(c_cbthis,ppath,tile,dir);

	path.length <- (ppath == null) ? 0 :
		       ppath.length + AIMap.DistanceManhattan(ppath.tile,tile);

	return path;
}
	function cATTAStar::Cleanup()
//	-----------------------------
{
	c_open = null;
	c_closed = null;
	c_goals = null;
}
	function cATTAStar::FindPath(iterations)
//	----------------------------------------
//	Returns		null if not found or
//			false if iterations reached or
//			path table
{
	while (c_open.Count() > 0 && (iterations == -1 || iterations-- > 0)) {
	  local path = c_open.Pop();
	  local tile = path.tile;

	  if(c_closed.HasItem(tile)) {
	    local dirb = c_closed.GetValue(tile);	// Tile direction bits

	    if((dirb & path.dirb) != 0) continue;	// Already seen

//	    Scan existing path to see if it is trying to backtrack on itself

	    local scanpath = path.prev;
	    local mismatch = false;

	    while (scanpath != null) {
	      if(scanpath.tile == tile) {
		if(!c_cbDirection(c_cbthis,tile,scanpath.dir,path.dir)) {
		  mismatch = true;
		  break;
		}
	      }

	      scanpath = scanpath.prev;
	    }

	    if(mismatch) continue;

	    c_closed.SetValue(tile,dirb | path.dirb);	// Seen new direction
	  }
	  else
	    c_closed.AddItem(tile,path.dirb);

//	  Check if we found a goal

	  foreach (goal in c_goals) {
	    if(typeof(goal) == "array") {
//	      if(tile == goal[0]) {
//		local neighbors = c_cbNeighbors(c_cbthis,path,tile);

//		foreach (node in neighbors) {
//		  if(node[0] == goal[1]) {
//		    Cleanup();
//		    return path;
//		  }
//		}
	      if(tile == goal[0] && path.prev.tile == goal[1]) {
		Cleanup();
		return path;
	      }
	    }
	    else if(goal == tile) {
	      Cleanup();
	      return path;
	    }
	  }

//	  Scan all neighbors

	  local neighbors = c_cbNeighbors(c_cbthis,path,tile);

	  foreach (node in neighbors) {
	    if((c_closed.GetValue(node[0]) & s_mapdirb[node[1]]) != 0) continue;

	    local npath = Path(path,node[0],node[1]);

//	    c_open.Insert(npath,
//		   npath.cost + c_cbEstimate(c_cbthis,node[0],node[1],c_goals));

	    local est = npath.cost + c_cbEstimate(c_cbthis,node[0],node[1],c_goals);
//	    AISign.BuildSign(npath.tile,est.tostring());

	    c_open.Insert(npath,est);
	  }
	}

	if(c_open.Count() > 0) return false;

	Cleanup();

	return null;
}
