	function cATTBase::Direction(offs)
//	----------------------------------
//	Return DIR_M_ index from map offset
{
	foreach (i,mapoffs in s_mapoffs)
	  if(offs == mapoffs) return i;

	Error("Invalid offset: " + offs);
}
	function cATTBase::BuiltSlope(tile,ptile)
//	-----------------------------------------
//	Return code that shows slope after building road/rail
{
	local	code;
	local	slope = AITile.GetSlope(tile);

	if(slope == AITile.SLOPE_FLAT) return ATT_SLP_FLAT;
	if(slope > AITile.SLOPE_STEEP) return ATT_SLP_SLOPED_RAISED;

	local	pslope = AITile.GetSlope(ptile);

	if(pslope > AITile.SLOPE_STEEP) pslope -= AITile.SLOPE_STEEP;

	if(pslope == AITile.SLOPE_FLAT ||
	   AITile.GetMaxHeight(tile) == AITile.GetMaxHeight(ptile)) {

	  code = s_built[pslope][slope];

	  if(!code) Error("Builtslope code wrong: "+pslope+" "+slope+
			  " "+ptile+" "+tile);
	}
	else {
	  code = s_stpbuilt[pslope][slope];		// Steep

	  if(!code) Error("Builtslope steep code wrong: "+pslope+" "+slope+
			  " "+ptile+" "+tile);
	}

	return code;
}
	function cATTBase::GetTiles(tile,radius)
//	----------------------------------------
{
	local	tiles = cATTTileList();

	tiles.AddSquare(tile,radius);
	return tiles;
}
	function cATTBase::GetTownTiles(townid,radius=20)
//	-------------------------------------------------
//	Return tiles that are within a town's influence, whatever that is
{
	local	tiles = GetTiles(AITown.GetLocation(townid),radius);

	tiles.Valuate(AITile.IsWithinTownInfluence,townid);
	tiles.KeepValue(1);

	return tiles;
}
	function cATTBase::GetTownAuthorityTiles(townid)
//	------------------------------------------------
//	Return list of tiles that are within a town's authority area and
//	to which the town is the closest.
{
	local	tiles = GetTiles(AITown.GetLocation(townid),s_dist_local_authority);

	tiles.Valuate(AITile.GetClosestTown);
	tiles.KeepValue(townid);

	return tiles;
}
	function cATTBase::LevelArea(tile,wid,hig,coast=false)
//	------------------------------------------------------
//	tile		Tile at North corner
//	wid		Length along X axis
//	hig		Length along Y axis
//	coast		if false, will not level if area contains water or coast tiles
//	Returns		Cost of leveling or -1 if can't do it
{
	if(!AIMap.IsValidTile(tile+AIMap.GetTileIndex(wid,hig))) return -1;

	local	list;

	if(!coast) {
	  list = cATTTileList();
	  list.AddRectangle(tile+s_offset[DIR_I_N],
			    tile+AIMap.GetTileIndex(wid,hig));

	  foreach (t,v in list)
	    if(AITile.IsWaterTile(t) || AITile.IsCoastTile(t)) return -1;
	}

//	Calculate best elevation

	local	minhig,maxhig,lvldiff;

	list = GetTileCornerHeights(tile,wid,hig);

	list.Sort(AIList.SORT_BY_VALUE,false);
	if((maxhig = list.GetValue(list.Begin())) < 1) maxhig = 1;

	list.Sort(AIList.SORT_BY_VALUE,true);
	if((minhig = list.GetValue(list.Begin())) < 1) minhig = 1;

	if((lvldiff = maxhig - minhig) > 2) return -1

	local	lvlhig;

	switch (lvldiff) {			// Elevation difference
	case 0:					// All at same height
	  lvlhig = maxhig;
	  break;

	case 1:
	  if((lvlhig = GetListAverage(list)) == 0) lvlhig = 1;
	  break;

	case 2:
	  lvlhig = maxhig - 1;
	  break;
	}

	local alist = AITileList();

	alist.AddRectangle(tile,tile+AIMap.GetTileIndex(wid-1,hig-1));

	if(lvldiff == 1) {
	  local test = AITestMode();

	  if(!LevelTileList(alist,lvlhig)) 
	    lvlhig = (lvlhig == maxhig) ? minhig : maxhig;
	}

	local	acc = AIAccounting();

	if(!LevelTileList(alist,lvlhig)) return -1;

	return acc.GetCosts();
}
	function cATTBase::LevelTileList(list,lvlhig)
//	---------------------------------------------
//	Level list of tiles by raising or lower one elevation step
//	lvlhig		Height to raise or lower North corner
{
	local	slope = [AITile.SLOPE_W,AITile.SLOPE_S,
			 AITile.SLOPE_E,AITile.SLOPE_N];
	local	corner = [AITile.CORNER_W,AITile.CORNER_S,
			  AITile.CORNER_E,AITile.CORNER_N];

	foreach (t,h in list) {
	  if(AITile.GetMaxHeight(t) == lvlhig) continue;

	  local raise=0,lower=0;

	  foreach (i,c in corner) {
	    local hig = AITile.GetCornerHeight(t,c);

	    if(hig > lvlhig)
	      lower = lower | slope[i];
	    else if(hig < lvlhig)
	      raise = AITile.SLOPE_N;
	  }

	  if(raise) {
	    while (!AITile.RaiseTile(t,raise)) {
	      if(AIError.GetLastError() != AIError.ERR_NOT_ENOUGH_CASH)
	        return false;

	      WaitForFunds();
	    }
	  }

	  if(lower) {
	    while (!AITile.LowerTile(t,lower)) {
	      if(AIError.GetLastError() != AIError.ERR_NOT_ENOUGH_CASH)
	        return false;

	      WaitForFunds();
	    }
	  }
	}

	return true;
}
	function cATTBase::CanBuildOnTile(tile,raise=false,exclude=null)
//	----------------------------------------------------------------
//	Check if town rating is high enough to build on tile
//	raise		True to raise town rating to POOR if lower
//	exclude		List of tiles not to build trees on
//	Returns		1/0 = true/false
{
	local	townid = AITile.GetClosestTown(tile);

	if(AITown.GetDistanceManhattanToTile(townid,tile) > s_dist_local_authority)
	  return 1;	// Outside town authority range

	if(!IsTownRatingLow(townid)) return 1;		// Not low
	if(!raise) return 0;				// Low, but no raise

	local	list = GetTownAuthorityTiles(townid);

	if(exclude) list.RemoveList(exclude);		// Do not plant on these

	list.Valuate(AITile.IsBuildable);
	list.KeepValue(1);

	list.Valuate(AITile.IsFarmTile);
	list.Sort(AIList.SORT_BY_VALUE,true);		// Use farm tiles last

	foreach (tile,v in list) {
	  while (1) {
	    if(!AITile.PlantTree(tile)) {
	      if(AIError.GetLastError() != AIError.ERR_NOT_ENOUGH_CASH) break;

	      Sleep(20);
	    }
	  }

	  if(!IsTownRatingLow(townid)) return 1;	// Not low
	}

	return 0;
}
	function cATTBase::DemolishTile(tile)
//	-------------------------------------
{
	if(AIBridge.IsBridgeTile(tile) || AITunnel.IsTunnelTile(tile)) return false;

	while (!AITile.DemolishTile(tile)) {
	  switch (AIError.GetLastError()) {
	  case AIError.ERR_NOT_ENOUGH_CASH:
	    Sleep(20);
	    break;

	  default:
	    return false;
	  }
	}

	return true;
}
	function cATTBase::HandleVehicleInTheWay(tile1,tile2)
//	-----------------------------------------------------
//	Could not build road segment, because vehicle was in the way
//	Attempt to build it for a few days, then quit
{
	local	gHandleTraffic = HandleTraffic();

	while (true) {
	  if(resume gHandleTraffic == YATT_E_TRAFFIC) return YATT_E_TRAFFIC;

	  if(AIRoad.BuildRoad(tile1,tile2)) break;
	  if(AIError.GetLastError() != AIError.ERR_VEHICLE_IN_THE_WAY)
	    return YATT_E_OTTD;
	}

	return YATT_E_SUCCESS;
}
	function cATTBase::HandleTraffic()
//	----------------------------------
{
	local	days = 0;
	local	start = AIDate.GetCurrentDate();


	while (true) {
	  local date = AIDate.GetCurrentDate();

	  if(date > start) {
	    days += (date - start);

	    if(days >= 8) break;

	    start = date;

	    local exec = AIExecMode();
	    s_g.att.CheckGameTasks();
	    exec = null;
	  }

	  Sleep(10);

	  yield YATT_E_SUCCESS;
	}

	Warning("Too much traffic detected");

	return YATT_E_TRAFFIC;
}
	function cATTBase::GetAirportCount(townid)
//	------------------------------------------
//	Return the airport count at a town
{
	local	tlist = GetTownAuthorityTiles(townid);

	tlist.Valuate(AIAirport.IsAirportTile);
	tlist.KeepValue(1);

	local apid = null;

	foreach (tile,v in tlist) {
	  local id = AIStation.GetStationID(tile);

	  if(AIAirport.IsAirportTile(tile)) {
	    if(apid == null)
	      apid = id;
	    else if(id != apid)			// Not same airport
	      return 2;
	  }
	}

	return (apid == null) ? 0 : 1;
}
