	function cATTBase::FindBestVehicle(method,cargo,smallplane=false)
//	-----------------------------------------------------------------
//	Return the best vehicle in a category to carry the cargo
//	Evaluates net profit over 100 tile route over life of vehicle
//	method		YATT_CMI_ code show type of route
//	cargo		Vehicle needs to carry this cargo
//	smallplane	If true, find only small planes
//	Returns		null or table with the following fields:
//			engid		engine selected
//			name		engine name
//			cargo		cargo id
//			method		YATT_CMI_ code (connection method)
//			tilday		speed in tiles per day
//			capacity	cargo capacity
//			maxage		engine maximum age in years
//			price		engine price
//			cost		price + running_cost * maxage
{
	local	methveh = [AIVehicle.VT_RAIL,AIVehicle.VT_ROAD,
			   AIVehicle.VT_AIR,AIVehicle.VT_WATER,
			   AIVehicle.VT_AIR];

	if(!s_cfg[s_cfgconn[method]]) return null;
	if(AIGameSettings.IsDisabledVehicleType(methveh[method])) return null;

	local	list = AIEngineList(methveh[method]);

	switch (method) {
	case YATT_CMI_ROAD:
	  list.Valuate(AIEngine.GetRoadType);
	  list.KeepValue(AIRoad.ROADTYPE_ROAD);
	  break;

	case YATT_CMI_RAIL:
	  return null;					// Temp

	case YATT_CMI_AIR:
	  list.Valuate(AIEngine.GetPlaneType);
	  list.RemoveValue(AIAirport.PT_HELICOPTER);

	  if(smallplane)
	    list.KeepValue(AIAirport.PT_SMALL_PLANE);
//	  else if(AIAirport.IsValidAirportType(AIAirport.AT_LARGE))
//	    list.KeepValue(AIAirport.PT_BIG_PLANE);

	  break;

	case YATT_CMI_HELI:
	  return null;					// Temp

	  list.Valuate(AIEngine.GetPlaneType);
	  list.KeepValue(AIAirport.PT_HELICOPTER);
	  break;
	}

//	list.Valuate(AIEngine.IsArticulated);
//	list.KeepValue(0);

	list.Valuate(AIEngine.CanRefitCargo,cargo);
	list.KeepValue(1);

	list.Valuate(AIEngine.IsBuildable);
	list.KeepValue(1);

	if(!list.Count()) return null;

//	Estimate yearly profit for each vehicle, return best one

	local	net,score,maxscore=0,ret={},vt={};

	vt.cargo <- cargo;
	vt.crgname <- AICargo.GetCargoLabel(cargo);
	vt.method <- method;

	list.Valuate(AIEngine.GetMaxSpeed);

	foreach (engid,speed in list) {
	  vt.tilday <- speed / 27.0;
	  vt.capacity <- AIEngine.GetCapacity(engid);
	  vt.maxage <- AIEngine.GetMaxAge(engid) / 366.0
	  vt.price <- AIEngine.GetPrice(engid);
	  vt.cost <- vt.price + AIEngine.GetRunningCost(engid) * vt.maxage;

	  net = EstimateProfit(vt,100);
	  score = net * (AIEngine.GetReliability(engid) / 100.0);

	  if(speed > 85) score *= speed;	// For station rating

	  if(score > maxscore) {
	    maxscore = score;

	    ret = clone vt;
	    ret.engid <- engid;
	    ret.name <- AIEngine.GetName(engid);
	  }
	}

	return (maxscore) ? ret : null;
}
	function cATTBase::GetBestVehicle(method,cargo)
//	-----------------------------------------------
//	Return the best vehicle found for a method and cargo
{
	local	vt = s_g.vehicles[method][cargo];

	if(!vt) return null;

	vt.price = AIEngine.GetPrice(vt.engid);		// Update price

	return clone vt;
}
	function cATTBase::GetMaxDistForVehicle(method,cargo,days=180)
//	--------------------------------------------------------------
//	Calculate max distance the best vehicle can travel in given days
//	method		Connection method (YATT_CMI_ code)
//	cargo		Cargo to carry
{
	local	t = s_g.vehicles[method][cargo];

	if(!t) return 0;

	local	dist = (t.tilday * days).tointeger();

	if(t.method == YATT_CMI_AIR) {
	  local speed = AIGameSettings.GetValue("plane_speed");

	  dist = dist * speed / 4;
	}

	return dist;
}
	function cATTBase::EstimateProfit(vt,dist)
//	------------------------------------------
//	Estimate net profit for vehicle
//	vt	table describing vehicle, created by GetBestVehicle
//	dist	distance to take vt.cargo
//	Returns		Adds new field "trips" to vt
//			Adds new field "days" to vt
//			Net profit for vehicle per year
{
	local	income,roundtrip;
	local	effect = AICargo.GetTownEffect(vt.cargo);
	local	intown = (effect == AICargo.TE_PASSENGERS || effect == AICargo.TE_MAIL);
	local	totown = (effect == AICargo.TE_GOODS || effect == AICargo.TE_FOOD);

	vt.days <- (dist / vt.tilday).tointeger();

	roundtrip = vt.days * 2;
	roundtrip += 8;				// Load/Unload(2/2) + 2*depot(2)

	switch (vt.method) {
	case YATT_CMI_ROAD:
	  if(intown)
	    roundtrip += 10;			// Load/Unload(2/2) + 2*city roads(3)
	  else if(totown)
	    roundtrip += 3;			// City roads(3)

	  break;

	case YATT_CMI_AIR:
	  if(intown) roundtrip += 4;		// Load/Unload(2/2)

	  roundtrip += 12;			// 2*airport delay(6)
	  break;
	}

	if(roundtrip > 360) return 0;

	vt.trips <- 365.0 / roundtrip;		// Trips per year

	income = AICargo.GetCargoIncome(vt.cargo,dist,vt.days) * 
		 			vt.capacity * vt.trips;

	if(s_cargo2x[vt.cargo]) income *= 2;		// Earns both ways

	return ((income * vt.maxage - vt.cost) / vt.maxage).tointeger();
}
	function cATTBase::GetAllowedVehCount(vehtype)
//	-----------------------------------------------
//	Return number of vehicles we can still buy in category
//	vehtype		YATT_CMI code
{
	local names = ["max_trains","max_roadveh","max_aircraft","max_ships",
		       "max_aircraft"];
	local types = [AIVehicle.VT_RAIL,AIVehicle.VT_ROAD,
		       AIVehicle.VT_AIR,AIVehicle.VT_WATER,AIVehicle.VT_AIR];

	local max = AIGameSettings.GetValue(names[vehtype]);
	local list = AIVehicleList();

	list.Valuate(AIVehicle.GetVehicleType);
	list.KeepValue(types[vehtype]);

	return max - list.Count();
}
