	class cATTRoadMgr extends cATTManager
//	-------------------------------------
{
	c_classname = null;

	constructor()
//	-------------
	{
	  ::cATTManager.constructor(YATT_CMI_ROAD);

	  c_classname = "cATTRoadMgr";
	}
}
	function cATTRoadMgr::EstimateAvgRoadRouteCost()
//	------------------------------------------------
//	Estimate minimum cost for building an average length road route
//	Uses passenger service for calculation
{
	local	engid = cATTRoadRoute.SelectRoadVehicle(s_crgpassengers);

	if(engid == null) return null;

	local	dist = (s_cdist.max[YATT_CMI_ROAD] * 1.25).tointeger();
	local	cost = AIEngine.GetPrice(engid) * 5;

	cost += AIRoad.GetBuildCost(AIRoad.ROADTYPE_ROAD,AIRoad.BT_BUS_STOP) * 2;
	cost += AIRoad.GetBuildCost(AIRoad.ROADTYPE_ROAD,AIRoad.BT_DEPOT) * 2;
	cost += AIRoad.GetBuildCost(AIRoad.ROADTYPE_ROAD,AIRoad.BT_ROAD) * dist * 2;

	cost += cATTRoadRoute.EstimateRoadLoopCost() * 2;

	return cost;
}
	function cATTRoadMgr::EvaluateRoadRoute(t)
//	------------------------------------------
//	t		Table with route info, prepared by GetSubsidyInfo
//			or FindService
//	Returns		YATT_E_ code or table with the following fields:
//			route		Surveyed route
//			cost		Total cost to build
{
//	If profit seems to be low, don't build route unless we are at
//	max route length and we still have vehicle room

	if(t.vt.net * 4 < t.vt.price &&
	   (s_cdist.max[YATT_CMI_ROAD] < s_cdist.abs[YATT_CMI_ROAD] ||
	    GetAllowedVehCount(YATT_CMI_ROAD) < 10))
	  return YATT_E_NOTPROFITABLE;

//	if(t.vehicles < 5) return YATT_E_NOTPROFITABLE;

	local	route,cost;

	route = cATTRoadRoute();
	route.InitFromTable(t);

	if((cost = route.EstimateRoadRouteCost()) < 0) return cost;	// Error

	return {route = route, cost = cost};
}
	function cATTRoadMgr::StartRoadService(route)
//	---------------------------------------------
//	Returns		true if service started ok
{
	local	errcode,success = route.BuildRoadRoute();

	if(success) {
	  Warning(route.GetRouteName() + ": Built road route");

	  switch (errcode = BuyRoadVehicle(route)) {	// First
	  case YATT_E_SUCCESS:
	  case YATT_E_NOFUNDS:
	    for (local i=1; i <= 2; i++)
	      BuyRoadVehicle(route);

	    break;

	  default:
	    Error("Cannot buy first vehicle: "+GetErrorStr(errcode));
	    success = false;
	  }
	}

	if(success) {
	  AddRoute(route);
	  AddTask(YATT_TSK_CHECKROUTE,30,route);
	  route.SetServiceState(cATTRoute.YATT_SST_BUYING);

	  s_g.att.SetStartupReserve(route,5-route.GetVehicleCount());
	}
	else
	  route.DemolishRoadRoute();

	return success;
}
	function cATTRoadMgr::StopService(route,unload)
//	-----------------------------------------------
//	Sell all vehicles and stop service
{
	if(route.c_srvstate != cATTRoute.YATT_SST_STOPPING) {
	  Warning(route.GetRouteName() + ": Stopping Service, unload="+unload);

	  route.SetServiceState(cATTRoute.YATT_SST_STOPPING);

	  foreach (i,vehid in route.c_vehids)
	    SendRoadVehicleToDepot(route,vehid,YATT_ORD_SELL,unload);
	}
}
	function cATTRoadMgr::OnLastVehicleSold(route)
//	----------------------------------------------
//	The last vehicle has been sold, demolish stations and delete route
{
	route.DemolishRoadRoute();

	RemoveRoute(route);		// From manager xref tables
}
	function cATTRoadMgr::AddTask(task,days,p1=null,p2=null)
//	--------------------------------------------------------
{
	return ::cATTBase.AddTask(this,"DoTask",task,days,p1,p2);
}
	function cATTRoadMgr::DoTask(t)
//	-------------------------------
{
	this = t.obj;

	switch (t.task) {
	case YATT_TSK_CHECKROUTE:			// p1 = route
	  if(t.p1.c_srvstate != cATTRoute.YATT_SST_STOPPING) CheckRoadRoute(t.p1);
	  break;

	default:
	  Error("Task not handled: "+t.task);
	}
}
	function cATTRoadMgr::DoDepotOrder(t)
//	-------------------------------------
{
	this = t.obj;

	switch (t.order) {
	case YATT_ORD_SELL:
	case YATT_ORD_UPGRADE:
	  delete c_vehs[t.vehid];			// Remove from xref
	  t.route.SellVehicle(t.vehid);

	  if(t.route.c_srvstate == cATTRoute.YATT_SST_STOPPING) {
	    if(!t.route.GetVehicleCount()) OnLastVehicleSold(t.route);
	  }
	  else if(t.order == YATT_ORD_UPGRADE)
	    BuyRoadVehicle(t.route,true);

	  break;
	}

	delete s_g.depord[t.vehid];
}
	function cATTRoadMgr::DoSavedTask(t)
//	------------------------------------
//	Money for a task has been saved, execute task
{
	this = t.obj;

	switch (t.task) {
	case YATT_TSK_BUYVEHICLE:	// p1=route p2=renew
	  t.p1.c_saving = false;

	  if(t.p1.c_srvstate != cATTRoute.YATT_SST_STOPPING)
	    BuyRoadVehicle(t.p1,t.p2,true);

	  break;
	}
}
	function cATTRoadMgr::CheckRoadRoute(route)
//	-------------------------------------------
{
	switch (route.CheckRoadRouteProduction()) {
	case YATT_MGR_BUYVEHICLE:
	  BuyRoadVehicle(route);
	  break;

	case YATT_MGR_SELLVEHICLE:
	  SendRoadVehicleToDepot(route,route.c_vehids[0],YATT_ORD_SELL);
	  break;

	case YATT_MGR_UPGRADEVEHICLE:
	  SendRoadVehicleToDepot(route,route.c_vehids[0],YATT_ORD_UPGRADE);
	  break;

	case YATT_MGR_UPGRADEFLEET:
	  foreach (i,vehid in route.c_vehids) {
	    if(AIVehicle.GetAge(vehid) > 365 * 3)
	      SendRoadVehicleToDepot(route,vehid,YATT_ORD_UPGRADE);
	  }
	  break;
	}

	local	days = (route.c_srvstate == cATTRoute.YATT_SST_STEADY) ? 45 : 15;

	AddTask(YATT_TSK_CHECKROUTE,days,route);
}
	function cATTRoadMgr::BuyRoadVehicle(route,renew=false,saved=false)
//	-------------------------------------------------------------------
//	Try and buy vehicle now. If not enough funds, set saving mode
//	and reschedule buying vehicle.
//	route		Route to buy vehicle for
//	renew		True if a vehicle is being renewed, save for it if needed
//	saved		Using saved funds, don't test global save
//	Returns		YATT_E_ code
{
	local	errcode;

	if(saved || !IsSaving()) {			// Global saving
	  switch (errcode = route.BuildRoadVehicle(renew)) {
	  case YATT_E_SUCCESS:
	    AddVehicle(route);				// Last added
	    return YATT_E_SUCCESS;

	  case YATT_E_OTTD:
	    switch (AIError.GetLastError()) {
	    case AIVehicle.ERR_VEHICLE_TOO_MANY:
	      return YATT_E_VEHLIMIT;

	    case AIError.ERR_NOT_ENOUGH_CASH:
	      break;

	    default:
	      Error("Could not buy road vehicle: "+AIError.GetLastErrorString());
	      return YATT_E_OTTD;
	    }
//	    Fall thru

	  case YATT_E_NOFUNDS:
	    SaveFunds(this,"DoSavedTask",YATT_TSK_BUYVEHICLE,route.c_engprice,saved,
		      route,renew);
	    route.c_saving = true;			// Route saving
	    return YATT_E_NOFUNDS;

	  default:
	    Error("Could not buy road vehicle: "+errcode);
	    return errcode;
	  }
	}
	else if(renew) {		// Replacing a vehicle, save for it
	  SaveFunds(this,"DoSavedTask",YATT_TSK_BUYVEHICLE,route.c_engprice,
		    true,route,renew);
	  return YATT_E_NOFUNDS;
	}
	else if(!route.c_saving) {
	  SaveFunds(this,"DoSavedTask",YATT_TSK_BUYVEHICLE,route.c_engprice,
		    false,route,renew);
	  route.c_saving = true;
	  return YATT_E_NOFUNDS;
	}

	return YATT_E_SAVING;
}
	function cATTRoadMgr::SendRoadVehicleToDepot(route,vehid,order,unload=true)
//	---------------------------------------------------------------------------
//	Send oldest vehicle to depot to be sold
{
	local	t = {obj=this, func = "DoDepotOrder",
		     route=route, vehid=vehid, order=order};

	s_g.depord[vehid] <- t;

	route.SendVehicleToDepot(vehid,unload);
}
	function cATTRoadMgr::VehicleWaitingInDepot(vehid)
//	--------------------------------------------------
//	Vehicle in depot, must be old if it had no task
{
	local	route = c_vehs[vehid];

	delete c_vehs[vehid];				// Remove from xref
	route.SellVehicle(vehid,true);			// Sell for renew

	BuyRoadVehicle(route,true);
}
	function cATTRoadMgr::DoYearEnd()
//	---------------------------------
//	Check for unprofitable routes
//	Must be called after yearend report that sums profit
{
	foreach (i,route in c_routes)
	  if(route.c_profit < 0 && route.c_profit2 < 0) StopService(route,false);
}
	function cATTRoadMgr::VehicleCrashed(vehid,tile,reason)
//	-------------------------------------------------------
{
	if(vehid in c_vehs) {
	  local route = c_vehs[vehid];

	  delete c_vehs[vehid];

	  if(vehid in s_g.depord) delete s_g.depord[vehid];

	  route.RemoveVehicle(vehid);

	  if(route.GetVehicleCount() == 0) BuyRoadVehicle(route,true);
	}
}
