	class cATTRoadPF extends cATTBase
//	---------------------------------
{
	c_classname = null;

	c_astar = null;				// AStar class

	c_maxlen = null;			// Max road length

	c_goal = null;				// Goal tile to use for detour
	c_detour = null;			// Max tiles over min distance

	c_ctile = null;				// Tile cost
	c_croad = null;				// New road tile cost
	c_cthrustop = null;			// Thru station cost
	c_cturn = null;				// Turn cost
	c_cslope = null;			// Slope cost
	c_craise = null;			// Foundation cost
	c_cfields = null;			// Fields clearing cost
	c_ccoast = null;			// Coast tile
	c_ctree = null;				// Tree cutting cost

	constructor()
//	-------------
	{
	  c_classname = "cATTRoadPF";

	  c_astar = cATTAStar(this,cbCost,cbEstimate,cbNeighbors,cbDirection);

	  c_maxlen = s_mapsizex + s_mapsizey;

	  c_goal = null;
	  c_detour = null;

	  c_ctile = 100;
	  c_croad = 20;
	  c_cthrustop = 250;
	  c_cturn = 80;
	  c_cslope = 200;
	  c_craise = 10;
	  c_cfields = 5;
	  c_ccoast = 300;
	  c_ctree = 1;
	}
}
	function cATTRoadPF::Initialize(start,goals,ignore=null)
//	--------------------------------------------------------
//	start		An array of [tile,dir] pairs
//			tile	Possible start tile
//			dir	DIR_M_ code, restricts direction from
//				start tile. If DIR_M_UNKNOWN, no restriction.
//	goals		An array of tiles or [tile,prev_tile] pairs
//			tile		Possible goal tile
//			prev_tile	Restricts path thru this tile to tile
//	ignore		An array of tiles to ignore
{
	c_astar.Initialize(start,goals,ignore);
}
	function cATTRoadPF::SetMaxRoadLength(len)
//	------------------------------------------
//	Sets the maximum length that the path can be
{
	c_maxlen = len;
}
	function cATTRoadPF::SetMaxDetour(goal,detour)
//	----------------------------------------------
//	Sets the maximum distance a path tile can be from the goal tile
//	goal		A goal tile, just one is used for efficiency
//	detour		Max distance of a path tile from the goal tile
{
	c_goal = goal;
	c_detour = detour;
}
	function cATTRoadPF::FindPath(iterations)
//	-----------------------------------------
//	Returns		null if not found or
//			false if iterations reached or
//			path table
{
	DebugDisable();
	local	test = AITestMode();
	local	path = c_astar.FindPath(iterations);
	DebugEnable(true);

	return path;
}
	function cATTRoadPF::cbCost(t,path,ntile,ndir)
//	----------------------------------------------
//	path		Last path node up to now
//	ntile		Next tile considered
//	ndir		Direction to next tile (DIR_M_ index)
{
	if(!path) return 0;

	this = t;

	local	tile = path.tile, cost = c_ctile,bridge;

//	Existing bridge or tunnel

	if((bridge = AIBridge.IsBridgeTile(ntile)) ||
		      AITunnel.IsTunnelTile(ntile)) {

	  local etile = (bridge) ? AIBridge.GetOtherBridgeEnd(ntile) :
				   AITunnel.GetOtherTunnelEnd(ntile);

	  if(etile == path.tile) 			// Other end
	    return path.cost + AIMap.DistanceManhattan(ntile,path.tile) * cost;
	}
	else {
	  if(ndir != path.dir) cost += c_cturn;		// Turn
	}

	local	bcode = BuiltSlope(tile,ntile);		// Slope after rd built

	if(bcode & ATT_SLP_SLOPED) cost += c_cslope;

	if(AIRoad.IsDriveThroughRoadStationTile(ntile)) cost += c_cthrustop;

	if(AIRoad.AreRoadTilesConnected(tile,ntile))
	  return path.cost + cost;			// Already has road

//	Need to build road

	cost += c_croad;				// Building cost

	if(bcode & ATT_SLP_RAISED) cost += c_craise;	// Foundation

	if(AITile.IsFarmTile(ntile))
	  cost += c_cfields;				// Fields
	else if(AITile.IsCoastTile(ntile))
	  cost += c_ccoast;				// Coast
	else if(AITile.HasTreeOnTile(ntile))
	  cost += c_ctree;

	return path.cost + cost;
}
	function cATTRoadPF::cbEstimate(t,tile,dir,goals)
//	-------------------------------------------------
{
	this = t;

	local	cmin = 100000000;

	if(typeof goals[0] == "array") {
	  foreach (arr in goals)
	    cmin = min(AIMap.DistanceManhattan(tile,arr[1]) * c_ctile,cmin);
	}
	else {
	  foreach (gtile in goals)
	    cmin = min(AIMap.DistanceManhattan(tile,gtile) * c_ctile,cmin);
	}

//	BuildSign(tile,cmin.tostring());

	return cmin;
}
	function cATTRoadPF::cbDirection(t,tile,dir,newdir)
//	---------------------------------------------------
{
	return false;
}
	function cATTRoadPF::cbNeighbors(t,path,tile)
//	---------------------------------------------
{
	this = t;

	if((c_detour && AIMap.DistanceManhattan(tile,c_goal) > c_detour) ||
	   path.length >= c_maxlen) return [];

	local	ntile,bridge;
	local	tiles = [];

//	Check if current tile is a pass exit end

	if(((bridge = AIBridge.IsBridgeTile(tile)) ||
		      AITunnel.IsTunnelTile(tile))) {

	  local etile = (bridge) ? AIBridge.GetOtherBridgeEnd(tile) :
				   AITunnel.GetOtherTunnelEnd(tile);
	  local	dist = AIMap.DistanceManhattan(tile,etile);
	  local noffs = (etile - tile) / dist;

	  if(!path.prev || s_mapoffs[path.dir] == noffs) {	// Enter end
	    tiles.push([etile,(path.prev) ? path.dir : Direction(noffs)]);
	    return tiles;
	  }

//	  At exit end, check next possible tile for road or another pass

	  ntile = tile + -noffs;			// Exit from pass

	  if(AIRoad.AreRoadTilesConnected(tile,ntile) ||
	     AITile.IsBuildable(ntile) || AIRoad.IsRoadTile(ntile)) {
	    tiles.push([ntile,path.dir]);
	  }
	  else if((bridge = AIBridge.IsBridgeTile(ntile)) ||
			    AITunnel.IsTunnelTile(ntile)) {

	    if(CheckPassStart(bridge,path,tile,ntile)) tiles.push([ntile,path.dir]);
	  }

	  return tiles;
	}

//	Loop through four directions, checking neighbors
//	If tile is a source tile, restrict dir unless DIR_M_UNKNOWN

	foreach (i,offs in s_mapoffs) {
	  if(!path.prev && path.dir != DIR_M_UNKNOWN && path.dir != i) continue;

	  ntile = tile + offs;

	  local	err,nextisroad = AIRoad.IsRoadTile(ntile);

	  if(nextisroad || AITile.IsBuildable(ntile)) {
	    if(nextisroad && AIRoad.AreRoadTilesConnected(tile,ntile))
	      tiles.push([ntile,i]);			// i is DIR_M_ index

	    else if(!path.prev ||
	            AIRoad.CanBuildConnectedRoadPartsHere(tile,
							  path.prev.tile,ntile)) {
	      if(AIRoad.BuildRoad(tile,ntile))
	        tiles.push([ntile,i]);			// i is DIR_M_ index

	      else {
		switch (AIError.GetLastError()) {
		case AIError.ERR_ALREADY_BUILT:
	          tiles.push([ntile,i]);		// i is DIR_M_ index
		  break;

		case AIError.ERR_VEHICLE_IN_THE_WAY:
	          if(AIRoad.BuildRoad(ntile,tile))	// From other direction
	            tiles.push([ntile,i]);		// i is DIR_M_ index
		}
	      }
	    }
	  }
	  else if((bridge = AIBridge.IsBridgeTile(ntile)) ||
			     AITunnel.IsTunnelTile(ntile)) {

	    if(CheckPassStart(bridge,path,tile,ntile)) tiles.push([ntile,i]);
	  }
	}

	return tiles;
}
	function cATTRoadPF::CheckPassStart(bridge,path,tile,ntile)
//	-----------------------------------------------------------
//	Next tile is bridge or tunnel, check if suitable to pass
//	bridge		true if next tile is a bridge, else tunnel
//	path		path upto now
//	tile		tile in path
//	ntile		next tile being considered
//	Returns		true if ntile added to tiles
{
	if(AITile.HasTransportType(ntile,AITile.TRANSPORT_ROAD)) {
	  local etile = (bridge) ? AIBridge.GetOtherBridgeEnd(ntile) :
				   AITunnel.GetOtherTunnelEnd(ntile);
	  local noffs = (etile - ntile) / AIMap.DistanceManhattan(ntile,etile);

	  if((path.dir == DIR_M_UNKNOWN || s_mapoffs[path.dir] == noffs) &&
	     (AITile.GetMaxHeight(tile) == AITile.GetMaxHeight(ntile) ||
	      (path.prev &&
	       AIRoad.CanBuildConnectedRoadPartsHere(tile,path.prev.tile,ntile))))
	    return true;
	}

	return false;
}
