	class cATTRoadRoute extends cATTRoute
//	-------------------------------------
{
	c_classname = null;
	c_road = null;					// Road instance
	c_sstop = null;					// Source stop info
	c_dstop = null;					// Dest stop info
	c_seq = null;					// Vehicle sequence#

	constructor()
//	-------------
	{
	  cATTRoute.constructor(YATT_CMI_ROAD);

	  c_classname = "cATTRoadRoute";
	  c_seq = 0;
	}
}
	function cATTRoadRoute::EstimateRoadRouteCost()
//	-----------------------------------------------
//	Evaluate route and return implementation cost
//	Returns		total cost to implement or YATT_E_ code
{
	if(s_g.rtelimchk) {
	  if(c_dist < s_cdist.min[YATT_CMI_ROAD]) return YATT_E_RTETOOSHORT;
	  if(c_dist > s_cdist.max[YATT_CMI_ROAD]) return YATT_E_RTETOOLONG;

	  if(c_srctown) {
	    if(AITown.GetPopulation(c_srcid) < 750) return YATT_E_LOWPOPULATION;
	    if(AITown.GetPopulation(c_dstid) < 750) return YATT_E_LOWPOPULATION;
	  }
	  else if(AIIndustry.GetLastMonthProduction(c_srcid,c_cargo) < 80)
	    return YATT_E_LOWPRODUCTION;
	}

	if(!(c_engsel = SelectRoadVehicle(c_cargo))) return YATT_E_NOVEHICLE;

	local stat = SurveyRoadRoute();
	if(stat < 0) return stat;

	return c_cost + AIEngine.GetPrice(c_engsel);
}
	function cATTRoadRoute::SurveyRoadRoute()
//	-----------------------------------------
//	Survey a road route
//	Enters route code into s_g.rtebad if route cannot ever be built
//	Calculates c_cost = cost of road, stations and depots and the estimated
//			    cost to demolish existing buildings to build stops
//	Returns		YATT_E_ code
{
	if(c_srctown)
	  c_cost = SurveyTownRoute();
	else if(c_dsttown)
	  c_cost = SurveySupplyRoute();
	else
	  c_cost = SurveyIndustryRoute();

	if(c_cost < 0) return c_cost;				// Error returned

	c_dist = AITile.GetDistanceManhattanToTile(c_stops[0].c_tile,c_stops[1].c_tile);

//	if(c_dist < s_cdist.min[YATT_CMI_ROAD] - 6) return YATT_E_RTETOOSHORT;
	if(c_road.c_dist > c_dist * 1.25) return YATT_E_INEFFICIENT;

	if(c_road.c_dist > GetMaxDistForVehicle(YATT_CMI_ROAD,c_cargo))
	  return YATT_E_RTETOOLONG;

	c_cost += c_stops[0].GetBuildCost();
	c_cost += c_stops[1].GetBuildCost();

	if(c_sstop && c_sstop.thrustop && c_sstop.roadloop)
	  c_cost += c_sstop.roadloop.c_cost;
	else if(!c_sstop)
	  c_cost += EstimateRoadLoopCost();

	if(c_dstop && c_dstop.thrustop && c_dstop.roadloop)
	  c_cost += c_dstop.roadloop.c_cost;
	else if(!c_dstop)
	  c_cost += EstimateRoadLoopCost();

	c_cost += c_depots[0].GetBuildCost();
	if(c_depots.len() > 1) c_cost += c_depots[1].GetBuildCost();

	c_length = c_road.c_dist;			// Actual route length
	c_maxveh = (c_length / 2.5).tointeger();

	return YATT_E_SUCCESS;
}
	function cATTRoadRoute::SurveyTownRoute()
//	-----------------------------------------
//	Survey all necessary parts to set up an in-town to in-town route
//	c_cargo should be passengers or mail
//	Returns		Cost to build road, without stops and depots or
//			YATT_E_ code
{
	if(IsTownRatingLow(c_srcid) || IsTownRatingLow(c_dstid))
	  return YATT_E_TOWNRATING;

	if(!(c_sstop = SurveyTownStop(c_srcid))) return YATT_E_NOSTOP;
	if(!(c_dstop = SurveyTownStop(c_dstid))) return YATT_E_NOSTOP;

	if(AITile.GetDistanceManhattanToTile(c_sstop.tile,c_dstop.tile) <
					     s_cdist.min[YATT_CMI_ROAD] - 6)
	  return YATT_E_RTETOOSHORT;

	local	src = GetStopTiles(c_sstop,true);
	local	dst = GetStopTiles(c_dstop,false);

	c_road = cATTRoad(src,dst);

	if(!c_road.FindPath()) return (s_g.abort) ? YATT_E_ABORTED : YATT_E_NOPATH;
	if(!c_road.TestBuildRoad()) return YATT_E_TESTBUILDFAILED;

	if(!c_sstop.thrustop) c_road.ExtendRoad(0,c_sstop.tile); // Add station tiles
	if(!c_dstop.thrustop) c_road.ExtendRoad(1,c_dstop.tile);

	local	stptype = (c_cargo == s_crgpassengers) ? cATTRoadStop.YATT_VT_BUS :
							 cATTRoadStop.YATT_VT_TRUCK;

	c_stops.push(cATTRoadStop(c_road.c_ends[0],c_road.c_front[0],
				  stptype,c_sstop.thrustop));
	c_stops.push(cATTRoadStop(c_road.c_ends[1],c_road.c_front[1],
				  stptype,c_dstop.thrustop));

	local stat = SurveyRouteDepots();

	if(stat != YATT_E_SUCCESS) return stat;

	return c_road.c_cost;				// Cost to build road
}
	function cATTRoadRoute::SurveyIndustryRoute()
//	---------------------------------------------
//	Survey all necessary parts to set up an ind-ind route
//	Returns		Cost to build road, without stops and depots
{
	local	src,dst;

	src = SurveyIndustryStop(c_srcid,true,c_dsttile);
	if(typeof src == "integer") return src;		// Error code

	dst = SurveyIndustryStop(c_dstid,false,c_srctile);
	if(typeof dst == "integer") return dst;		// Error code

	c_road = cATTRoad(src.arr,dst.arr);

	if(!c_road.FindPath()) return (s_g.abort) ? YATT_E_ABORTED : YATT_E_NOPATH;
	if(!c_road.TestBuildRoad()) return YATT_E_TESTBUILDFAILED;

	if(!CanBuildOnTile(c_road.c_ends[0])) return YATT_E_TOWNRATING;

	c_stops.push(cATTRoadStop(c_road.c_ends[0],c_road.c_front[0],
				  cATTRoadStop.YATT_VT_TRUCK,(c_sstop != null)));

	if(!dst.stop) {					// New dst stop
	  if(!CanBuildOnTile(c_road.c_ends[1])) return YATT_E_TOWNRATING;

	  c_stops.push(cATTRoadStop(c_road.c_ends[1],c_road.c_front[1],
				    cATTRoadStop.YATT_VT_TRUCK,(c_dstop != null)));
	}
	else
	  c_stops.push(dst.stop);			// Existing stop

	local stat = SurveyRouteDepots();

	if(stat != YATT_E_SUCCESS) return stat;

	return c_road.c_cost;				// Cost to build road
}
	function cATTRoadRoute::SurveySupplyRoute()
//	-------------------------------------------
//	Survey all necessary parts to set up an indusrty to town supply route
//	Returns		Cost to build road, without stops and depots
{
	local	dtile,src,dst;

	if(IsTownRatingLow(c_dstid)) return YATT_E_TOWNRATING;

	src = SurveyIndustryStop(c_srcid,true,c_dsttile);
	if(typeof src == "integer") return src;		// Error code

	if(!(c_dstop = SurveyTownStop(c_dstid))) return YATT_E_NOSTOP;

	dst = GetStopTiles(c_dstop,false);

	c_road = cATTRoad(src.arr,dst);

	if(!c_road.FindPath()) return (s_g.abort) ? YATT_E_ABORTED : YATT_E_NOPATH;
	if(!c_road.TestBuildRoad()) return YATT_E_TESTBUILDFAILED;

	if(!CanBuildOnTile(c_road.c_ends[0])) return YATT_E_TOWNRATING;

	if(!c_dstop.thrustop) c_road.ExtendRoad(1,c_dstop.tile);

	c_stops.push(cATTRoadStop(c_road.c_ends[0],c_road.c_front[0],
				  cATTRoadStop.YATT_VT_TRUCK,(c_sstop != null)));

	c_stops.push(cATTRoadStop(c_road.c_ends[1],c_road.c_front[1],
				  cATTRoadStop.YATT_VT_TRUCK,c_dstop.thrustop));

	local stat = SurveyRouteDepots();

	if(stat != YATT_E_SUCCESS) return stat;

	return c_road.c_cost;				// Cost to build road
}
	function cATTRoadRoute::SurveyRouteDepots()
//	-------------------------------------------
//	Survey places for one or two depots
//	Returns		YATT_E_ code
{
	local	srcdep,dstdep,ignore;

	ignore = (c_sstop && c_sstop.thrustop && c_sstop.roadloop) ?
			c_sstop.roadloop.GetUnbuiltTiles() : null;

	if(!(srcdep = c_road.SurveyRoadDepot(0,ignore))) return YATT_E_NODEPOT;

	if(!srcdep.depot) {				// New depot
	  if(!CanBuildOnTile(srcdep.tile)) return YATT_E_TOWNRATING;

	  c_depots.push(cATTRoadDepot(srcdep.tile,srcdep.front));
	}
	else
	  c_depots.push(srcdep.depot);			// Existing depot

	ignore = (c_dstop && c_dstop.thrustop && c_dstop.roadloop) ?
			c_dstop.roadloop.GetUnbuiltTiles() :  null;

	if((dstdep = c_road.SurveyRoadDepot(1,ignore)) &&
	   AIMap.DistanceManhattan(srcdep.tile,dstdep.tile) > 15) {

	  if(!dstdep.depot) {
	    if(CanBuildOnTile(dstdep.tile))
	      c_depots.push(cATTRoadDepot(dstdep.tile,dstdep.front));
	  }
	  else
	    c_depots.push(dstdep.depot);		// Existing depot
	}

	return YATT_E_SUCCESS;
}
	function cATTRoadRoute::SurveyIndustryStop(indid,prod,dsttile)
//	--------------------------------------------------------------
//	Find suitable location for a truck stop around an industry
//	indid		Industry ID
//	prod		Producing industry
//	dsttile		The other end of the route
//	Returns		YATT_E_ code or table with following fields:
//			stop	cATTRoadStop or null
//			arr	Array of tiles suitable for pathfinder
{
	if(!prod) {
	  local	t = SurveyExistingIndustryDstStop(indid);

	  if(t) return t;
	}

	local	covrad = AIStation.GetCoverageRadius(AIStation.STATION_TRUCK_STOP);
	local	list = (prod) ? AITileList_IndustryProducing(indid,covrad) :
				AITileList_IndustryAccepting(indid,covrad);

	ExcludeStationTiles(list,false,false);	// Remove tiles blocked by stations

	list.Valuate(AITile.GetSlope);
	list.KeepValue(AITile.SLOPE_FLAT);

	list.Valuate(AITile.IsBuildable);
	list.KeepValue(1);

	if(list.IsEmpty()) return YATT_E_NOSTOP;

	foreach (tile,v in list)		// Check town rating
	  list.SetValue(tile,CanBuildOnTile(tile));

	list.KeepValue(1);

	if(list.IsEmpty()) return YATT_E_TOWNRATING;

//	Find 5 best locations

	if(list.Count() > 5) {
	  list.Valuate(AITile.GetDistanceManhattanToTile,dsttile);
	  list.Sort(AIList.SORT_BY_VALUE,true);
	}

	local	arr = [],cnt=0;
	local	test = AITestMode();

	foreach (tile,v in list) {
	  if(cnt == 5) break;			// Have enough tiles

	  for (local i=0; i <= 2; i+=2) {	// Two directions +-X, +-Y
	    if(cnt == 5) break;			// Have enough tiles

	    local fb = [tile + s_mapoffs[i],tile + s_mapoffs[i+1]];
	    local canbuild = false;

	    for (local j=0; j < 2; j++) {	// Front and back
	      if(AITile.GetSlope(fb[j]) != AITile.SLOPE_FLAT) break;
	      if(!CanConnectTiles(tile,fb[j])) break;

	      if(j == 1) canbuild = true;
	    }

	    if(canbuild) {
	      if(prod) {			// To specific directions
	        arr.push([tile,i]);
	        arr.push([tile,i+1]);
	      }
	      else {				// From specific tiles
		arr.push([tile,fb[0]]);
		arr.push([tile,fb[1]]);
	      }

	      cnt++;
	    }
	  }
	}

	if(!cnt) return YATT_E_NOSTOP;

	return {stop=null, arr=arr};
}
	function cATTRoadRoute::SurveyExistingIndustryDstStop(iid)
//	----------------------------------------------------------
//	See if an existing destination stop can be used at the industry
//	iid		Industry id
//	Returns		null or table with following fields:
//			stop	cATTRoadStop
//			arr	Thru stop [tile] for pathfinder
{
	local	idcode = GetDstIDCode();

	if(!(idcode in s_g.rtestp)) return null;	// No stops here

	foreach (i,rte in s_g.rtestp[idcode]) {
	  if(!rte.c_dsttown && rte.c_dstid == iid) {
	    local stop = rte.c_stops[1];

	    if(!stop.c_thrustop || stop.c_refcnt == 3) continue;

	    return {stop=stop, arr=[[stop.c_tile,stop.c_front],
				    [stop.c_tile,stop.c_back]]};
	  }
	}

	return null;
}
	function cATTRoadRoute::SurveyTownStop(townid)
//	----------------------------------------------
//	Tries to find a drive through stop
//	Returns		null or a table with the following fields:
//			thrustop	true if drive-through
//			tile		station tile
//			roads		arr of surrounding road tiles (normal only)
//			front		tile at one end (thru only)
//			roadloop	cATTRoad (thru only)
{
	local	t = SurveyTownThroughStop(townid);

	if(t)
	  t.thrustop <- true;
	else
	  return null;

	return t;
}
	function cATTRoadRoute::SurveyTownThroughStop(townid)
//	-----------------------------------------------------
//	Find Suitable location for a drive-through stop in a town
//	Makes sure a loop from front-to-back exists or can be built
//	The max length of this loop is 20 tiles
//	Returns		null or table with the following fields
//			tile		Through station tile
//			front		Tile at one end
//			roadloop	cATTRoad showing front-to-back loop
{
	local	stntype,stptype;

	if(c_cargo == s_crgpassengers) {
	  stntype = AIStation.STATION_BUS_STOP;
	  stptype = cATTRoadStop.YATT_VT_BUS;
	}
	else {
	  stntype = AIStation.STATION_TRUCK_STOP;
	  stptype = cATTRoadStop.YATT_VT_TRUCK;
	}

	local	covrad = AIStation.GetCoverageRadius(stntype);
	local	list = GetTownTiles(townid,10);

	ExcludeStationTiles(list,true,(stntype == AIStation.STATION_BUS_STOP));

	list.Valuate(AIRoad.IsRoadTile);
	list.KeepValue(1);

	list.Valuate(AITile.GetSlope);
	list.KeepValue(AITile.SLOPE_FLAT);

	list.Valuate(AIRoad.GetNeighbourRoadCount);
	list.KeepValue(2);

	list.Valuate(AITile.GetCargoAcceptance,c_cargo,1,1,covrad);

	if(s_g.rtelimchk)
	  list.KeepAboveValue((c_cargo == s_crgpassengers) ? 74 : 24);

	local	test = AITestMode();

	while (true) {
	  if(list.IsEmpty()) return null;

	  local tile = list.Begin();
	  local	arr = FindConnectingRoads(tile);

	  if(arr.len() == 2 && arr[0] - tile == tile - arr[1] &&	// Straight
	     AIRoad.AreRoadTilesConnected(arr[0],tile) &&
	     AIRoad.AreRoadTilesConnected(arr[1],tile)) {

	    local roadstop = cATTRoadStop(tile,arr[0],stptype,true,arr[1]);

	    if(roadstop.BuildDriveThroughRoadStop()) {
	      local roadloop = cATTRoad([[arr[0],DIR_M_UNKNOWN]],[arr[1]],[tile]);

	      if(!roadloop.FindPath(20) || !roadloop.TestBuildRoad())
		roadloop = null;

	      return {tile=tile, front=arr[0], roadloop=roadloop};
	    }
	  }

	  list.RemoveTile(tile);
	}
}
	function cATTRoadRoute::SurveyThroughStopLoop(tile,front,depot=null)
//	--------------------------------------------------------------------
//	Check if road loop around a through stop can be built
//	tile		Where stop will go
//	front		Front of stop
//	depot		Tile for future depot, ignore tile
//	Returns		null or a table with the following fields:
//			thrustop	true
//			tile		station tile
//			front		tile at one end
//			roadloop	cATTRoad
{
	if(!SurveyThroughStopFront(tile,front)) return null;

	local	back = tile - (front - tile);

	if(!SurveyThroughStopFront(tile,back)) return null;

	local	ignore = [tile];

	if(depot) ignore.push(depot);		// Ignore depot tile

	local	roadloop = cATTRoad([[front,DIR_M_UNKNOWN]],[back],ignore);

	if(roadloop.FindPath(16,-1) && roadloop.TestBuildRoad()) {
	  roadloop.ExtendRoad(0,tile);		// Extend to station tile
	  roadloop.ExtendRoad(1,tile);		// Both ends

	  return {thrustop=true, tile=tile, front=front, roadloop=roadloop};
	}

	return null;
}
	function cATTRoadRoute::SurveyThroughStopFront(tile,front)
//	----------------------------------------------------------
//	Check if the front tile can connect with the loop
{
	if(AITile.GetSlope(front) != AITile.SLOPE_FLAT) return false;
	if(AITile.IsBuildable(front) && CanBuildOnTile(front)) return true;
	if(!AIRoad.IsRoadTile(front)) return false;

	if(AITile.IsStationTile(front) &&
	   AIRoad.GetRoadStationFrontTile(front) != tile &&
	   AIRoad.GetDriveThroughBackTile(front) != tile) return false;

	return true;
}
	function cATTRoadRoute::EstimateRoadLoopCost()
//	----------------------------------------------
//	If we cannot plan the through stop loop cost, try to estimate its cost
//	Assume 8 road tiles, half of which are in fields
{
	return AIRoad.GetBuildCost(AIRoad.ROADTYPE_ROAD,AIRoad.BT_ROAD) * 8 +
	       AITile.GetBuildCost(AITile.BT_CLEAR_FIELDS) * 4;
}
	function cATTRoadRoute::BuildRoadRoute()
//	----------------------------------------
//	Build the surveyed route
//	Returns		true if route built ok
{
	local	success=true;

//	Industry source and not through stop, try to make a thru stop

	if(!c_srctown && !c_sstop) {		
	  success = c_road.BuildRoad(0,5);	// Build 5 road segments

	  if(success) {
	    c_sstop = SurveyThroughStopLoop(c_stops[0].c_tile,
					    c_stops[0].c_front,
					    c_depots[0].c_tile);
	    c_stops[0].SetThroughStop();	// Always build thru stop
	  }
	}

//	Industry destination and not through stop, try to make a thru stop
//	Check to see if route is sharing a destination stop

	if(success && !c_dsttown && !c_stops[1].c_built && !c_dstop) {
	  success = c_road.BuildRoad(1,5);	// Build 5 road segments

	  if(success) {
	    c_dstop = SurveyThroughStopLoop(c_stops[1].c_tile,c_stops[1].c_front,
				(c_depots.len() > 1) ? c_depots[1].c_tile : null);
	    c_stops[1].SetThroughStop();	// Always build thru stop
	  }
	}

	if(success && (success = c_stops[0].BuildRoadStop()))
	  c_rtbuilt = c_rtbuilt | YATT_RTB_STOP0;

	if(success && (success = c_stops[1].BuildRoadStop()))
	  c_rtbuilt = c_rtbuilt | YATT_RTB_STOP1;

	if(success && (success = c_depots[0].BuildRoadDepot()));
	  c_rtbuilt = c_rtbuilt | YATT_RTB_DEPOT0;

	if(success && c_depots.len() > 1 && (success = c_depots[1].BuildRoadDepot()))
	  c_rtbuilt = c_rtbuilt | YATT_RTB_DEPOT1;

	if(success && c_sstop && c_sstop.thrustop && c_sstop.roadloop)
	  success = c_sstop.roadloop.BuildRoad();

	if(success && c_dstop && c_dstop.thrustop && c_dstop.roadloop)
	  success = c_dstop.roadloop.BuildRoad();

	if(success) success = c_road.BuildRoad();

	c_road = null;		// No longer need this data
	c_sstop = null;
	c_dstop = null;

	return success;
}
	function cATTRoadRoute::DemolishRoadRoute()
//	-------------------------------------------
//	Does not demolish the road tiles
//	Returns		If all stations and depots were demolished
{
	Warning(c_crgname + " - " + c_srcname + " to " + c_dstname +
	        ": Demolishing road route");

	if(c_rtbuilt & YATT_RTB_STOP0) c_stops[0].DemolishRoadStop();
	if(c_rtbuilt & YATT_RTB_STOP1) c_stops[1].DemolishRoadStop();

	if(c_rtbuilt & YATT_RTB_DEPOT0) c_depots[0].DemolishRoadDepot();
	if(c_rtbuilt & YATT_RTB_DEPOT1) c_depots[1].DemolishRoadDepot();

	if(c_grpid != null) AIGroup.DeleteGroup(c_grpid);
}
	function cATTRoadRoute::SelectRoadVehicle(cargo)
//	------------------------------------------------
//	Select a vehicle to carry cargo
//	Returns		Vehicle type selected or null if no vehicle found
{
	local	t = GetBestVehicle(YATT_CMI_ROAD,cargo);

	return (t) ? t.engid : null;
}
	function cATTRoadRoute::BuildRoadVehicle(renew=false)
//	-----------------------------------------------------
//	Build selected road vehicle and put it into service
//	Returns		YATT_E_ code
{
	c_engprice = AIEngine.GetPrice(c_engsel);

	if(!GetFunds(c_engprice)) return YATT_E_NOFUNDS;

	local	startidx = 0;				// Start at this end

	if(c_srctown && GetVehicleCount() > 0 &&
	   GetCargoWaitingAtStop(1) < GetCargoWaitingAtStop(0)) startidx = 1;

	local	depidx = (!startidx || c_depots.len() == 1) ? 0 : 1;
	local	vehid = AIVehicle.BuildVehicle(c_depots[depidx].c_tile,c_engsel);

	if(!AIVehicle.IsValidVehicle(vehid)) return YATT_E_OTTD;

	if(AIEngine.GetCargoType(c_engsel) != c_cargo)		// Refit needed
	  AIVehicle.RefitVehicle(vehid,c_cargo);

	c_vehids.push(vehid);
	c_seq++;

	c_totcap += AIVehicle.GetCapacity(vehid,c_cargo);	// Total capacity

//	Add orders to handle old age
//	1:Goto stop 0
//	a:Jump to order + 3 (2) when load > 0 (if secondary cargo)
//	b:Goto depot 0
//	c:Jump to order 1
//	2:Goto depot 1 (if exists)
//	3:Goto stop 1
//	4:Jump to order +3 (7) when age >= max_age - 1
//	5:Goto depot 0
//	6:Jump to order 1
//	7:Jump to order +2 (9) when load == 0
//	8:Go to stop 0 (Unload)
//	9:Go to nearest depot

	local	breakdowns = AIGameSettings.GetValue("vehicle_breakdowns");
	local	srcorder = AIOrder.AIOF_NON_STOP_INTERMEDIATE;
	local	dstorder = AIOrder.AIOF_NON_STOP_INTERMEDIATE;
	local	rawind = true;

//	If industry source, full load only on raw industry

	if(!c_srctown) {
	  rawind =  AIIndustryType.IsRawIndustry(
				AIIndustry.GetIndustryType(c_srcid));

	  if(rawind) srcorder = srcorder | AIOrder.AIOF_FULL_LOAD_ANY;
	}
	else if(GetVehicleCount() <= 3 || (c_seq % 2)) {	// First 3 or odd
	  srcorder = srcorder | AIOrder.AIOF_FULL_LOAD_ANY;
	  dstorder = dstorder | AIOrder.AIOF_FULL_LOAD_ANY;
	}

	AIOrder.AppendOrder(vehid,c_stops[0].c_tile,srcorder);	// Goto stop 0

//	Order to loop until cargo available (if not raw industry)

	local loopidx;

	if(!rawind) {
	  loopidx = AppendCondOrder(vehid,0,AIOrder.OC_LOAD_PERCENTAGE,
				    AIOrder.CF_MORE_THAN,0);

	  AIOrder.AppendOrder(vehid,c_depots[0].c_tile,
			      AIOrder.AIOF_NON_STOP_INTERMEDIATE);

	  AppendCondOrder(vehid,0);				// Jump to order 1
	}

	if(c_depots.len() > 1 && breakdowns)			// Goto depot 1
	  AIOrder.AppendOrder(vehid,c_depots[1].c_tile,
			      AIOrder.AIOF_NON_STOP_INTERMEDIATE);

	AIOrder.AppendOrder(vehid,c_stops[1].c_tile,dstorder);	// Goto stop 1
	local	dstordidx = AIOrder.GetOrderCount(vehid) - 1;

	local ageidx = AppendCondOrder(vehid,0,AIOrder.OC_AGE,AIOrder.CF_MORE_EQUALS,
					       AIVehicle.GetMaxAge(vehid)/365-1);

	if(breakdowns)						// Goto depot 0
	  AIOrder.AppendOrder(vehid,c_depots[0].c_tile,
			      AIOrder.AIOF_NON_STOP_INTERMEDIATE);

	AppendCondOrder(vehid,0);				// Jump to order 1

	local loadidx = AppendCondOrder(vehid,0,AIOrder.OC_LOAD_PERCENTAGE,
						AIOrder.CF_EQUALS,0);

	AIOrder.AppendOrder(vehid,c_stops[0].c_tile,
			    AIOrder.AIOF_NON_STOP_INTERMEDIATE |
			    AIOrder.AIOF_UNLOAD | AIOrder.AIOF_NO_LOAD);

	AIOrder.AppendOrder(vehid,0,
			    AIOrder.AIOF_NON_STOP_INTERMEDIATE |
			    AIOrder.AIOF_GOTO_NEAREST_DEPOT |
			    AIOrder.AIOF_STOP_IN_DEPOT);

	if(!rawind) AIOrder.SetOrderJumpTo(vehid,loopidx,loopidx+3);

	AIOrder.SetOrderJumpTo(vehid,ageidx,loadidx);		// Adjust jumpto
	AIOrder.SetOrderJumpTo(vehid,loadidx,loadidx+2);	// Adjust jumpto

	if(startidx) AIOrder.SkipToOrder(vehid,dstordidx);	// Other end

//	Create a vehicle group for the route, if needed

	if(c_grpid == null) CreateGroup(AIVehicle.VT_ROAD);

	AIGroup.MoveVehicle(c_grpid,vehid);
	AIVehicle.StartStopVehicle(vehid);

	if(!renew) c_datebought = AIDate.GetCurrentDate();

//	Info(GetRouteName() + ": Bought " + AIVehicle.GetName(vehid));

	return YATT_E_SUCCESS;
}
	function cATTRoadRoute::FindSurroundingRoads(tile)
//	--------------------------------------------------
//	Find surrounding roads that can be accessed from tile
//	Tile may have a building on it, so no roadbuilding test is done
//	Returns		array of tiles with roads
{
	local	elev = AITile.GetMaxHeight(tile);
	local	arr = [];

	foreach (i,offs in s_mapoffs) {
	  local ntile = tile + offs;

	  if(AIRoad.IsRoadTile(ntile) && AITile.GetMaxHeight(ntile) == elev)
	    arr.push(ntile);
	}

	return arr;
}
	function cATTRoadRoute::FindConnectingRoads(tile)
//	-------------------------------------------------
//	Find surrounding roads that can be accessed by road from tile
//	Returns		array of tiles with roads
{
	local	arr = [];

	foreach (i,offs in s_mapoffs) {
	  local ntile = tile + offs;

	  if(AIRoad.IsRoadTile(ntile) && AIRoad.AreRoadTilesConnected(tile,ntile))
	    arr.push(ntile);
	}

	return arr;
}
	function cATTRoadRoute::GetStopTiles(t,source)
//	----------------------------------------------
//	Returns array of tiles from tiles in table t describing stop type
//	t		Table returned by SurveyTownStop
//	source		If true, the tiles will be converted to PFSource format
{
	local	arr = (t.thrustop) ? [t.tile] : t.roads;

	return (source) ? ConvertArrayToPFSource(arr) : arr;
}
	function cATTRoadRoute::ConvertArrayToPFSource(tiles)
//	-----------------------------------------------------
//	Convert an array of tiles to be suitable as source to pathfinder
//	tile		array of tiles
//	Returns		array of [tile,dir] pairs
{
	local	arr = [];

	foreach (i,tile in tiles)
	  arr.push([tile,DIR_M_UNKNOWN]);

	return arr;
}
	function cATTRoadRoute::CanConnectTiles(tile1,tile2)
//	----------------------------------------------------
//	Assumes Test mode
{
	while (!AIRoad.BuildRoad(tile1,tile2)) {
	  switch (AIError.GetLastError()) {
	  case AIError.ERR_ALREADY_BUILT:
	    return true;

	  case AIError.ERR_VEHICLE_IN_THE_WAY:
	    Sleep(10);
	    break;

	  default:
	    return false;
	  }
	}

	return true;
}
	function cATTRoadRoute::CheckRoadRouteProduction()
//	--------------------------------------------------
//	Check production on a route, look for improvements
//	Returns		YATT_MGR code, tells mgr what to do
{
	local	surplus = GetCargoSurplus();

	switch (NeedMoreRoadVehicles(surplus)) {
	case YATT_NMV_YES:
	  if(!GetAllowedVehCount(YATT_CMI_ROAD)) break;	// Limit reached

	  c_engprice = AIEngine.GetPrice(c_engsel);
	  c_srvstate = YATT_SST_BUYING;

	  return YATT_MGR_BUYVEHICLE;

	case YATT_NMV_UPGRADE:
	  c_engprice = AIEngine.GetPrice(c_engsel);
	  return YATT_MGR_UPGRADEVEHICLE;

	case YATT_NMV_UPGRADEFLEET:
	  c_engprice = AIEngine.GetPrice(c_engsel);
	  return YATT_MGR_UPGRADEFLEET;
	}

	if(NeedLessRoadVehicles(surplus)) {
	  c_srvstate = YATT_SST_SELLING;

	  return YATT_MGR_SELLVEHICLE;
	}

	if(c_srvstate != YATT_SST_STEADY) {
	  local	date = AIDate.GetCurrentDate();

	  if(date - c_datebought > 180 && date - c_datesold > 180)
	    c_srvstate = YATT_SST_STEADY;
	}

	return YATT_MGR_NONE;
}
	function cATTRoadRoute::NeedMoreRoadVehicles(surplus)
//	-----------------------------------------------------
//	Check if we need to buy a new road vehicle
//	If surplus < c_psurplus, surplus going down, don't buy
//	Returns		YATT_NMV_ code
{
	c_engsel = SelectRoadVehicle(c_cargo);

	if(!GetVehicleCount()) return YATT_NMV_YES;	// No vehicles

	local	vehid = c_vehids[0];

	if(AIVehicle.GetAge(vehid) > 365 * 3) {
	  if(c_profit < 0) return YATT_NMV_NO;		// Could be traffic

	  if(AIVehicle.GetEngineType(vehid) != c_engsel)
	    return YATT_NMV_UPGRADEFLEET;

	  if(c_avgprofit < c_avgprofit2 * 0.80 &&
	     AIDate.GetCurrentDate() - c_datebought >= 730)
	    return YATT_NMV_UPGRADEFLEET;
	}

	if(c_stops[c_waitidx].GetCargoRating(c_cargo) > 50) {
	  if(surplus < c_psurplus) {
	    c_psurplus -= (c_psurplus - surplus) / 6;
	    return YATT_NMV_NO;
	  }

	  c_psurplus = surplus;

	  local vehcap = AIEngine.GetCapacity(c_engsel);

	  if(surplus < vehcap) return YATT_NMV_NO;
	}

	return (GetVehicleCount() < c_maxveh)? YATT_NMV_YES : YATT_NMV_NO;
}
	function cATTRoadRoute::NeedLessRoadVehicles(surplus)
//	-----------------------------------------------------
//	surplus		-ve number indicating overcapacity
//	Returns		true if we need to sell a vehicle
{
	if(!GetVehicleCount()) return false;		// No vehicles

	if(c_waiting > 0 || c_stops[c_waitidx].GetCargoRating(c_cargo) < 67) 
	  return false;

	local	vehid = c_vehids[0];
	local	vehcap = AIVehicle.GetCapacity(vehid,c_cargo);
	local	days = AIDate.GetCurrentDate() - c_datesold;

//	Overcapacity > 6x and more than 30 days since we sold a vehicle, sell
	if(-surplus > vehcap * 6 && days > 30) return true;

//	Less than 90 days sice vehicle sold a vehicle or overcapacity < 3x, keep
	if(days < 90 || -surplus < vehcap * 3) return false;

	local age = AIVehicle.GetAge(vehid);		// Oldest vehicle age (days)

//	Older than 3 years and overcapacity > 5x, sell
	if(age > 3 * 365 && -surplus > vehcap * 5) return true;

	return (age > AIVehicle.GetMaxAge(vehid) / 2);
}
	function cATTRoadRoute::ExcludeStationTiles(list,intown,bus)
//	------------------------------------------------------------
//	Find opponent station tiles and exclude 1x1 rectangle around them
//	list		List of tiles to scan for stations
//	intown		Tiles are in town, can build next to own station
//			else must be more than one tile away
//	bus		Bus station, try not to overlap our bus stations
//	Returns		No return value, but list is modified by removing
//			tiles that are stations or around enemy stations
{
	local	slist = AITileList();

	slist.AddList(list);

	slist.Valuate(AITile.IsStationTile);
	slist.KeepValue(1);
	if(slist.IsEmpty()) return;

	local	elist = cATTTileList();

	foreach (tile,v in slist) {
	  if(intown && AICompany.IsMine(AITile.GetOwner(tile)))	{// Our station
	    if(bus) {
	      local id = AIStation.GetStationID(tile);

	      if(AIStation.HasStationType(id,AIStation.STATION_TRUCK_STOP))
	        elist.AddTile(tile);			// Exclude only 1 tile
	      else
		elist.AddSquare(tile,6);
	    }
	    else
	      elist.AddTile(tile);			// Exclude only 1 tile
	  }
	  else
	    elist.AddSquare(tile,1);
	}

	list.RemoveList(elist);
}
