	class cATTRoadStop extends cATTStation
//	--------------------------------------
{
	static	YATT_VT_TRUCK	= 1;
	static	YATT_VT_BUS	= 2;

	static	s_stype = [null,cATTStation.YATT_STN_TRUCK,
				cATTStation.YATT_STN_BUS];

	static	s_vehtype = [null,AIRoad.ROADVEHTYPE_TRUCK,
				  AIRoad.ROADVEHTYPE_BUS];

	c_classname = null;

	c_vehtype = null;			// Road vehicle type
	c_front = null;				// Front tile
	c_thrustop = null;			// Drive-through stop
	c_back = null;				// For through stops

	constructor(tile,front,vehtype,thrustop,back=null)
//	--------------------------------------------------
//	tile		Station tile
//	front		Station front tile
//	vehtype		YATT_VT_ code
//	thrustop	True for drivethrough stop
//	back		Drivethrough station back tile (calculated if null)
	{
	  ::cATTStation.constructor(s_stype[vehtype]);

	  c_classname = "cATTRoadStop";

	  c_tile = tile;
	  c_front = front;
	  c_vehtype = vehtype;
	  c_thrustop = thrustop;

	  if((c_back = back) == null && thrustop) c_back = tile - (front - tile);
	}
}
	function cATTRoadStop::BuildRoadStop(id=AIStation.STATION_NEW)
//	--------------------------------------------------------------
//	Returns		true if stop built ok
{
	if(c_built) {				// Already built
	  c_refcnt++;				// Incr reference count
	  return true;
	}

	local	stat = true;

	if(!c_thrustop) {
	  DebugEnable();

	  if(!AITile.IsBuildable(c_tile)) stat = DemolishTile(c_tile);

	  if(stat) {
	    while (!(stat = AIRoad.BuildRoadStation(c_tile,c_front,
						    s_vehtype[c_vehtype],id))) {
	      switch (AIError.GetLastError()) {
	      case AIError.ERR_VEHICLE_IN_THE_WAY:
	        Sleep(10);
	        break;

	      case AIError.ERR_NOT_ENOUGH_CASH:
	        WaitForFunds();
	        break;

	      default:
		return false;
	      }
	    }
	  }
	}
	else
	  stat = BuildDriveThroughRoadStop(id);

	if(stat) {
	  SetStationInfo(c_tile);
	  c_built = true;
	  c_refcnt = 1;
	}

	return stat;
}
	function cATTRoadStop::BuildDriveThroughRoadStop(id=AIStation.STATION_NEW)
//	--------------------------------------------------------------------------
//	Can be used to test build thru stop
{
	local	reversed = false;
	local	gHandleTraffic = null;

	while (!AIRoad.BuildDriveThroughRoadStation(c_tile,c_front,
						    s_vehtype[c_vehtype],id)) {
	  switch (AIError.GetLastError()) {
	  case AIError.ERR_VEHICLE_IN_THE_WAY:
	    if(!gHandleTraffic) gHandleTraffic = HandleTraffic();
	    if(resume gHandleTraffic == YATT_E_TRAFFIC) return false;
	    break;

	  case AIError.ERR_NOT_ENOUGH_CASH:
	    WaitForFunds();
	    break;

	  case AIRoad.ERR_ROAD_DRIVE_THROUGH_WRONG_DIRECTION:
	    if(reversed) return false;

	    local tile = c_front;		// Try other way

	    c_front = c_back;
	    c_back = tile;
	    reversed = true;
	    break;

	  default:
	    return false;
	  }
	}

	return true;
}
	function cATTRoadStop::GetBuildCost()
//	-------------------------------------
//	Return cost of building a station, plus cost of clearing if needed
{
	local	cost = AIRoad.GetBuildCost(AIRoad.ROADTYPE_ROAD,
				(c_vehtype == YATT_VT_TRUCK) ? AIRoad.BT_TRUCK_STOP :
							       AIRoad.BT_BUS_STOP);

	if(!c_thrustop && !AITile.IsBuildable(c_tile))
	  cost += AITile.GetBuildCost(AITile.BT_CLEAR_FIELDS);

	return cost;
}
	function cATTRoadStop::DemolishRoadStop()
//	-----------------------------------------
{
	if(!c_built) return true;			// Never built

	if(c_refcnt > 1) {				// More than 1 rte
	  c_refcnt--;					// Decr reference count
	  return true;
	}

	while (!AIRoad.RemoveRoadStation(c_tile)) {
	  switch (AIError.GetLastError()) {
	  case AIError.ERR_VEHICLE_IN_THE_WAY:
	    Sleep(10);
	    break;

	  case AIError.ERR_NOT_ENOUGH_CASH:
	    WaitForFunds();
	    break;

	  default:
	    Error("Error removing road stop: "+AIError.GetLastErrorString());
	    return false;
	  }
	}

	c_id = null;
	c_built = false;
	c_refcnt = 0;

	return true;
}
	function cATTRoadStop::SetThroughStop()
//	---------------------------------------
//	Modify stop to be built to a through stop
{
	c_thrustop = true;
	c_back = c_tile - (c_front - c_tile);
}
