require("utilities.nut");
require("cATTList.nut");
require("cATTBase.nut");
require("cATTBase_map.nut");
require("cATTBase_veh.nut");
require("cATTTaskList.nut");
require("cATTAStar.nut");
require("cATTRoadPF.nut");
require("cATTRoad.nut");
require("cATTStation.nut");
require("cATTRoadStop.nut");
require("cATTDepot.nut");
require("cATTRoadDepot.nut");
require("cATTRoute.nut");
require("cATTRoadRoute.nut");
require("cATTManager.nut");
require("cATTRoadMgr.nut");
require("cATTAirport.nut");
require("cATTAirRoute.nut");
require("cATTAirMgr.nut");
require("cATT.nut");
require("cATT_events.nut");
require("cATT_findservice.nut");
require("cATT_route.nut");
require("cATT_vehicles.nut");

	class YATT extends AIController
//	-------------------------------
{
	static	YATT_DT_TABLE		= 1;	// Save descriptor types
	static	YATT_DT_ARRAY		= 2;
	static	YATT_DT_INSTANCE	= 3;
	static	YATT_DT_FLOAT		= 4;

	static	s_class = {	cATT		= cATT,
				cATTAirMgr	= cATTAirMgr,
				cATTList	= cATTList,
				cATTRoad	= cATTRoad,
				cATTRoadDepot	= cATTRoadDepot,
				cATTRoadMgr	= cATTRoadMgr,
				cATTRoadPF	= cATTRoadPF,
				cATTRoadRoute	= cATTRoadRoute,
				cATTRoadStop	= cATTRoadStop,
				cATTTaskList	= cATTTaskList,
				cATTTileList	= cATTTileList,
				};

	c_att = null;
	c_addr = null;				// Addr : data xref
	c_data = null;				// Saved data

	constructor() {
	  c_att = cATT();
	  c_addr = null;
	  c_data = null;
	}
}
	function YATT::SetCompanyName()
//	-------------------------------
{
	if(!AICompany.SetName("YATT")) {
	  local i = 2;

	  while(!AICompany.SetName("YATT #" + i)) {
	    i = i + 1;
	    if(i > 255) break;
	  }
	}

	AICompany.SetPresidentName("A. President");
}
	function YATT::Start()
//	----------------------
{
	SetCompanyName();

	c_att.Main();
}
	function YATT::Stop()
//	---------------------
{
}
	function YATT::Save()
//	---------------------
{
	AILog.Warning("Saving YATT game data");

	c_data = {};

	local	t = {version = "1.0"};

	t.c_att <- GetValue(c_att);
	t.s_g <- GetValue(c_att.s_g);
	t.s_cdist <- GetValue(c_att.s_cdist);

	t.c_data <- c_data;				// Referenced data

	AILog.Warning("Save done");

	c_data = null;

	return t;
}
	function YATT::Load(version,t)
//	------------------------------
{
	AILog.Warning("Loading YATT game data version "+version);

	c_addr = {};				// Dereferenced descriptors
	c_data = t.c_data;			// Data referenced by descriptors

	c_att = SetValue(t.c_att);
	MergeTables(c_att.s_g,SetValue(t.s_g));
	MergeTables(c_att.s_cdist,SetValue(t.s_cdist));

	c_att.c_loaded = true;			// Loaded from saved game

	AILog.Warning("Load done");

	c_addr = null;
	c_data = null;
}
	function YATT::GetValue(val)
//	----------------------------
{
	switch(typeof val) {
	case "integer":
	case "string":
	case "bool":
	  return val;

	case "float":		// OTTD does not save floats (why?!)
	  return {d_type=YATT_DT_FLOAT, d_obj=val.tostring()};

	case "table":
	  return GetTableFields(val);
	  break;

	case "array":
	  return GetArrayFields(val);
	  break;

	case "instance":
	  return GetInstanceFields(val);
	  break;

	case "null":
	case "function":
	case "class":
	case "generator":
	  return null;

	default:
	  AILog.Error("Unknown type: "+typeof val);
	}
}
	function YATT::GetTableFields(tab)
//	----------------------------------
{
	local	addr = tab.tostring();				// Table address
	local	ret = {d_type=YATT_DT_TABLE, d_addr=addr};

	if(addr in c_data) return ret;

	local	t = {};

	c_data[addr] <- {d_type=YATT_DT_TABLE, d_obj=t};	// Table saved

	foreach (name,val in tab)
	  t[name] <- GetValue(tab[name]);

	return ret;
}
	function YATT::GetArrayFields(arr)
//	----------------------------------
{
	local	addr = arr.tostring();				// Array address
	local	ret = {d_type=YATT_DT_ARRAY, d_addr=addr};

	if(addr in c_data) return ret;

	local	a = [];

	c_data[addr] <- {d_type=YATT_DT_ARRAY, d_obj=a};	// Array saved

	foreach (i,val in arr)
	  a.push(GetValue(arr[i]));

	return ret;
}
	function YATT::GetInstanceFields(inst)
//	--------------------------------------
//	Returns a table with only the c_* fields of the instance
//	If an instance should not be saved, a c_classname field should still
//	exist with an '!' in the first character position
{
	if(!("c_classname" in inst)) {
	  AILog.Error("c_classname missing from instance: "+inst.tostring());

	  return null;
	}

	if(inst.c_classname.slice(0,1) == "!") return null;	// Do not save

	local	addr = inst.tostring();				// Instance address
	local	ret = {d_type=YATT_DT_INSTANCE, d_addr=addr};

	if(addr in c_data) return ret;

	local	islist = (inst.c_classname == "cATTList");
	local	t = {};

	c_data[addr] <- {d_type=YATT_DT_INSTANCE, d_obj=t};	// Instance saved

//	For lists, preserve class name and data

	if(islist) {
	  t.c_classname <- inst.c_classname;

	  for (local item=inst.Begin(); !inst.IsEnd(); item=inst.Next())
	    t[item] <- inst.GetValue(item);
	}
	else {
	  foreach (name,val in inst.getclass()) {
	    if(name.slice(0,2) != "c_") continue;	// Not class variable

	    t[name] <- GetValue(inst[name]);		// Type passed this way
	  }
	}

	return ret;
}
	function YATT::SetValue(val)
//	----------------------------
//	There should be no arrays nor instances directly in any field
//	They are saved with a descriptor to allow referencing
{
	switch(typeof val) {
	case "integer":
	case "string":
	case "bool":
	  return val;

	case "table":
	  return ProcessDescriptor(val);
	  break;

	case "null":
	case "class":
	  return null;

	default:
	  AILog.Error("Unknown type: "+typeof val);
	}
}
	function YATT::ProcessDescriptor(tab)
//	-------------------------------------
//	Fixup references to actual objects
{
	if(tab.d_type == YATT_DT_FLOAT) return tab.d_obj.tofloat();

	if(tab.d_addr in c_addr) return c_addr[tab.d_addr];	// Actual data

	local obj;

	switch (tab.d_type) {
	case YATT_DT_TABLE:
	  obj = SetTableFields(c_data[tab.d_addr].d_obj);
	  break;

	case YATT_DT_ARRAY:
	  obj = SetArrayFields(c_data[tab.d_addr].d_obj);
	  break;

	case YATT_DT_INSTANCE:
	  obj = SetInstanceFields(c_data[tab.d_addr].d_obj);
	  break;
	}

	c_addr[tab.d_addr] <- obj;

	return obj;
}
	function YATT::SetTableFields(tab)
//	----------------------------------
{
	local	t = {};

	foreach (name,val in tab)
	  t[name] <- SetValue(tab[name]);

	return t;
}
	function YATT::SetArrayFields(arr)
//	----------------------------------
{
	local	a = [];

	foreach (i,val in arr)
	  a.push(SetValue(arr[i]));

	return a;
}
	function YATT::SetInstanceFields(t)
//	-----------------------------------
//	Create an instance and restore the data
{
	if(!(t.c_classname in s_class)) {
	  AILog.Error("Missing class name: "+t.c_classname);
	  return null;
	}

	if(t.c_classname == "cATTList") return SetATTListFields(t);

	local	obj = s_class[t.c_classname].instance();

	foreach (name,val in t)
	  obj[name] = SetValue(t[name]);		// Type passed this way

	return obj;
}
	function YATT::SetATTListFields(t)
//	----------------------------------
//	AIList needs special processing
{
	local	obj = cATTList();

	foreach (item,val in t)
	  if(item != "c_classname") obj.AddItem(item,val);

	return obj;
}
	function YATT::MergeTables(dst,src)
//	-----------------------------------
//	Merge the fields in table src into table dst
{
	foreach (name,val in src)
	  dst[name] <- src[name];
}
