	function GetCargoID(effect)
//	---------------------------
//	Get the cargo id of a cargo that has a specific town effect
//	effect		AICargo.TE_ code
//	Returns		a valid CargoID
{
	local	list = AICargoList();

	list.Valuate(AICargo.GetTownEffect);
	list.KeepValue(effect);

	return (list.IsEmpty()) ? null : list.Begin();
}
	function ListCargo()
//	--------------------
{
	local	crglist = AICargoList();

	crglist.Sort(AIList.SORT_BY_ITEM,true);

	foreach (cargo,v in crglist)
	  AILog.Info("  Cargo "+((cargo < 10) ? " " : "")+
			cargo+": "+AICargo.GetCargoLabel(cargo));
}
	function GetTwoWayCargo()
//	-------------------------
//	Return an array with element set to 1 if cargo can be taken both ways
{
	local	crglist = AICargoList();

	crglist.Sort(AIList.SORT_BY_ITEM,true);

	local	indlist = AIIndustryTypeList();

	foreach (indtype,v in indlist) {
	  local prod = AIIndustryType.GetProducedCargo(indtype);
	  local acc  = AIIndustryType.GetAcceptedCargo(indtype);

	  foreach (cargo,v in prod)
	    crglist.SetValue(cargo,acc.HasItem(cargo) ? 1 : 0);
	}

	local	arr = [],i=0;

	foreach (cargo,v in crglist) {
	  while (cargo > i) {			// Missing cargo id
	    arr.push(null);
	    i++;
	  }

	  arr.push(v);
	  i++;					// Array element count
	}

	arr[GetCargoID(AICargo.TE_PASSENGERS)] = 1;
	arr[GetCargoID(AICargo.TE_MAIL)] = 1;

	return arr;
}
	function CanDemolishTile(tile)
//	------------------------------
{
	local test = AITestMode();
	local success = AITile.DemolishTile(tile);

	test = null;
	return success;
}
	function GetYATTSettings()
//	--------------------------
//	Return table of YATT settings
{
	local	t = {};
	local	names = ["rail","road","air","ship",
			 "subsidies","pax","freight",
			 "efficiency","looptime"];

	foreach (v in names)
	  t[v] <- AIController.GetSetting(v);

	return t;
}
	function GetMinArrValue(arr)
//	----------------------------
//	Return the minimum value in the array
{
	local	minval = 0x7FFFFFFF;

	foreach (i,v in arr)
	  if(v != null && v < minval) minval = v;

	return minval;
}
	function GetMaxArrValue(arr)
//	----------------------------
//	Return the maximum value in the array
{
	local	maxval = 0x80000000;

	foreach (i,v in arr)
	  if(v != null && v > maxval) maxval = v;

	return maxval;
}
	function ShowIDs()
//	------------------
{
	local list = AITownList();
	list.Valuate(AITown.GetLocation);

	foreach (id,loc in list)
	  AISign.BuildSign(loc,id);

	local list = AIIndustryList();
	list.Valuate(AIIndustry.GetLocation);

	foreach (id,loc in list)
	  AISign.BuildSign(loc,"i"+id);
}
	function MarkPath(path)
//	-----------------------
{
	if(typeof path == "array") {
	  foreach (i,p in path) {
	    local str = i + ":  " + p.cost;
	    AISign.BuildSign(p.tile,str);
	  }
	}
	else {
	  while (path) {
	    local str = path.length + ":  " + path.cost;
	    AISign.BuildSign(path.tile,str);

	    path = path.prev;
	  }
	}
}
	function MarkList(list,label=null)
//	----------------------------------
{
	foreach (tile,value in list)
	  AISign.BuildSign(tile,(label) ? label : value);
}
	function BuildSign(tile,text)
//	-----------------------------
{
	local	exec = AIExecMode();

	AISign.BuildSign(tile,text);
}
	function ClearSigns(all=false)
//	------------------------------
{
	local	signs = AISignList();

	for (local sign=signs.Begin(); !signs.IsEnd(); sign=signs.Next())
	  if(all || AISign.GetName(sign)[0] == 'p') AISign.RemoveSign(sign);
}
	function GetListAverage(list)
//	-----------------------------
//	Find average of list values
{
	local	sum = 0;

	foreach (tile,v in list)
	  sum += v;

	return (sum.tofloat() / list.Count() + 0.5).tointeger();
}
	function GetRectTileIndex(tile,xlen,ylen)
//	-----------------------------------------
//	Return the tile that the specified distance away from a tile
//	tile	Tile to calculate from
//	xlen	Distance in the X direction
//	ylen	Distance in the Y direction
{
	return tile + AIMap.GetTileIndex(xlen-1,ylen-1);
}
	function GetTileCornerHeights(tile,wid,hig)
//	-------------------------------------------
//	tile		Tile at North corner
//	wid		Length along X axis
//	hig		Length along Y axis
//	Returns		List of tiles with North corner heights in rectangle
{
	local	list = AITileList();

//	Add tiles of rectangle plus an extra row and column

	list.AddRectangle(tile,tile+AIMap.GetTileIndex(wid,hig));
	list.Valuate(AITile.GetCornerHeight,AITile.CORNER_N);

	return list;
}
	function IsTownRatingLow(townid)
//	--------------------------------
//	Returns true if town will not allow building
{
	local	rating = AITown.GetRating(townid,AICompany.COMPANY_SELF);

	return (rating == AITown.TOWN_RATING_APPALLING ||
		rating == AITown.TOWN_RATING_VERY_POOR);
}
	function ListSettings()
//	-----------------------
//	List important game settings
{
	local	kwords = [
			"!Difficulty",
				"vehicle_breakdowns",
				"terrain_type",
				"quantity_sea_lakes",
			"!Construction",
				"build_on_slopes",
				"autoslope",
				"dist_local_authority",
			"!Station",
				"join_stations",
				"station_spread",
				"adjacent_stations",
				"distant_join_stations",
				"never_expire_airports",
				"station_noise_level",
			"!Vehicle",
				"never_expire_vehicles",
				"max_trains",
				"max_roadveh",
				"max_aircraft",
				"max_ships",
				"plane_speed",
				"plane_crashes",
			"!AI",
				"ai_disable_veh_train",
				"ai_disable_veh_roadveh",
				"ai_disable_veh_aircraft",
				"ai_disable_veh_ship",
			];

	foreach (i,str in kwords) {
	  if(str[0] == '!') {
	    AILog.Info("");
	    AILog.Info(str.slice(1));
	    AILog.Info("");
	  }
	  else if(AIGameSettings.IsValid(str))
	    AILog.Info(str+": "+AIGameSettings.GetValue(str));
	  else
	    AILog.Warning("Invalid game setting: "+str);
	}

	AILog.Info("");
}
	function AddKeyValue(t,key,value)
//	---------------------------------
//	Add a value to a key in a table, values are stored in an array
//	t		Table to use
//	key		Key in table
//	value		Value to add to array associated with key
{
	if(key in t)					// Key already exists
	  t[key].push(value);
	else
	  t[key] <- [value];
}
	function RemoveKeyValue(t,key,value)
//	------------------------------------
//	Remove a value from a key in table
//	Returns		true if value was removed
{
	if(key in t) {
	  foreach (i,v in t[key]) {
	    if(v == value) {
	      t[key].remove(i);

	      if(!t[key].len()) delete t[key];

	      return true;
	    }
	  }
	}

	return false;
}
